#line 1 "ad_riscv.cpp"
//
// Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
// Copyright (c) 2014, 2020, Red Hat Inc. All rights reserved.
// Copyright (c) 2020, 2022, Huawei Technologies Co., Ltd. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_riscv.hpp"
#include "adfiles/ad_riscv.hpp"
#include "memory/allocation.inline.hpp"
#include "asm/macroAssembler.inline.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/markOop.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"

//SourceForm

#line 755 "src/hotspot/cpu/riscv/riscv.ad"


// Derived RegMask with conditionally allocatable registers

RegMask _ANY_REG32_mask;
RegMask _ANY_REG_mask;
RegMask _PTR_REG_mask;
RegMask _NO_SPECIAL_REG32_mask;
RegMask _NO_SPECIAL_REG_mask;
RegMask _NO_SPECIAL_PTR_REG_mask;

void reg_mask_init() {

  _ANY_REG32_mask = _ALL_REG32_mask;
  _ANY_REG32_mask.Remove(OptoReg::as_OptoReg(x0->as_VMReg()));

  _ANY_REG_mask = _ALL_REG_mask;
  _ANY_REG_mask.SUBTRACT(_ZR_REG_mask);

  _PTR_REG_mask = _ALL_REG_mask;
  _PTR_REG_mask.SUBTRACT(_ZR_REG_mask);

  _NO_SPECIAL_REG32_mask = _ALL_REG32_mask;
  _NO_SPECIAL_REG32_mask.SUBTRACT(_NON_ALLOCATABLE_REG32_mask);

  _NO_SPECIAL_REG_mask = _ALL_REG_mask;
  _NO_SPECIAL_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

  _NO_SPECIAL_PTR_REG_mask = _ALL_REG_mask;
  _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_NON_ALLOCATABLE_REG_mask);

  // x27 is not allocatable when compressed oops is on
  if (UseCompressedOops) {
    _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(x27->as_VMReg()));
    _NO_SPECIAL_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
    _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_HEAPBASE_REG_mask);
  }

  // x8 is not allocatable when PreserveFramePointer is on
  if (PreserveFramePointer) {
    _NO_SPECIAL_REG32_mask.Remove(OptoReg::as_OptoReg(x8->as_VMReg()));
    _NO_SPECIAL_REG_mask.SUBTRACT(_FP_REG_mask);
    _NO_SPECIAL_PTR_REG_mask.SUBTRACT(_FP_REG_mask);
  }
}

// is_CAS(int opcode, bool maybe_volatile)
//
// return true if opcode is one of the possible CompareAndSwapX
// values otherwise false.
bool is_CAS(int opcode, bool maybe_volatile)
{
  switch (opcode) {
    // We handle these
    case Op_CompareAndSwapI:
    case Op_CompareAndSwapL:
    case Op_CompareAndSwapP:
    case Op_CompareAndSwapN:
#if INCLUDE_SHENANDOAHGC
    case Op_ShenandoahCompareAndSwapP:
    case Op_ShenandoahCompareAndSwapN:
#endif
    case Op_CompareAndSwapB:
    case Op_CompareAndSwapS:
    case Op_GetAndSetI:
    case Op_GetAndSetL:
    case Op_GetAndSetP:
    case Op_GetAndSetN:
    case Op_GetAndAddI:
    case Op_GetAndAddL:
      return true;
    case Op_CompareAndExchangeI:
    case Op_CompareAndExchangeN:
    case Op_CompareAndExchangeB:
    case Op_CompareAndExchangeS:
    case Op_CompareAndExchangeL:
    case Op_CompareAndExchangeP:
    case Op_WeakCompareAndSwapB:
    case Op_WeakCompareAndSwapS:
    case Op_WeakCompareAndSwapI:
    case Op_WeakCompareAndSwapL:
    case Op_WeakCompareAndSwapP:
    case Op_WeakCompareAndSwapN:
      return maybe_volatile;
    default:
      return false;
  }
}

// predicate controlling translation of CAS
//
// returns true if CAS needs to use an acquiring load otherwise false
bool needs_acquiring_load_reserved(const Node *n)
{
  assert(n != NULL && is_CAS(n->Opcode(), true), "expecting a compare and swap");

  LoadStoreNode* ldst = n->as_LoadStore();
  if (n != NULL && is_CAS(n->Opcode(), false)) {
    assert(ldst != NULL && ldst->trailing_membar() != NULL, "expected trailing membar");
  } else {
    return ldst != NULL && ldst->trailing_membar() != NULL;
  }
  // so we can just return true here
  return true;
}

// predicate controlling translation of StoreCM
//
// returns true if a StoreStore must precede the card write otherwise
// false

bool unnecessary_storestore(const Node *storecm)
{
  assert(storecm->Opcode()  == Op_StoreCM, "expecting a StoreCM");

  // we need to generate a dmb ishst between an object put and the
  // associated card mark when we are using CMS without conditional
  // card marking

  if (UseConcMarkSweepGC && !UseCondCardMark) {
    return false;
  }

  // a storestore is unnecesary in all other cases

  return true;
}

#define __ _masm.

// advance declarations for helper functions to convert register
// indices to register objects

// the ad file has to provide implementations of certain methods
// expected by the generic code
//
// REQUIRED FUNCTIONALITY

//=============================================================================

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.

int MachCallStaticJavaNode::ret_addr_offset()
{
  // jal
  return 1 * NativeInstruction::instruction_size;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 7 * NativeInstruction::instruction_size; // movptr, jal
}

int MachCallRuntimeNode::ret_addr_offset() {
  // for generated stubs the call will be
  //   jal(addr)
  // or with far branches
  //   jal(trampoline_stub)
  // for real runtime callouts it will be 11 instructions
  // see riscv_enc_java_to_runtime
  //   la(t1, retaddr)                ->  auipc + addi
  //   la(t0, RuntimeAddress(addr))   ->  lui + addi + slli + addi + slli + addi
  //   addi(sp, sp, -2 * wordSize)    ->  addi
  //   sd(t1, Address(sp, wordSize))  ->  sd
  //   jalr(t0)                       ->  jalr
  CodeBlob *cb = CodeCache::find_blob(_entry_point);
  if (cb != NULL) {
    return 1 * NativeInstruction::instruction_size;
  } else {
    return 11 * NativeInstruction::instruction_size;
  }
}

//
// Compute padding required for nodes which need alignment
//

// With RVC a call instruction may get 2-byte aligned.
// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const
{
  // to make sure the address of jal 4-byte aligned.
  return align_up(current_offset, alignment_required()) - current_offset;
}

// With RVC a call instruction may get 2-byte aligned.
// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const
{
  // skip the movptr in MacroAssembler::ic_call():
  // lui + addi + slli + addi + slli + addi
  // Though movptr() has already 4-byte aligned with or without RVC,
  // We need to prevent from further changes by explicitly calculating the size.
  const int movptr_size = 6 * NativeInstruction::instruction_size;
  current_offset += movptr_size;
  // to make sure the address of jal 4-byte aligned.
  return align_up(current_offset, alignment_required()) - current_offset;
}

// Indicate if the safepoint node needs the polling page as an input

// the shared code plants the oop data at the start of the generated
// code for the safepoint node and that needs ot be at the load
// instruction itself. so we cannot plant a mov of the safepoint poll
// address followed by a load. setting this to true means the mov is
// scheduled as a prior instruction. that's better for scheduling
// anyway.

bool SafePointNode::needs_polling_address_input()
{
  return true;
}

//=============================================================================

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL);
  st->print("BREAKPOINT");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);
  Assembler::CompressibleRegion cr(&_masm);
  __ ebreak();
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm); // nops shall be 2-byte under RVC for alignment purposes.
    for (int i = 0; i < _count; i++) {
      __ nop();
    }
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count * (UseRVC ? NativeInstruction::compressed_instruction_size : NativeInstruction::instruction_size);
  }

//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int Compile::ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  assert_cond(st != NULL);
  st->print("-- \t// MachConstantBaseNode (empty encoding)");
}
#endif

#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL && ra_ != NULL);
  Compile* C = ra_->C;

  int framesize = C->frame_slots() << LogBytesPerInt;

  if (C->need_stack_bang(framesize)) {
    st->print("# stack bang size=%d\n\t", framesize);
  }

  st->print("sd  fp, [sp, #%d]\n\t", - 2 * wordSize);
  st->print("sd  ra, [sp, #%d]\n\t", - wordSize);
  if (PreserveFramePointer) { st->print("sub  fp, sp, #%d\n\t", 2 * wordSize); }
  st->print("sub sp, sp, #%d\n\t", framesize);
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  // n.b. frame size includes space for return pc and fp
  const int framesize = C->frame_size_in_bytes();

  // insert a nop at the start of the prolog so we can patch in a
  // branch if we need to invalidate the method later
  __ nop();

  assert_cond(C != NULL);

  int bangsize = C->bang_size_in_bytes();
  if (C->need_stack_bang(bangsize)) {
    __ generate_stack_overflow_check(bangsize);
  }

  __ build_frame(framesize);

  if (VerifyStackAtCalls) {
    Unimplemented();
  }

  C->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    Compile::ConstantTable& constant_table = C->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  assert_cond(ra_ != NULL);
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0;
}

//=============================================================================

#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(st != NULL && ra_ != NULL);
  Compile* C = ra_->C;
  assert_cond(C != NULL);
  int framesize = C->frame_size_in_bytes();

  st->print("# pop frame %d\n\t", framesize);

  if (framesize == 0) {
    st->print("ld  ra, [sp,#%d]\n\t", (2 * wordSize));
    st->print("ld  fp, [sp,#%d]\n\t", (3 * wordSize));
    st->print("add sp, sp, #%d\n\t", (2 * wordSize));
  } else {
    st->print("add  sp, sp, #%d\n\t", framesize);
    st->print("ld  ra, [sp,#%d]\n\t", - 2 * wordSize);
    st->print("ld  fp, [sp,#%d]\n\t", - wordSize);
  }

  if (do_polling() && C->is_method_compilation()) {
    st->print("# touch polling page\n\t");
    st->print("li  t0, #0x%lx\n\t", p2i(os::get_polling_page()));
    st->print("ld  zr, [t0]");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);
  assert_cond(C != NULL);
  int framesize = C->frame_size_in_bytes();

  __ remove_frame(framesize);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    __ read_polling_page(t0, os::get_polling_page(), relocInfo::poll_return_type);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  assert_cond(ra_ != NULL);
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 for polling page.
}
const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

// This method seems to be obsolete. It is declared in machnode.hpp
// and defined in all *.ad files, but it is never called. Should we
// get rid of it?
int MachEpilogNode::safepoint_offset() const {
  assert(do_polling(), "no return for this epilog node");
  return 4;
}

//=============================================================================

// Figure out which register class each belongs in: rc_int, rc_float or
// rc_stack.
enum RC { rc_bad, rc_int, rc_float, rc_stack };

static enum RC rc_class(OptoReg::Name reg) {

  if (reg == OptoReg::Bad) {
    return rc_bad;
  }

  // we have 30 int registers * 2 halves
  // (t0 and t1 are omitted)
  int slots_of_int_registers = RegisterImpl::max_slots_per_register * (RegisterImpl::number_of_registers - 2);
  if (reg < slots_of_int_registers) {
    return rc_int;
  }

  // we have 32 float register * 2 halves
  int slots_of_float_registers = FloatRegisterImpl::max_slots_per_register * FloatRegisterImpl::number_of_registers;
  if (reg < slots_of_int_registers + slots_of_float_registers) {
    return rc_float;
  }

  // Between float regs & stack is the flags regs.
  assert(OptoReg::is_stack(reg), "blow up if spilling flags");

  return rc_stack;
}

uint MachSpillCopyNode::implementation(CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream *st) const {
  assert_cond(ra_ != NULL);
  Compile* C = ra_->C;

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(this);
  OptoReg::Name dst_lo = ra_->get_reg_first(this);

  enum RC src_hi_rc = rc_class(src_hi);
  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_hi_rc = rc_class(dst_hi);
  enum RC dst_lo_rc = rc_class(dst_lo);

  assert(src_lo != OptoReg::Bad && dst_lo != OptoReg::Bad, "must move at least 1 register");

  if (src_hi != OptoReg::Bad) {
    assert((src_lo & 1) == 0 && src_lo + 1 == src_hi &&
           (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi,
           "expected aligned-adjacent pairs");
  }

  if (src_lo == dst_lo && src_hi == dst_hi) {
    return 0;            // Self copy, no move.
  }

  bool is64 = (src_lo & 1) == 0 && src_lo + 1 == src_hi &&
              (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi;
  int src_offset = ra_->reg2offset(src_lo);
  int dst_offset = ra_->reg2offset(dst_lo);

  if (cbuf != NULL) {
    MacroAssembler _masm(cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    switch (src_lo_rc) {
      case rc_int:
        if (dst_lo_rc == rc_int) {  // gpr --> gpr copy
          if (!is64 && this->ideal_reg() != Op_RegI) { // zero extended for narrow oop or klass
            __ zero_extend(as_Register(Matcher::_regEncode[dst_lo]), as_Register(Matcher::_regEncode[src_lo]), 32);
          } else {
            __ mv(as_Register(Matcher::_regEncode[dst_lo]), as_Register(Matcher::_regEncode[src_lo]));
          }
        } else if (dst_lo_rc == rc_float) { // gpr --> fpr copy
          if (is64) {
            __ fmv_d_x(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       as_Register(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_w_x(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       as_Register(Matcher::_regEncode[src_lo]));
          }
        } else {                    // gpr --> stack spill
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          __ spill(as_Register(Matcher::_regEncode[src_lo]), is64, dst_offset);
        }
        break;
      case rc_float:
        if (dst_lo_rc == rc_int) {  // fpr --> gpr copy
          if (is64) {
            __ fmv_x_d(as_Register(Matcher::_regEncode[dst_lo]),
                       as_FloatRegister(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_x_w(as_Register(Matcher::_regEncode[dst_lo]),
                       as_FloatRegister(Matcher::_regEncode[src_lo]));
          }
        } else if (dst_lo_rc == rc_float) { // fpr --> fpr copy
          if (is64) {
            __ fmv_d(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
          } else {
            __ fmv_s(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
          }
        } else {                    // fpr --> stack spill
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                   is64, dst_offset);
        }
        break;
      case rc_stack:
        if (dst_lo_rc == rc_int) {  // stack --> gpr load
          if (this->ideal_reg() == Op_RegI) {
            __ unspill(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
          } else { // // zero extended for narrow oop or klass
            __ unspillu(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
          }
        } else if (dst_lo_rc == rc_float) { // stack --> fpr load
          __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     is64, src_offset);
        } else {                    // stack --> stack copy
          assert(dst_lo_rc == rc_stack, "spill to bad register class");
          if (this->ideal_reg() == Op_RegI) {
            __ unspill(t0, is64, src_offset);
          } else { // zero extended for narrow oop or klass
            __ unspillu(t0, is64, src_offset);
          }
          __ spill(t0, is64, dst_offset);
        }
        break;
      default:
        ShouldNotReachHere();
    }
  }

  if (st != NULL) {
    st->print("spill ");
    if (src_lo_rc == rc_stack) {
      st->print("[sp, #%d] -> ", src_offset);
    } else {
      st->print("%s -> ", Matcher::regName[src_lo]);
    }
    if (dst_lo_rc == rc_stack) {
      st->print("[sp, #%d]", dst_offset);
    } else {
      st->print("%s", Matcher::regName[dst_lo]);
    }
    st->print("\t# spill size = %d", is64 ? 64 : 32);
  }

  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  if (ra_ == NULL) {
    st->print("N%d = SpillCopy(N%d)", _idx, in(1)->_idx);
  } else {
    implementation(NULL, ra_, false, st);
  }
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  assert_cond(ra_ != NULL && st != NULL);
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("add %s, sp, #%d\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);

  assert_cond(ra_ != NULL);
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg    = ra_->get_encode(this);

  if (is_imm_in_range(offset, 12, 0)) {
    __ addi(as_Register(reg), sp, offset);
  } else if (is_imm_in_range(offset, 32, 0)) {
    __ li32(t0, offset);
    __ add(as_Register(reg), sp, t0);
  } else {
    ShouldNotReachHere();
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_).
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());

  if (is_imm_in_range(offset, 12, 0)) {
    return NativeInstruction::instruction_size;
  } else {
    return 3 * NativeInstruction::instruction_size; // lui + addiw + add;
  }
}

//=============================================================================

#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  assert_cond(st != NULL);
  st->print_cr("# MachUEPNode");
  if (UseCompressedClassPointers) {
    st->print_cr("\tlwu t0, [j_rarg0, oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (Universe::narrow_klass_shift() != 0) {
      st->print_cr("\tdecode_klass_not_null t0, t0");
    }
  } else {
    st->print_cr("\tld t0, [j_rarg0, oopDesc::klass_offset_in_bytes()]\t# compressed klass");
  }
  st->print_cr("\tbeq t0, t1, ic_hit");
  st->print_cr("\tj, SharedRuntime::_ic_miss_stub\t # Inline cache check");
  st->print_cr("\tic_hit:");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  // This is the unverified entry point.
  MacroAssembler _masm(&cbuf);

  Label skip;
  __ cmp_klass(j_rarg0, t1, t0, skip);
  __ far_jump(RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  __ bind(skip);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  assert_cond(ra_ != NULL);
  return MachNode::size(ra_);
}

// REQUIRED EMIT CODE

//=============================================================================

// Emit exception handler code.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf)
{
  // la_patchable t0, #exception_blob_entry_point
  // jr (offset)t0
  // or
  // j #exception_blob_entry_point
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ far_jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf)
{
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

  __ auipc(ra, 0);
  __ far_jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));

  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;

}
// REQUIRED MATCHER CODE

//=============================================================================

const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode)) {
    return false;
  }

  switch (opcode) {
    case Op_PopCountI:
    case Op_PopCountL:
      return UsePopCountInstruction;

    case Op_CountLeadingZerosI:
    case Op_CountLeadingZerosL:
    case Op_CountTrailingZerosI:
    case Op_CountTrailingZerosL:
      return UseZbb;
  }

  return true; // Per default match rules are supported.
}

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen) {
  return false;
}

const bool Matcher::has_predicated_vectors(void) {
  return false;
}

const int Matcher::float_pressure(int default_pressure_threshold) {
  return default_pressure_threshold;
}

int Matcher::regnum_to_fpu_offset(int regnum)
{
  Unimplemented();
  return 0;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
// |---label(L1)-----|
// |-----------------|
// |-----------------|----------eq: float-------------------
// |-----------------| // far_cmpD_branch   |   cmpD_branch
// |------- ---------|    feq;              |      feq;
// |-far_cmpD_branch-|    beqz done;        |      bnez L;
// |-----------------|    j L;              |
// |-----------------|    bind(done);       |
// |-----------------|--------------------------------------
// |-----------------| // so shortBrSize = br_size - 4;
// |-----------------| // so offs = offset - shortBrSize + 4;
// |---label(L2)-----|
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  int shortBrSize = br_size - 4;
  int offs = offset - shortBrSize + 4;
  return (-4096 <= offs && offs < 4096);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  // Probably always true, even if a temp register is required.
  return true;
}

// true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  return 0;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt) / type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  return max_vector_size(bt);
}

// Vector ideal reg.
const uint Matcher::vector_ideal_reg(int len) {
  ShouldNotReachHere();
  return 0;
}

const uint Matcher::vector_shift_count_ideal_reg(int size) {
  fatal("vector shift is not supported");
  return Node::NotAMachineReg;
}

// AES support not yet implemented
const bool Matcher::pass_original_key_for_aes() {
  return false;
}

// RISC-V supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return true;
}

// false => size gets scaled to BytesPerLong, ok.
const bool Matcher::init_array_count_is_in_bytes = false;

// Use conditional move (CMOVL)
const int Matcher::long_cmove_cost() {
  // long cmoves are no more expensive than int cmoves
  return 0;
}

const int Matcher::float_cmove_cost() {
  // float cmoves are no more expensive than int cmoves
  return 0;
}

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_postalloc_expand = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

// This affects two different things:
//  - how Decode nodes are matched
//  - how ImplicitNullCheck opportunities are recognized
// If true, the matcher will try to remove all Decodes and match them
// (as operands) into nodes. NullChecks are not prepared to deal with
// Decodes by final_graph_reshaping().
// If false, final_graph_reshaping() forces the decode behind the Cmp
// for a NullCheck. The matcher matches the Decode node into a register.
// Implicit_null_check optimization moves the Decode along with the
// memory operation back up before the NullCheck.
bool Matcher::narrow_oop_use_complex_address() {
  return Universe::narrow_oop_shift() == 0;
}

bool Matcher::narrow_klass_use_complex_address() {
// TODO
// decide whether we need to set this to true
  return false;
}

bool Matcher::const_oop_prefer_decode() {
  // Prefer ConN+DecodeN over ConP in simple compressed oops mode.
  return Universe::narrow_oop_base() == NULL;
}

bool Matcher::const_klass_prefer_decode() {
  // Prefer ConNKlass+DecodeNKlass over ConP in simple compressed klass mode.
  return Universe::narrow_klass_base() == NULL;
}

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = false;

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
  Unimplemented();
}

// Advertise here if the CPU requires explicit rounding operations to
// implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = false;

// Are floats converted to double when stored to stack during
// deoptimization?
bool Matcher::float_in_double() { return false; }

// Do ints take an entire long register or just half?
// The relevant question is how the int is callee-saved:
// the whole long is written but de-opt'ing will have to extract
// the relevant 32 bits.
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  R10_num || reg == R10_H_num ||
    reg ==  R11_num || reg == R11_H_num ||
    reg ==  R12_num || reg == R12_H_num ||
    reg ==  R13_num || reg == R13_H_num ||
    reg ==  R14_num || reg == R14_H_num ||
    reg ==  R15_num || reg == R15_H_num ||
    reg ==  R16_num || reg == R16_H_num ||
    reg ==  R17_num || reg == R17_H_num ||
    reg ==  F10_num || reg == F10_H_num ||
    reg ==  F11_num || reg == F11_H_num ||
    reg ==  F12_num || reg == F12_H_num ||
    reg ==  F13_num || reg == F13_H_num ||
    reg ==  F14_num || reg == F14_H_num ||
    reg ==  F15_num || reg == F15_H_num ||
    reg ==  F16_num || reg == F16_H_num ||
    reg ==  F17_num || reg == F17_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con(jlong divisor) {
  return false;
}

RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI.
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL.
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL.
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return FP_REG_mask();
}

bool size_fits_all_mem_uses(AddPNode* addp, int shift) {
  assert_cond(addp != NULL);
  for (DUIterator_Fast imax, i = addp->fast_outs(imax); i < imax; i++) {
    Node* u = addp->fast_out(i);
    if (u != NULL && u->is_Mem()) {
      int opsize = u->as_Mem()->memory_size();
      assert(opsize > 0, "unexpected memory operand size");
      if (u->as_Mem()->memory_size() != (1 << shift)) {
        return false;
      }
    }
  }
  return true;
}

const bool Matcher::convi2l_type_required = false;

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  return clone_base_plus_offset_address(m, mstack, address_visited);
}

void Compile::reshape_address(AddPNode* addp) {
}


#line 1057 "ad_riscv.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( DEFAULT_COST == 100, "Expect (100) to equal 100");
  assert( ALU_COST == 100, "Expect (1 * DEFAULT_COST) to equal 100");
  assert( LOAD_COST == 300, "Expect (3 * DEFAULT_COST) to equal 300");
  assert( STORE_COST == 100, "Expect (1 * DEFAULT_COST) to equal 100");
  assert( XFER_COST == 300, "Expect (3 * DEFAULT_COST) to equal 300");
  assert( BRANCH_COST == 200, "Expect (2 * DEFAULT_COST) to equal 200");
  assert( IMUL_COST == 1000, "Expect (10 * DEFAULT_COST) to equal 1000");
  assert( IDIVSI_COST == 3400, "Expect (34 * DEFAULT_COST) to equal 3400");
  assert( IDIVDI_COST == 6600, "Expect (66 * DEFAULT_COST) to equal 6600");
  assert( FMUL_SINGLE_COST == 500, "Expect (5 * DEFAULT_COST) to equal 500");
  assert( FMUL_DOUBLE_COST == 700, "Expect (7 * DEFAULT_COST) to equal 700");
  assert( FDIV_COST == 2000, "Expect (20 * DEFAULT_COST) to equal 2000");
  assert( FSQRT_COST == 2500, "Expect (25 * DEFAULT_COST) to equal 2500");
  assert( VOLATILE_REF_COST == 1000, "Expect (10 * DEFAULT_COST) to equal 1000");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R7
  'C', // R7_H
  'C', // R28
  'C', // R28_H
  'C', // R29
  'C', // R29_H
  'C', // R30
  'C', // R30_H
  'C', // R31
  'C', // R31_H
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R9
  'C', // R9_H
  'C', // R18
  'C', // R18_H
  'C', // R19
  'C', // R19_H
  'C', // R20
  'C', // R20_H
  'C', // R21
  'C', // R21_H
  'C', // R22
  'C', // R22_H
  'C', // R24
  'C', // R24_H
  'C', // R25
  'C', // R25_H
  'C', // R26
  'C', // R26_H
  'N', // R23
  'N', // R23_H
  'C', // R27
  'C', // R27_H
  'N', // R4
  'N', // R4_H
  'N', // R8
  'N', // R8_H
  'N', // R0
  'N', // R0_H
  'N', // R1
  'N', // R1_H
  'N', // R2
  'N', // R2_H
  'N', // R3
  'N', // R3_H
  'C', // F0
  'C', // F0_H
  'C', // F1
  'C', // F1_H
  'C', // F2
  'C', // F2_H
  'C', // F3
  'C', // F3_H
  'C', // F4
  'C', // F4_H
  'C', // F5
  'C', // F5_H
  'C', // F6
  'C', // F6_H
  'C', // F7
  'C', // F7_H
  'C', // F28
  'C', // F28_H
  'C', // F29
  'C', // F29_H
  'C', // F30
  'C', // F30_H
  'C', // F31
  'C', // F31_H
  'C', // F10
  'C', // F10_H
  'C', // F11
  'C', // F11_H
  'C', // F12
  'C', // F12_H
  'C', // F13
  'C', // F13_H
  'C', // F14
  'C', // F14_H
  'C', // F15
  'C', // F15_H
  'C', // F16
  'C', // F16_H
  'C', // F17
  'C', // F17_H
  'C', // F8
  'C', // F8_H
  'C', // F9
  'C', // F9_H
  'C', // F18
  'C', // F18_H
  'C', // F19
  'C', // F19_H
  'C', // F20
  'C', // F20_H
  'C', // F21
  'C', // F21_H
  'C', // F22
  'C', // F22_H
  'C', // F23
  'C', // F23_H
  'C', // F24
  'C', // F24_H
  'C', // F25
  'C', // F25_H
  'C', // F26
  'C', // F26_H
  'C', // F27
  'C', // F27_H
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // R7
  'C', // R7_H
  'C', // R28
  'C', // R28_H
  'C', // R29
  'C', // R29_H
  'C', // R30
  'C', // R30_H
  'C', // R31
  'C', // R31_H
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'E', // R9
  'E', // R9_H
  'E', // R18
  'E', // R18_H
  'E', // R19
  'E', // R19_H
  'E', // R20
  'E', // R20_H
  'E', // R21
  'E', // R21_H
  'E', // R22
  'E', // R22_H
  'E', // R24
  'E', // R24_H
  'E', // R25
  'E', // R25_H
  'E', // R26
  'E', // R26_H
  'E', // R23
  'E', // R23_H
  'E', // R27
  'E', // R27_H
  'N', // R4
  'N', // R4_H
  'E', // R8
  'E', // R8_H
  'N', // R0
  'N', // R0_H
  'C', // R1
  'C', // R1_H
  'E', // R2
  'E', // R2_H
  'N', // R3
  'N', // R3_H
  'C', // F0
  'C', // F0_H
  'C', // F1
  'C', // F1_H
  'C', // F2
  'C', // F2_H
  'C', // F3
  'C', // F3_H
  'C', // F4
  'C', // F4_H
  'C', // F5
  'C', // F5_H
  'C', // F6
  'C', // F6_H
  'C', // F7
  'C', // F7_H
  'C', // F28
  'C', // F28_H
  'C', // F29
  'C', // F29_H
  'C', // F30
  'C', // F30_H
  'C', // F31
  'C', // F31_H
  'C', // F10
  'C', // F10_H
  'C', // F11
  'C', // F11_H
  'C', // F12
  'C', // F12_H
  'C', // F13
  'C', // F13_H
  'C', // F14
  'C', // F14_H
  'C', // F15
  'C', // F15_H
  'C', // F16
  'C', // F16_H
  'C', // F17
  'C', // F17_H
  'E', // F8
  'E', // F8_H
  'E', // F9
  'E', // F9_H
  'E', // F18
  'E', // F18_H
  'E', // F19
  'E', // F19_H
  'E', // F20
  'E', // F20_H
  'E', // F21
  'E', // F21_H
  'E', // F22
  'E', // F22_H
  'E', // F23
  'E', // F23_H
  'E', // F24
  'E', // F24_H
  'E', // F25
  'E', // F25_H
  'E', // F26
  'E', // F26_H
  'E', // F27
  'E', // F27_H
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegFlags // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  uimmI6_ge32_rule,
  /*   13 */  immI_le_4_rule,
  /*   14 */  immI_16_rule,
  /*   15 */  immI_24_rule,
  /*   16 */  immI_31_rule,
  /*   17 */  immI_63_rule,
  /*   18 */  immIAdd_rule,
  /*   19 */  immISub_rule,
  /*   20 */  immI5_rule,
  /*   21 */  immL5_rule,
  /*   22 */  immL_rule,
  /*   23 */  immL0_rule,
  /*   24 */  immP_rule,
  /*   25 */  immP0_rule,
  /*   26 */  immP_1_rule,
  /*   27 */  immPollPage_rule,
  /*   28 */  immByteMapBase_rule,
  /*   29 */  immI_16bits_rule,
  /*   30 */  immIpowerOf2_rule,
  /*   31 */  immL_32bits_rule,
  /*   32 */  immL_M1_rule,
  /*   33 */  immL_pc_off_rule,
  /*   34 */  immLAdd_rule,
  /*   35 */  immLSub_rule,
  /*   36 */  immN_rule,
  /*   37 */  immN0_rule,
  /*   38 */  immNKlass_rule,
  /*   39 */  immD_rule,
  /*   40 */  immD0_rule,
  /*   41 */  immF_rule,
  /*   42 */  immF0_rule,
  /*   43 */  immIOffset_rule,
  /*   44 */  immLOffset_rule,
  /*   45 */  immIScale_rule,
  /*   46 */  iRegI_rule,
  /*   47 */  iRegINoSp_rule,
  /*   48 */  iRegI_R10_rule,
  /*   49 */  iRegI_R12_rule,
  /*   50 */  iRegI_R13_rule,
  /*   51 */  iRegI_R14_rule,
  /*   52 */  iRegL_rule,
  /*   53 */  iRegLNoSp_rule,
  /*   54 */  iRegL_R28_rule,
  /*   55 */  iRegL_R29_rule,
  /*   56 */  iRegL_R30_rule,
  /*   57 */  iRegP_rule,
  /*   58 */  iRegPNoSp_rule,
  /*   59 */  iRegP_R10_rule,
  /*   60 */  iRegP_R11_rule,
  /*   61 */  iRegP_R12_rule,
  /*   62 */  iRegP_R13_rule,
  /*   63 */  iRegP_R14_rule,
  /*   64 */  iRegP_R15_rule,
  /*   65 */  iRegP_R16_rule,
  /*   66 */  iRegP_R28_rule,
  /*   67 */  iRegN_rule,
  /*   68 */  iRegNNoSp_rule,
  /*   69 */  iRegIHeapbase_rule,
  /*   70 */  iRegL_R10_rule,
  /*   71 */  fRegF_rule,
  /*   72 */  fRegD_rule,
  /*   73 */  javaThread_RegP_rule,
  /*   74 */  indirect_rule,
  /*   75 */  indOffI_rule,
  /*   76 */  indOffL_rule,
  /*   77 */  indirectN_rule,
  /*   78 */  indOffIN_rule,
  /*   79 */  indOffLN_rule,
  /*   80 */  thread_anchor_pc_rule,
  /*   81 */  stackSlotI_rule,
  /*   82 */  stackSlotF_rule,
  /*   83 */  stackSlotD_rule,
  /*   84 */  stackSlotL_rule,
  /*   85 */  iRegL2I_rule,
  /*   86 */  cmpOp_rule,
  /*   87 */  cmpOpU_rule,
  /*   88 */  cmpOpEqNe_rule,
  /*   89 */  cmpOpULtGe_rule,
  /*   90 */  cmpOpUEqNeLeGt_rule,
  /*   91 */  rFlagsReg_rule,
  /*   92 */  inline_cache_RegP_rule,
  // last operand
  /*   93 */  memory_rule,
  /*   94 */  iRegIorL2I_rule,
  /*   95 */  iRegIorL_rule,
  /*   96 */  iRegNorP_rule,
  /*   97 */  iRegILNP_rule,
  /*   98 */  iRegILNPNoSp_rule,
  /*   99 */  immIorL_rule,
  // last operand class
  /*  100 */  _DecodeN_iRegN__rule,
  /*  101 */  _LoadB_memory__rule,
  /*  102 */  _LoadUB_memory__rule,
  /*  103 */  _LoadS_memory__rule,
  /*  104 */  _LoadUS_memory__rule,
  /*  105 */  _LoadI_memory__rule,
  /*  106 */  _ConvI2L__LoadI_memory___rule,
  /*  107 */  _Binary_iRegP_iRegP_rule,
  /*  108 */  _Binary_iRegL_iRegL_rule,
  /*  109 */  _Binary_iRegI_iRegI_rule,
  /*  110 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  111 */  _Binary_iRegN_iRegN_rule,
  /*  112 */  _ConvL2I_iRegL__rule,
  /*  113 */  _ConvI2L_iRegI__rule,
  /*  114 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  115 */  _RShiftL_iRegL_immI_63_rule,
  /*  116 */  _CastP2X_iRegP__rule,
  /*  117 */  _Binary_fRegF_fRegF_rule,
  /*  118 */  _Binary_fRegD_fRegD_rule,
  /*  119 */  _NegF_fRegF__rule,
  /*  120 */  _NegD_fRegD__rule,
  /*  121 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  122 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  123 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  124 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  125 */  _ConvI2L_iRegIorL2I__rule,
  /*  126 */  _CastP2X__DecodeN_iRegN___rule,
  /*  127 */  _CmpI_iRegI_iRegI_rule,
  /*  128 */  _CmpU_iRegI_iRegI_rule,
  /*  129 */  _CmpL_iRegL_iRegL_rule,
  /*  130 */  _CmpUL_iRegL_iRegL_rule,
  /*  131 */  _CmpP_iRegP_iRegP_rule,
  /*  132 */  _CmpN_iRegN_iRegN_rule,
  /*  133 */  _CmpF_fRegF_fRegF_rule,
  /*  134 */  _CmpD_fRegD_fRegD_rule,
  /*  135 */  _CmpI_iRegI_immI0_rule,
  /*  136 */  _CmpU_iRegI_immI0_rule,
  /*  137 */  _CmpL_iRegL_immL0_rule,
  /*  138 */  _CmpUL_iRegL_immL0_rule,
  /*  139 */  _CmpP_iRegP_immP0_rule,
  /*  140 */  _CmpN_iRegN_immN0_rule,
  /*  141 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  142 */  _Binary_cmpOp__CmpI_iRegI_iRegI_rule,
  /*  143 */  _Binary_iRegINoSp_iRegI_rule,
  /*  144 */  _Binary_cmpOpU__CmpU_iRegI_iRegI_rule,
  /*  145 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  146 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  147 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  148 */  _PartialSubtypeCheck_iRegP_R14_iRegP_R10_rule,
  /*  149 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  150 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  151 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  152 */  _Binary_iRegP_R13_immI_1_rule,
  /*  153 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  154 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  155 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  156 */  _LShiftL_iRegL_immIScale_rule,
  /*  157 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  158 */  _XorI_iRegI_immI_M1_rule,
  /*  159 */  _XorL_iRegL_immL_M1_rule,
  /*  160 */  _AndI_iRegIorL2I_immIpowerOf2_rule,
  // last internally defined operand
  /*  161 */  iRegINoSp_rule,
  /*  162 */  iRegLNoSp_rule,
  /*  163 */  iRegPNoSp_rule,
  /*  164 */  iRegPNoSp_rule,
  /*  165 */  iRegPNoSp_rule,
  /*  166 */  iRegPNoSp_rule,
  /*  167 */  iRegPNoSp_rule,
  /*  168 */  iRegNNoSp_rule,
  /*  169 */  iRegNNoSp_rule,
  /*  170 */  iRegNNoSp_rule,
  /*  171 */  fRegF_rule,
  /*  172 */  fRegF_rule,
  /*  173 */  fRegD_rule,
  /*  174 */  fRegD_rule,
  /*  175 */  rFlagsReg_rule,
  /*  176 */  rFlagsReg_rule,
  /*  177 */  rFlagsReg_rule,
  /*  178 */  fRegF_rule,
  /*  179 */  fRegD_rule,
  /*  180 */  iRegINoSp_rule,
  /*  181 */  iRegLNoSp_rule,
  /*  182 */  fRegF_rule,
  /*  183 */  fRegD_rule,
  /*  184 */  iRegPNoSp_rule,
  /*  185 */  iRegLNoSp_rule,
  /*  186 */  iRegPNoSp_rule,
  /*  187 */  iRegI_rule,
  /*  188 */  iRegPNoSp_rule,
  /*  189 */  iRegINoSp_rule,
  /*  190 */  iRegINoSp_rule,
  /*  191 */  iRegINoSp_rule,
  /*  192 */  fRegF_rule,
  /*  193 */  iRegLNoSp_rule,
  /*  194 */  fRegD_rule,
  /*  195 */  iRegINoSp_rule,
  /*  196 */  fRegF_rule,
  /*  197 */  iRegLNoSp_rule,
  /*  198 */  fRegD_rule,
  /*  199 */  rFlagsReg_rule,
  /*  200 */  javaThread_RegP_rule,
  /*  201 */  rFlagsReg_rule,
  /*  202 */  rFlagsReg_rule,
  /*  203 */  iRegINoSp_rule,
  /*  204 */  iRegINoSp_rule,
  /*  205 */  iRegINoSp_rule,
  /*  206 */  iRegINoSp_rule,
  /*  207 */  iRegINoSp_rule,
  /*  208 */  iRegINoSp_rule,
  /*  209 */  iRegINoSp_rule,
  /*  210 */  iRegLNoSp_rule,
  /*  211 */  iRegINoSp_rule,
  /*  212 */  iRegLNoSp_rule,
  /*  213 */  iRegINoSp_rule,
  /*  214 */  iRegLNoSp_rule,
  /*  215 */  iRegINoSp_rule,
  /*  216 */  iRegLNoSp_rule,
  /*  217 */  iRegINoSp_rule,
  /*  218 */  iRegLNoSp_rule,
  /*  219 */  iRegINoSp_rule,
  /*  220 */  iRegLNoSp_rule,
  /*  221 */  iRegLNoSp_rule,
  /*  222 */  iRegLNoSp_rule,
  /*  223 */  iRegINoSp_rule,
  /*  224 */  iRegPNoSp_rule,
  /*  225 */  iRegNNoSp_rule,
  /*  226 */  iRegPNoSp_rule,
  /*  227 */  iRegNNoSp_rule,
  /*  228 */  fRegF_rule,
  /*  229 */  fRegD_rule,
  /*  230 */  Universe_rule,
  /*  231 */  Universe_rule,
  /*  232 */  Universe_rule,
  /*  233 */  Universe_rule,
  /*  234 */  Universe_rule,
  /*  235 */  Universe_rule,
  /*  236 */  Universe_rule,
  /*  237 */  Universe_rule,
  /*  238 */  Universe_rule,
  /*  239 */  Universe_rule,
  /*  240 */  Universe_rule,
  /*  241 */  Universe_rule,
  /*  242 */  Universe_rule,
  /*  243 */  Universe_rule,
  /*  244 */  Universe_rule,
  /*  245 */  Universe_rule,
  /*  246 */  Universe_rule,
  /*  247 */  iRegPNoSp_rule,
  /*  248 */  iRegINoSp_rule,
  /*  249 */  iRegINoSp_rule,
  /*  250 */  iRegINoSp_rule,
  /*  251 */  iRegINoSp_rule,
  /*  252 */  iRegINoSp_rule,
  /*  253 */  iRegINoSp_rule,
  /*  254 */  iRegINoSp_rule,
  /*  255 */  iRegINoSp_rule,
  /*  256 */  iRegINoSp_rule,
  /*  257 */  iRegINoSp_rule,
  /*  258 */  iRegINoSp_rule,
  /*  259 */  iRegINoSp_rule,
  /*  260 */  iRegINoSp_rule,
  /*  261 */  iRegINoSp_rule,
  /*  262 */  iRegINoSp_rule,
  /*  263 */  iRegLNoSp_rule,
  /*  264 */  iRegNNoSp_rule,
  /*  265 */  iRegPNoSp_rule,
  /*  266 */  iRegINoSp_rule,
  /*  267 */  iRegINoSp_rule,
  /*  268 */  iRegINoSp_rule,
  /*  269 */  iRegLNoSp_rule,
  /*  270 */  iRegNNoSp_rule,
  /*  271 */  iRegPNoSp_rule,
  /*  272 */  iRegINoSp_rule,
  /*  273 */  iRegINoSp_rule,
  /*  274 */  iRegINoSp_rule,
  /*  275 */  iRegINoSp_rule,
  /*  276 */  iRegINoSp_rule,
  /*  277 */  iRegINoSp_rule,
  /*  278 */  iRegINoSp_rule,
  /*  279 */  iRegINoSp_rule,
  /*  280 */  iRegINoSp_rule,
  /*  281 */  iRegINoSp_rule,
  /*  282 */  iRegINoSp_rule,
  /*  283 */  iRegINoSp_rule,
  /*  284 */  iRegINoSp_rule,
  /*  285 */  iRegLNoSp_rule,
  /*  286 */  iRegINoSp_rule,
  /*  287 */  iRegPNoSp_rule,
  /*  288 */  iRegINoSp_rule,
  /*  289 */  iRegLNoSp_rule,
  /*  290 */  iRegINoSp_rule,
  /*  291 */  iRegPNoSp_rule,
  /*  292 */  iRegLNoSp_rule,
  /*  293 */  Universe_rule,
  /*  294 */  iRegLNoSp_rule,
  /*  295 */  Universe_rule,
  /*  296 */  iRegINoSp_rule,
  /*  297 */  Universe_rule,
  /*  298 */  iRegINoSp_rule,
  /*  299 */  Universe_rule,
  /*  300 */  iRegLNoSp_rule,
  /*  301 */  Universe_rule,
  /*  302 */  iRegLNoSp_rule,
  /*  303 */  Universe_rule,
  /*  304 */  iRegINoSp_rule,
  /*  305 */  Universe_rule,
  /*  306 */  iRegINoSp_rule,
  /*  307 */  Universe_rule,
  /*  308 */  iRegINoSp_rule,
  /*  309 */  iRegINoSp_rule,
  /*  310 */  iRegINoSp_rule,
  /*  311 */  iRegPNoSp_rule,
  /*  312 */  iRegLNoSp_rule,
  /*  313 */  iRegPNoSp_rule,
  /*  314 */  iRegLNoSp_rule,
  /*  315 */  iRegLNoSp_rule,
  /*  316 */  iRegINoSp_rule,
  /*  317 */  iRegINoSp_rule,
  /*  318 */  iRegLNoSp_rule,
  /*  319 */  iRegLNoSp_rule,
  /*  320 */  iRegINoSp_rule,
  /*  321 */  iRegLNoSp_rule,
  /*  322 */  iRegINoSp_rule,
  /*  323 */  iRegLNoSp_rule,
  /*  324 */  iRegLNoSp_rule,
  /*  325 */  iRegINoSp_rule,
  /*  326 */  iRegINoSp_rule,
  /*  327 */  iRegLNoSp_rule,
  /*  328 */  iRegLNoSp_rule,
  /*  329 */  iRegINoSp_rule,
  /*  330 */  iRegLNoSp_rule,
  /*  331 */  iRegINoSp_rule,
  /*  332 */  iRegINoSp_rule,
  /*  333 */  iRegINoSp_rule,
  /*  334 */  iRegINoSp_rule,
  /*  335 */  iRegINoSp_rule,
  /*  336 */  iRegINoSp_rule,
  /*  337 */  iRegLNoSp_rule,
  /*  338 */  iRegLNoSp_rule,
  /*  339 */  iRegLNoSp_rule,
  /*  340 */  iRegLNoSp_rule,
  /*  341 */  iRegLNoSp_rule,
  /*  342 */  iRegLNoSp_rule,
  /*  343 */  iRegLNoSp_rule,
  /*  344 */  iRegINoSp_rule,
  /*  345 */  iRegLNoSp_rule,
  /*  346 */  fRegF_rule,
  /*  347 */  fRegD_rule,
  /*  348 */  fRegF_rule,
  /*  349 */  fRegD_rule,
  /*  350 */  fRegF_rule,
  /*  351 */  fRegD_rule,
  /*  352 */  fRegF_rule,
  /*  353 */  fRegD_rule,
  /*  354 */  fRegF_rule,
  /*  355 */  fRegD_rule,
  /*  356 */  fRegF_rule,
  /*  357 */  fRegF_rule,
  /*  358 */  fRegD_rule,
  /*  359 */  fRegD_rule,
  /*  360 */  fRegF_rule,
  /*  361 */  fRegF_rule,
  /*  362 */  fRegD_rule,
  /*  363 */  fRegD_rule,
  /*  364 */  fRegF_rule,
  /*  365 */  fRegF_rule,
  /*  366 */  fRegD_rule,
  /*  367 */  fRegD_rule,
  /*  368 */  fRegF_rule,
  /*  369 */  fRegD_rule,
  /*  370 */  fRegF_rule,
  /*  371 */  fRegD_rule,
  /*  372 */  iRegINoSp_rule,
  /*  373 */  iRegINoSp_rule,
  /*  374 */  iRegINoSp_rule,
  /*  375 */  iRegINoSp_rule,
  /*  376 */  iRegINoSp_rule,
  /*  377 */  iRegINoSp_rule,
  /*  378 */  iRegLNoSp_rule,
  /*  379 */  iRegLNoSp_rule,
  /*  380 */  iRegLNoSp_rule,
  /*  381 */  iRegLNoSp_rule,
  /*  382 */  iRegLNoSp_rule,
  /*  383 */  iRegLNoSp_rule,
  /*  384 */  iRegINoSp_rule,
  /*  385 */  iRegLNoSp_rule,
  /*  386 */  iRegINoSp_rule,
  /*  387 */  iRegINoSp_rule,
  /*  388 */  Universe_rule,
  /*  389 */  Universe_rule,
  /*  390 */  Universe_rule,
  /*  391 */  Universe_rule,
  /*  392 */  Universe_rule,
  /*  393 */  Universe_rule,
  /*  394 */  Universe_rule,
  /*  395 */  Universe_rule,
  /*  396 */  iRegLNoSp_rule,
  /*  397 */  iRegINoSp_rule,
  /*  398 */  iRegLNoSp_rule,
  /*  399 */  fRegD_rule,
  /*  400 */  fRegF_rule,
  /*  401 */  iRegINoSp_rule,
  /*  402 */  fRegF_rule,
  /*  403 */  iRegLNoSp_rule,
  /*  404 */  fRegF_rule,
  /*  405 */  iRegINoSp_rule,
  /*  406 */  fRegD_rule,
  /*  407 */  iRegLNoSp_rule,
  /*  408 */  fRegD_rule,
  /*  409 */  iRegINoSp_rule,
  /*  410 */  iRegINoSp_rule,
  /*  411 */  iRegNNoSp_rule,
  /*  412 */  iRegPNoSp_rule,
  /*  413 */  iRegPNoSp_rule,
  /*  414 */  iRegNNoSp_rule,
  /*  415 */  iRegPNoSp_rule,
  /*  416 */  stackSlotI_rule,
  /*  417 */  stackSlotF_rule,
  /*  418 */  stackSlotL_rule,
  /*  419 */  stackSlotD_rule,
  /*  420 */  iRegINoSp_rule,
  /*  421 */  iRegINoSp_rule,
  /*  422 */  iRegINoSp_rule,
  /*  423 */  iRegINoSp_rule,
  /*  424 */  iRegINoSp_rule,
  /*  425 */  iRegINoSp_rule,
  /*  426 */  iRegINoSp_rule,
  /*  427 */  Universe_rule,
  /*  428 */  Universe_rule,
  /*  429 */  Universe_rule,
  /*  430 */  Universe_rule,
  /*  431 */  Universe_rule,
  /*  432 */  Universe_rule,
  /*  433 */  Universe_rule,
  /*  434 */  Universe_rule,
  /*  435 */  Universe_rule,
  /*  436 */  Universe_rule,
  /*  437 */  Universe_rule,
  /*  438 */  Universe_rule,
  /*  439 */  Universe_rule,
  /*  440 */  Universe_rule,
  /*  441 */  Universe_rule,
  /*  442 */  Universe_rule,
  /*  443 */  Universe_rule,
  /*  444 */  Universe_rule,
  /*  445 */  Universe_rule,
  /*  446 */  Universe_rule,
  /*  447 */  Universe_rule,
  /*  448 */  Universe_rule,
  /*  449 */  Universe_rule,
  /*  450 */  Universe_rule,
  /*  451 */  Universe_rule,
  /*  452 */  Universe_rule,
  /*  453 */  Universe_rule,
  /*  454 */  Universe_rule,
  /*  455 */  Universe_rule,
  /*  456 */  Universe_rule,
  /*  457 */  Universe_rule,
  /*  458 */  Universe_rule,
  /*  459 */  Universe_rule,
  /*  460 */  Universe_rule,
  /*  461 */  Universe_rule,
  /*  462 */  Universe_rule,
  /*  463 */  Universe_rule,
  /*  464 */  Universe_rule,
  /*  465 */  Universe_rule,
  /*  466 */  Universe_rule,
  /*  467 */  Universe_rule,
  /*  468 */  Universe_rule,
  /*  469 */  Universe_rule,
  /*  470 */  Universe_rule,
  /*  471 */  Universe_rule,
  /*  472 */  Universe_rule,
  /*  473 */  Universe_rule,
  /*  474 */  Universe_rule,
  /*  475 */  Universe_rule,
  /*  476 */  Universe_rule,
  /*  477 */  Universe_rule,
  /*  478 */  Universe_rule,
  /*  479 */  Universe_rule,
  /*  480 */  Universe_rule,
  /*  481 */  Universe_rule,
  /*  482 */  Universe_rule,
  /*  483 */  Universe_rule,
  /*  484 */  Universe_rule,
  /*  485 */  Universe_rule,
  /*  486 */  Universe_rule,
  /*  487 */  Universe_rule,
  /*  488 */  Universe_rule,
  /*  489 */  Universe_rule,
  /*  490 */  Universe_rule,
  /*  491 */  Universe_rule,
  /*  492 */  Universe_rule,
  /*  493 */  Universe_rule,
  /*  494 */  iRegINoSp_rule,
  /*  495 */  iRegINoSp_rule,
  /*  496 */  iRegINoSp_rule,
  /*  497 */  iRegINoSp_rule,
  /*  498 */  iRegLNoSp_rule,
  /*  499 */  iRegLNoSp_rule,
  /*  500 */  iRegLNoSp_rule,
  /*  501 */  iRegLNoSp_rule,
  /*  502 */  Universe_rule,
  /*  503 */  Universe_rule,
  /*  504 */  Universe_rule,
  /*  505 */  Universe_rule,
  /*  506 */  Universe_rule,
  /*  507 */  iRegP_R15_rule,
  /*  508 */  iRegI_R10_rule,
  /*  509 */  iRegI_R10_rule,
  /*  510 */  iRegI_R10_rule,
  /*  511 */  iRegI_R10_rule,
  /*  512 */  iRegI_R10_rule,
  /*  513 */  iRegI_R10_rule,
  /*  514 */  iRegI_R10_rule,
  /*  515 */  iRegI_R10_rule,
  /*  516 */  iRegI_R10_rule,
  /*  517 */  iRegI_R10_rule,
  /*  518 */  iRegI_R10_rule,
  /*  519 */  Universe_rule,
  /*  520 */  Universe_rule,
  /*  521 */  iRegI_R10_rule,
  /*  522 */  iRegI_R10_rule,
  /*  523 */  iRegI_R10_rule,
  /*  524 */  iRegI_R10_rule,
  /*  525 */  Universe_rule,
  /*  526 */  Universe_rule,
  /*  527 */  Universe_rule,
  /*  528 */  iRegP_R10_rule,
  /*  529 */  Universe_rule,
  /*  530 */  Universe_rule,
  /*  531 */  Universe_rule,
  /*  532 */  iRegINoSp_rule,
  /*  533 */  iRegINoSp_rule,
  /*  534 */  iRegINoSp_rule,
  /*  535 */  iRegINoSp_rule,
  /*  536 */  iRegLNoSp_rule,
  /*  537 */  iRegINoSp_rule,
  /*  538 */  iRegLNoSp_rule,
  /*  539 */  iRegINoSp_rule,
  /*  540 */  iRegINoSp_rule,
  /*  541 */  iRegPNoSp_rule,
  /*  542 */  iRegPNoSp_rule,
  /*  543 */  iRegLNoSp_rule,
  /*  544 */  iRegLNoSp_rule,
  /*  545 */  iRegLNoSp_rule,
  /*  546 */  iRegLNoSp_rule,
  /*  547 */  iRegINoSp_rule,
  /*  548 */  iRegINoSp_rule,
  /*  549 */  iRegINoSp_rule,
  /*  550 */  iRegINoSp_rule,
  /*  551 */  iRegLNoSp_rule,
  /*  552 */  iRegLNoSp_rule,
  /*  553 */  iRegINoSp_rule,
  /*  554 */  iRegINoSp_rule,
  /*  555 */  iRegLNoSp_rule,
  /*  556 */  iRegLNoSp_rule,
  /*  557 */  iRegINoSp_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  iRegP_rule,
  /*   76 */  iRegP_rule,
  /*   77 */  iRegN_rule,
  /*   78 */  _DecodeN_iRegN__rule,
  /*   79 */  _DecodeN_iRegN__rule,
  /*   80 */  javaThread_RegP_rule,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  iRegL_rule,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  // last operand
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  // last operand class
  /*  100 */  iRegN_rule,
  /*  101 */  memory_rule,
  /*  102 */  memory_rule,
  /*  103 */  memory_rule,
  /*  104 */  memory_rule,
  /*  105 */  memory_rule,
  /*  106 */  _LoadI_memory__rule,
  /*  107 */  iRegP_rule,
  /*  108 */  iRegL_rule,
  /*  109 */  iRegI_rule,
  /*  110 */  iRegI_R12_rule,
  /*  111 */  iRegN_rule,
  /*  112 */  iRegL_rule,
  /*  113 */  iRegI_rule,
  /*  114 */  iRegIorL2I_rule,
  /*  115 */  iRegL_rule,
  /*  116 */  iRegP_rule,
  /*  117 */  fRegF_rule,
  /*  118 */  fRegD_rule,
  /*  119 */  fRegF_rule,
  /*  120 */  fRegD_rule,
  /*  121 */  _NegF_fRegF__rule,
  /*  122 */  fRegF_rule,
  /*  123 */  _NegD_fRegD__rule,
  /*  124 */  fRegD_rule,
  /*  125 */  iRegIorL2I_rule,
  /*  126 */  _DecodeN_iRegN__rule,
  /*  127 */  iRegI_rule,
  /*  128 */  iRegI_rule,
  /*  129 */  iRegL_rule,
  /*  130 */  iRegL_rule,
  /*  131 */  iRegP_rule,
  /*  132 */  iRegN_rule,
  /*  133 */  fRegF_rule,
  /*  134 */  fRegD_rule,
  /*  135 */  iRegI_rule,
  /*  136 */  iRegI_rule,
  /*  137 */  iRegL_rule,
  /*  138 */  iRegL_rule,
  /*  139 */  iRegP_rule,
  /*  140 */  iRegN_rule,
  /*  141 */  _DecodeN_iRegN__rule,
  /*  142 */  cmpOp_rule,
  /*  143 */  iRegINoSp_rule,
  /*  144 */  cmpOpU_rule,
  /*  145 */  cmpOp_rule,
  /*  146 */  cmpOpU_rule,
  /*  147 */  iRegLNoSp_rule,
  /*  148 */  iRegP_R14_rule,
  /*  149 */  iRegP_R11_rule,
  /*  150 */  iRegP_R13_rule,
  /*  151 */  iRegP_R13_rule,
  /*  152 */  iRegP_R13_rule,
  /*  153 */  iRegP_R11_rule,
  /*  154 */  iRegIorL2I_rule,
  /*  155 */  iRegIorL2I_rule,
  /*  156 */  iRegL_rule,
  /*  157 */  _ConvI2L_iRegI__rule,
  /*  158 */  iRegI_rule,
  /*  159 */  iRegL_rule,
  /*  160 */  iRegIorL2I_rule,
  // last internally defined operand
  /*  161 */  immI_rule,
  /*  162 */  immL_rule,
  /*  163 */  immP_rule,
  /*  164 */  immP0_rule,
  /*  165 */  immP_1_rule,
  /*  166 */  immPollPage_rule,
  /*  167 */  immByteMapBase_rule,
  /*  168 */  immN_rule,
  /*  169 */  immN0_rule,
  /*  170 */  immNKlass_rule,
  /*  171 */  immF_rule,
  /*  172 */  immF0_rule,
  /*  173 */  immD_rule,
  /*  174 */  immD0_rule,
  /*  175 */  memory_rule,
  /*  176 */  indirect_rule,
  /*  177 */  indirect_rule,
  /*  178 */  fRegF_rule,
  /*  179 */  fRegD_rule,
  /*  180 */  iRegIorL2I_rule,
  /*  181 */  iRegL_rule,
  /*  182 */  fRegF_rule,
  /*  183 */  fRegD_rule,
  /*  184 */  iRegL_rule,
  /*  185 */  iRegP_rule,
  /*  186 */  iRegPNoSp_rule,
  /*  187 */  iRegI_rule,
  /*  188 */  iRegPNoSp_rule,
  /*  189 */  iRegI_rule,
  /*  190 */  iRegP_rule,
  /*  191 */  stackSlotF_rule,
  /*  192 */  stackSlotI_rule,
  /*  193 */  stackSlotD_rule,
  /*  194 */  stackSlotL_rule,
  /*  195 */  fRegF_rule,
  /*  196 */  iRegI_rule,
  /*  197 */  fRegD_rule,
  /*  198 */  iRegL_rule,
  /*  199 */  _PartialSubtypeCheck_iRegP_R14_iRegP_R10_rule,
  /*  200 */  0,
  /*  201 */  iRegP_rule,
  /*  202 */  iRegP_rule,
  /*  203 */  iRegIorL2I_rule,
  /*  204 */  iRegL_rule,
  /*  205 */  iRegIorL2I_rule,
  /*  206 */  iRegL_rule,
  /*  207 */  iRegIorL2I_rule,
  /*  208 */  iRegL_rule,
  /*  209 */  iRegI_rule,
  /*  210 */  iRegL_rule,
  /*  211 */  memory_rule,
  /*  212 */  _LoadB_memory__rule,
  /*  213 */  memory_rule,
  /*  214 */  _LoadUB_memory__rule,
  /*  215 */  memory_rule,
  /*  216 */  _LoadS_memory__rule,
  /*  217 */  memory_rule,
  /*  218 */  _LoadUS_memory__rule,
  /*  219 */  memory_rule,
  /*  220 */  _LoadI_memory__rule,
  /*  221 */  _ConvI2L__LoadI_memory___rule,
  /*  222 */  memory_rule,
  /*  223 */  memory_rule,
  /*  224 */  memory_rule,
  /*  225 */  memory_rule,
  /*  226 */  memory_rule,
  /*  227 */  memory_rule,
  /*  228 */  memory_rule,
  /*  229 */  memory_rule,
  /*  230 */  memory_rule,
  /*  231 */  memory_rule,
  /*  232 */  memory_rule,
  /*  233 */  memory_rule,
  /*  234 */  memory_rule,
  /*  235 */  memory_rule,
  /*  236 */  memory_rule,
  /*  237 */  memory_rule,
  /*  238 */  memory_rule,
  /*  239 */  memory_rule,
  /*  240 */  memory_rule,
  /*  241 */  memory_rule,
  /*  242 */  memory_rule,
  /*  243 */  memory_rule,
  /*  244 */  memory_rule,
  /*  245 */  memory_rule,
  /*  246 */  memory_rule,
  /*  247 */  indirect_rule,
  /*  248 */  indirect_rule,
  /*  249 */  indirect_rule,
  /*  250 */  indirect_rule,
  /*  251 */  indirect_rule,
  /*  252 */  indirect_rule,
  /*  253 */  indirect_rule,
  /*  254 */  indirect_rule,
  /*  255 */  indirect_rule,
  /*  256 */  indirect_rule,
  /*  257 */  indirect_rule,
  /*  258 */  indirect_rule,
  /*  259 */  indirect_rule,
  /*  260 */  indirect_rule,
  /*  261 */  indirect_rule,
  /*  262 */  indirect_rule,
  /*  263 */  indirect_rule,
  /*  264 */  indirect_rule,
  /*  265 */  indirect_rule,
  /*  266 */  indirect_rule,
  /*  267 */  indirect_rule,
  /*  268 */  indirect_rule,
  /*  269 */  indirect_rule,
  /*  270 */  indirect_rule,
  /*  271 */  indirect_rule,
  /*  272 */  indirect_rule,
  /*  273 */  indirect_rule,
  /*  274 */  indirect_rule,
  /*  275 */  indirect_rule,
  /*  276 */  indirect_rule,
  /*  277 */  indirect_rule,
  /*  278 */  indirect_rule,
  /*  279 */  indirect_rule,
  /*  280 */  indirect_rule,
  /*  281 */  indirect_rule,
  /*  282 */  indirect_rule,
  /*  283 */  indirect_rule,
  /*  284 */  indirect_rule,
  /*  285 */  indirect_rule,
  /*  286 */  indirect_rule,
  /*  287 */  indirect_rule,
  /*  288 */  indirect_rule,
  /*  289 */  indirect_rule,
  /*  290 */  indirect_rule,
  /*  291 */  indirect_rule,
  /*  292 */  indirect_rule,
  /*  293 */  indirect_rule,
  /*  294 */  indirect_rule,
  /*  295 */  indirect_rule,
  /*  296 */  indirect_rule,
  /*  297 */  indirect_rule,
  /*  298 */  indirect_rule,
  /*  299 */  indirect_rule,
  /*  300 */  indirect_rule,
  /*  301 */  indirect_rule,
  /*  302 */  indirect_rule,
  /*  303 */  indirect_rule,
  /*  304 */  indirect_rule,
  /*  305 */  indirect_rule,
  /*  306 */  indirect_rule,
  /*  307 */  indirect_rule,
  /*  308 */  iRegIorL2I_rule,
  /*  309 */  iRegIorL2I_rule,
  /*  310 */  _ConvL2I_iRegL__rule,
  /*  311 */  iRegP_rule,
  /*  312 */  _ConvI2L_iRegI__rule,
  /*  313 */  iRegP_rule,
  /*  314 */  iRegL_rule,
  /*  315 */  iRegL_rule,
  /*  316 */  iRegIorL2I_rule,
  /*  317 */  iRegIorL2I_rule,
  /*  318 */  iRegL_rule,
  /*  319 */  iRegL_rule,
  /*  320 */  immI0_rule,
  /*  321 */  immL0_rule,
  /*  322 */  iRegIorL2I_rule,
  /*  323 */  iRegL_rule,
  /*  324 */  iRegL_rule,
  /*  325 */  iRegIorL2I_rule,
  /*  326 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  327 */  iRegL_rule,
  /*  328 */  _RShiftL_iRegL_immI_63_rule,
  /*  329 */  iRegIorL2I_rule,
  /*  330 */  iRegL_rule,
  /*  331 */  iRegIorL2I_rule,
  /*  332 */  iRegIorL2I_rule,
  /*  333 */  iRegIorL2I_rule,
  /*  334 */  iRegIorL2I_rule,
  /*  335 */  iRegIorL2I_rule,
  /*  336 */  iRegIorL2I_rule,
  /*  337 */  iRegL_rule,
  /*  338 */  iRegL_rule,
  /*  339 */  iRegL_rule,
  /*  340 */  iRegL_rule,
  /*  341 */  _CastP2X_iRegP__rule,
  /*  342 */  iRegL_rule,
  /*  343 */  iRegL_rule,
  /*  344 */  iRegI_rule,
  /*  345 */  iRegL_rule,
  /*  346 */  fRegF_rule,
  /*  347 */  fRegD_rule,
  /*  348 */  fRegF_rule,
  /*  349 */  fRegD_rule,
  /*  350 */  fRegF_rule,
  /*  351 */  fRegD_rule,
  /*  352 */  fRegF_rule,
  /*  353 */  fRegD_rule,
  /*  354 */  _NegF_fRegF__rule,
  /*  355 */  _NegD_fRegD__rule,
  /*  356 */  fRegF_rule,
  /*  357 */  fRegF_rule,
  /*  358 */  fRegD_rule,
  /*  359 */  fRegD_rule,
  /*  360 */  _NegF_fRegF__rule,
  /*  361 */  _NegF_fRegF__rule,
  /*  362 */  _NegD_fRegD__rule,
  /*  363 */  _NegD_fRegD__rule,
  /*  364 */  fRegF_rule,
  /*  365 */  fRegF_rule,
  /*  366 */  fRegD_rule,
  /*  367 */  fRegD_rule,
  /*  368 */  fRegF_rule,
  /*  369 */  fRegD_rule,
  /*  370 */  fRegF_rule,
  /*  371 */  fRegD_rule,
  /*  372 */  iRegI_rule,
  /*  373 */  iRegI_rule,
  /*  374 */  iRegI_rule,
  /*  375 */  iRegI_rule,
  /*  376 */  iRegI_rule,
  /*  377 */  iRegI_rule,
  /*  378 */  iRegL_rule,
  /*  379 */  iRegL_rule,
  /*  380 */  iRegL_rule,
  /*  381 */  iRegL_rule,
  /*  382 */  iRegL_rule,
  /*  383 */  iRegL_rule,
  /*  384 */  iRegIorL2I_rule,
  /*  385 */  iRegL_rule,
  /*  386 */  iRegIorL2I_rule,
  /*  387 */  iRegIorL2I_rule,
  /*  388 */  0,
  /*  389 */  0,
  /*  390 */  0,
  /*  391 */  0,
  /*  392 */  0,
  /*  393 */  0,
  /*  394 */  0,
  /*  395 */  0,
  /*  396 */  iRegIorL2I_rule,
  /*  397 */  iRegL_rule,
  /*  398 */  _ConvI2L_iRegIorL2I__rule,
  /*  399 */  fRegF_rule,
  /*  400 */  fRegD_rule,
  /*  401 */  fRegF_rule,
  /*  402 */  iRegIorL2I_rule,
  /*  403 */  fRegF_rule,
  /*  404 */  iRegL_rule,
  /*  405 */  fRegD_rule,
  /*  406 */  iRegIorL2I_rule,
  /*  407 */  fRegD_rule,
  /*  408 */  iRegL_rule,
  /*  409 */  _CastP2X_iRegP__rule,
  /*  410 */  _CastP2X__DecodeN_iRegN___rule,
  /*  411 */  iRegP_rule,
  /*  412 */  iRegN_rule,
  /*  413 */  iRegN_rule,
  /*  414 */  iRegP_rule,
  /*  415 */  iRegN_rule,
  /*  416 */  fRegF_rule,
  /*  417 */  iRegI_rule,
  /*  418 */  fRegD_rule,
  /*  419 */  iRegL_rule,
  /*  420 */  fRegF_rule,
  /*  421 */  fRegD_rule,
  /*  422 */  iRegL_rule,
  /*  423 */  iRegI_rule,
  /*  424 */  iRegIorL2I_rule,
  /*  425 */  iRegI_rule,
  /*  426 */  iRegI_rule,
  /*  427 */  0,
  /*  428 */  cmpOpEqNe_rule,
  /*  429 */  cmpOp_rule,
  /*  430 */  cmpOp_rule,
  /*  431 */  cmpOpU_rule,
  /*  432 */  cmpOpU_rule,
  /*  433 */  cmpOp_rule,
  /*  434 */  cmpOp_rule,
  /*  435 */  cmpOpU_rule,
  /*  436 */  cmpOpU_rule,
  /*  437 */  cmpOpU_rule,
  /*  438 */  cmpOpU_rule,
  /*  439 */  cmpOpU_rule,
  /*  440 */  cmpOpU_rule,
  /*  441 */  cmpOp_rule,
  /*  442 */  cmpOp_rule,
  /*  443 */  cmpOp_rule,
  /*  444 */  cmpOp_rule,
  /*  445 */  cmpOp_rule,
  /*  446 */  cmpOp_rule,
  /*  447 */  cmpOpUEqNeLeGt_rule,
  /*  448 */  cmpOpUEqNeLeGt_rule,
  /*  449 */  cmpOp_rule,
  /*  450 */  cmpOp_rule,
  /*  451 */  cmpOpUEqNeLeGt_rule,
  /*  452 */  cmpOpUEqNeLeGt_rule,
  /*  453 */  cmpOpEqNe_rule,
  /*  454 */  cmpOpEqNe_rule,
  /*  455 */  cmpOpEqNe_rule,
  /*  456 */  cmpOpEqNe_rule,
  /*  457 */  cmpOpEqNe_rule,
  /*  458 */  cmpOpEqNe_rule,
  /*  459 */  cmpOp_rule,
  /*  460 */  cmpOp_rule,
  /*  461 */  cmpOp_rule,
  /*  462 */  cmpOpU_rule,
  /*  463 */  cmpOpU_rule,
  /*  464 */  cmpOp_rule,
  /*  465 */  cmpOp_rule,
  /*  466 */  cmpOpU_rule,
  /*  467 */  cmpOpU_rule,
  /*  468 */  cmpOpU_rule,
  /*  469 */  cmpOpU_rule,
  /*  470 */  cmpOpU_rule,
  /*  471 */  cmpOpU_rule,
  /*  472 */  cmpOp_rule,
  /*  473 */  cmpOp_rule,
  /*  474 */  cmpOp_rule,
  /*  475 */  cmpOp_rule,
  /*  476 */  cmpOp_rule,
  /*  477 */  cmpOp_rule,
  /*  478 */  cmpOpUEqNeLeGt_rule,
  /*  479 */  cmpOpUEqNeLeGt_rule,
  /*  480 */  cmpOpULtGe_rule,
  /*  481 */  cmpOpULtGe_rule,
  /*  482 */  cmpOp_rule,
  /*  483 */  cmpOp_rule,
  /*  484 */  cmpOpUEqNeLeGt_rule,
  /*  485 */  cmpOpUEqNeLeGt_rule,
  /*  486 */  cmpOpULtGe_rule,
  /*  487 */  cmpOpULtGe_rule,
  /*  488 */  cmpOpEqNe_rule,
  /*  489 */  cmpOpEqNe_rule,
  /*  490 */  cmpOpEqNe_rule,
  /*  491 */  cmpOpEqNe_rule,
  /*  492 */  cmpOpEqNe_rule,
  /*  493 */  cmpOpEqNe_rule,
  /*  494 */  _Binary_cmpOp__CmpI_iRegI_iRegI_rule,
  /*  495 */  _Binary_cmpOpU__CmpU_iRegI_iRegI_rule,
  /*  496 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  497 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  498 */  _Binary_cmpOp__CmpL_iRegL_iRegL_rule,
  /*  499 */  _Binary_cmpOpU__CmpUL_iRegL_iRegL_rule,
  /*  500 */  _Binary_cmpOp__CmpI_iRegI_iRegI_rule,
  /*  501 */  _Binary_cmpOpU__CmpU_iRegI_iRegI_rule,
  /*  502 */  0,
  /*  503 */  0,
  /*  504 */  0,
  /*  505 */  0,
  /*  506 */  0,
  /*  507 */  iRegP_R14_rule,
  /*  508 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  509 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  510 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  511 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  512 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  513 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  514 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  515 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  516 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  517 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  518 */  _Binary_iRegP_R11_iRegI_R12_rule,
  /*  519 */  iRegL_R29_rule,
  /*  520 */  immL_rule,
  /*  521 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  522 */  _Binary_iRegP_R11_iRegP_R13_rule,
  /*  523 */  iRegP_R11_rule,
  /*  524 */  iRegP_R11_rule,
  /*  525 */  iRegP_rule,
  /*  526 */  iRegPNoSp_rule,
  /*  527 */  iRegPNoSp_rule,
  /*  528 */  0,
  /*  529 */  0,
  /*  530 */  0,
  /*  531 */  0,
  /*  532 */  _CastP2X_iRegP__rule,
  /*  533 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  534 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  535 */  iRegIorL2I_rule,
  /*  536 */  _ConvI2L_iRegIorL2I__rule,
  /*  537 */  iRegIorL2I_rule,
  /*  538 */  iRegL_rule,
  /*  539 */  iRegIorL2I_rule,
  /*  540 */  iRegIorL2I_rule,
  /*  541 */  iRegP_rule,
  /*  542 */  iRegP_rule,
  /*  543 */  iRegL_rule,
  /*  544 */  _LShiftL_iRegL_immIScale_rule,
  /*  545 */  iRegL_rule,
  /*  546 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  547 */  iRegI_rule,
  /*  548 */  iRegI_rule,
  /*  549 */  iRegI_rule,
  /*  550 */  _XorI_iRegI_immI_M1_rule,
  /*  551 */  iRegL_rule,
  /*  552 */  _XorL_iRegL_immL_M1_rule,
  /*  553 */  iRegI_rule,
  /*  554 */  _XorI_iRegI_immI_M1_rule,
  /*  555 */  iRegL_rule,
  /*  556 */  _XorL_iRegL_immL_M1_rule,
  /*  557 */  _AndI_iRegIorL2I_immIpowerOf2_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  immIOffset_rule,
  /*   76 */  immLOffset_rule,
  /*   77 */  0,
  /*   78 */  immIOffset_rule,
  /*   79 */  immLOffset_rule,
  /*   80 */  immL_pc_off_rule,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  // last operand
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  // last operand class
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  iRegP_rule,
  /*  108 */  iRegL_rule,
  /*  109 */  iRegI_rule,
  /*  110 */  iRegI_R13_rule,
  /*  111 */  iRegN_rule,
  /*  112 */  0,
  /*  113 */  0,
  /*  114 */  immI_31_rule,
  /*  115 */  immI_63_rule,
  /*  116 */  0,
  /*  117 */  fRegF_rule,
  /*  118 */  fRegD_rule,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  fRegF_rule,
  /*  122 */  _NegF_fRegF__rule,
  /*  123 */  fRegD_rule,
  /*  124 */  _NegD_fRegD__rule,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  iRegI_rule,
  /*  128 */  iRegI_rule,
  /*  129 */  iRegL_rule,
  /*  130 */  iRegL_rule,
  /*  131 */  iRegP_rule,
  /*  132 */  iRegN_rule,
  /*  133 */  fRegF_rule,
  /*  134 */  fRegD_rule,
  /*  135 */  immI0_rule,
  /*  136 */  immI0_rule,
  /*  137 */  immL0_rule,
  /*  138 */  immL0_rule,
  /*  139 */  immP0_rule,
  /*  140 */  immN0_rule,
  /*  141 */  immP0_rule,
  /*  142 */  _CmpI_iRegI_iRegI_rule,
  /*  143 */  iRegI_rule,
  /*  144 */  _CmpU_iRegI_iRegI_rule,
  /*  145 */  _CmpL_iRegL_iRegL_rule,
  /*  146 */  _CmpUL_iRegL_iRegL_rule,
  /*  147 */  iRegL_rule,
  /*  148 */  iRegP_R10_rule,
  /*  149 */  iRegI_R12_rule,
  /*  150 */  iRegI_R14_rule,
  /*  151 */  immI_le_4_rule,
  /*  152 */  immI_1_rule,
  /*  153 */  iRegP_R13_rule,
  /*  154 */  immI_24_rule,
  /*  155 */  immI_16_rule,
  /*  156 */  immIScale_rule,
  /*  157 */  immIScale_rule,
  /*  158 */  immI_M1_rule,
  /*  159 */  immL_M1_rule,
  /*  160 */  immIpowerOf2_rule,
  // last internally defined operand
  /*  161 */  0,
  /*  162 */  0,
  /*  163 */  0,
  /*  164 */  0,
  /*  165 */  0,
  /*  166 */  0,
  /*  167 */  0,
  /*  168 */  0,
  /*  169 */  0,
  /*  170 */  0,
  /*  171 */  0,
  /*  172 */  0,
  /*  173 */  0,
  /*  174 */  0,
  /*  175 */  _Binary_iRegP_iRegP_rule,
  /*  176 */  _Binary_iRegL_iRegL_rule,
  /*  177 */  _Binary_iRegI_iRegI_rule,
  /*  178 */  0,
  /*  179 */  0,
  /*  180 */  0,
  /*  181 */  0,
  /*  182 */  0,
  /*  183 */  0,
  /*  184 */  0,
  /*  185 */  0,
  /*  186 */  0,
  /*  187 */  0,
  /*  188 */  0,
  /*  189 */  0,
  /*  190 */  0,
  /*  191 */  0,
  /*  192 */  0,
  /*  193 */  0,
  /*  194 */  0,
  /*  195 */  0,
  /*  196 */  0,
  /*  197 */  0,
  /*  198 */  0,
  /*  199 */  immP0_rule,
  /*  200 */  0,
  /*  201 */  iRegP_rule,
  /*  202 */  iRegP_rule,
  /*  203 */  0,
  /*  204 */  0,
  /*  205 */  0,
  /*  206 */  0,
  /*  207 */  0,
  /*  208 */  0,
  /*  209 */  0,
  /*  210 */  0,
  /*  211 */  0,
  /*  212 */  0,
  /*  213 */  0,
  /*  214 */  0,
  /*  215 */  0,
  /*  216 */  0,
  /*  217 */  0,
  /*  218 */  0,
  /*  219 */  0,
  /*  220 */  0,
  /*  221 */  immL_32bits_rule,
  /*  222 */  0,
  /*  223 */  0,
  /*  224 */  0,
  /*  225 */  0,
  /*  226 */  0,
  /*  227 */  0,
  /*  228 */  0,
  /*  229 */  0,
  /*  230 */  immI0_rule,
  /*  231 */  immI0_rule,
  /*  232 */  iRegIorL2I_rule,
  /*  233 */  immI0_rule,
  /*  234 */  iRegIorL2I_rule,
  /*  235 */  immI0_rule,
  /*  236 */  iRegIorL2I_rule,
  /*  237 */  immI0_rule,
  /*  238 */  iRegL_rule,
  /*  239 */  immL0_rule,
  /*  240 */  iRegP_rule,
  /*  241 */  immP0_rule,
  /*  242 */  iRegN_rule,
  /*  243 */  immN0_rule,
  /*  244 */  fRegF_rule,
  /*  245 */  fRegD_rule,
  /*  246 */  iRegN_rule,
  /*  247 */  0,
  /*  248 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  249 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  250 */  _Binary_iRegI_iRegI_rule,
  /*  251 */  _Binary_iRegL_iRegL_rule,
  /*  252 */  _Binary_iRegP_iRegP_rule,
  /*  253 */  _Binary_iRegN_iRegN_rule,
  /*  254 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  255 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  256 */  _Binary_iRegI_iRegI_rule,
  /*  257 */  _Binary_iRegL_iRegL_rule,
  /*  258 */  _Binary_iRegP_iRegP_rule,
  /*  259 */  _Binary_iRegN_iRegN_rule,
  /*  260 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  261 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  262 */  _Binary_iRegI_iRegI_rule,
  /*  263 */  _Binary_iRegL_iRegL_rule,
  /*  264 */  _Binary_iRegN_iRegN_rule,
  /*  265 */  _Binary_iRegP_iRegP_rule,
  /*  266 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  267 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  268 */  _Binary_iRegI_iRegI_rule,
  /*  269 */  _Binary_iRegL_iRegL_rule,
  /*  270 */  _Binary_iRegN_iRegN_rule,
  /*  271 */  _Binary_iRegP_iRegP_rule,
  /*  272 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  273 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  274 */  _Binary_iRegI_iRegI_rule,
  /*  275 */  _Binary_iRegL_iRegL_rule,
  /*  276 */  _Binary_iRegN_iRegN_rule,
  /*  277 */  _Binary_iRegP_iRegP_rule,
  /*  278 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  279 */  _Binary_iRegI_R12_iRegI_R13_rule,
  /*  280 */  _Binary_iRegI_iRegI_rule,
  /*  281 */  _Binary_iRegL_iRegL_rule,
  /*  282 */  _Binary_iRegN_iRegN_rule,
  /*  283 */  _Binary_iRegP_iRegP_rule,
  /*  284 */  iRegI_rule,
  /*  285 */  iRegL_rule,
  /*  286 */  iRegN_rule,
  /*  287 */  iRegP_rule,
  /*  288 */  iRegI_rule,
  /*  289 */  iRegL_rule,
  /*  290 */  iRegN_rule,
  /*  291 */  iRegP_rule,
  /*  292 */  iRegL_rule,
  /*  293 */  iRegL_rule,
  /*  294 */  immLAdd_rule,
  /*  295 */  immLAdd_rule,
  /*  296 */  iRegIorL2I_rule,
  /*  297 */  iRegIorL2I_rule,
  /*  298 */  immIAdd_rule,
  /*  299 */  immIAdd_rule,
  /*  300 */  iRegL_rule,
  /*  301 */  iRegL_rule,
  /*  302 */  immLAdd_rule,
  /*  303 */  immLAdd_rule,
  /*  304 */  iRegIorL2I_rule,
  /*  305 */  iRegIorL2I_rule,
  /*  306 */  immIAdd_rule,
  /*  307 */  immIAdd_rule,
  /*  308 */  iRegIorL2I_rule,
  /*  309 */  immIAdd_rule,
  /*  310 */  immIAdd_rule,
  /*  311 */  iRegL_rule,
  /*  312 */  uimmI6_ge32_rule,
  /*  313 */  immLAdd_rule,
  /*  314 */  iRegL_rule,
  /*  315 */  immLAdd_rule,
  /*  316 */  iRegIorL2I_rule,
  /*  317 */  immISub_rule,
  /*  318 */  iRegL_rule,
  /*  319 */  immLSub_rule,
  /*  320 */  iRegIorL2I_rule,
  /*  321 */  iRegL_rule,
  /*  322 */  iRegIorL2I_rule,
  /*  323 */  iRegL_rule,
  /*  324 */  iRegL_rule,
  /*  325 */  iRegIorL2I_rule,
  /*  326 */  immI_31_rule,
  /*  327 */  iRegL_rule,
  /*  328 */  immI_63_rule,
  /*  329 */  iRegIorL2I_rule,
  /*  330 */  iRegL_rule,
  /*  331 */  iRegIorL2I_rule,
  /*  332 */  immI_rule,
  /*  333 */  iRegIorL2I_rule,
  /*  334 */  immI_rule,
  /*  335 */  iRegIorL2I_rule,
  /*  336 */  immI_rule,
  /*  337 */  iRegIorL2I_rule,
  /*  338 */  immI_rule,
  /*  339 */  iRegIorL2I_rule,
  /*  340 */  immI_rule,
  /*  341 */  immI_rule,
  /*  342 */  iRegIorL2I_rule,
  /*  343 */  immI_rule,
  /*  344 */  immI_M1_rule,
  /*  345 */  immL_M1_rule,
  /*  346 */  fRegF_rule,
  /*  347 */  fRegD_rule,
  /*  348 */  fRegF_rule,
  /*  349 */  fRegD_rule,
  /*  350 */  fRegF_rule,
  /*  351 */  fRegD_rule,
  /*  352 */  _Binary_fRegF_fRegF_rule,
  /*  353 */  _Binary_fRegD_fRegD_rule,
  /*  354 */  _Binary_fRegF_fRegF_rule,
  /*  355 */  _Binary_fRegD_fRegD_rule,
  /*  356 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  357 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  358 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  359 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  360 */  _Binary__NegF_fRegF__fRegF_rule,
  /*  361 */  _Binary_fRegF__NegF_fRegF__rule,
  /*  362 */  _Binary__NegD_fRegD__fRegD_rule,
  /*  363 */  _Binary_fRegD__NegD_fRegD__rule,
  /*  364 */  fRegF_rule,
  /*  365 */  fRegF_rule,
  /*  366 */  fRegD_rule,
  /*  367 */  fRegD_rule,
  /*  368 */  fRegF_rule,
  /*  369 */  fRegD_rule,
  /*  370 */  0,
  /*  371 */  0,
  /*  372 */  iRegI_rule,
  /*  373 */  immIAdd_rule,
  /*  374 */  iRegI_rule,
  /*  375 */  immIAdd_rule,
  /*  376 */  iRegI_rule,
  /*  377 */  immIAdd_rule,
  /*  378 */  iRegL_rule,
  /*  379 */  immLAdd_rule,
  /*  380 */  iRegL_rule,
  /*  381 */  immLAdd_rule,
  /*  382 */  iRegL_rule,
  /*  383 */  immLAdd_rule,
  /*  384 */  0,
  /*  385 */  0,
  /*  386 */  0,
  /*  387 */  0,
  /*  388 */  0,
  /*  389 */  0,
  /*  390 */  0,
  /*  391 */  0,
  /*  392 */  0,
  /*  393 */  0,
  /*  394 */  0,
  /*  395 */  0,
  /*  396 */  0,
  /*  397 */  0,
  /*  398 */  immL_32bits_rule,
  /*  399 */  0,
  /*  400 */  0,
  /*  401 */  0,
  /*  402 */  0,
  /*  403 */  0,
  /*  404 */  0,
  /*  405 */  0,
  /*  406 */  0,
  /*  407 */  0,
  /*  408 */  0,
  /*  409 */  0,
  /*  410 */  0,
  /*  411 */  0,
  /*  412 */  0,
  /*  413 */  0,
  /*  414 */  0,
  /*  415 */  0,
  /*  416 */  0,
  /*  417 */  0,
  /*  418 */  0,
  /*  419 */  0,
  /*  420 */  fRegF_rule,
  /*  421 */  fRegD_rule,
  /*  422 */  iRegL_rule,
  /*  423 */  iRegI_rule,
  /*  424 */  immI0_rule,
  /*  425 */  iRegI_rule,
  /*  426 */  iRegI_rule,
  /*  427 */  0,
  /*  428 */  rFlagsReg_rule,
  /*  429 */  _CmpI_iRegI_iRegI_rule,
  /*  430 */  _CmpI_iRegI_iRegI_rule,
  /*  431 */  _CmpU_iRegI_iRegI_rule,
  /*  432 */  _CmpU_iRegI_iRegI_rule,
  /*  433 */  _CmpL_iRegL_iRegL_rule,
  /*  434 */  _CmpL_iRegL_iRegL_rule,
  /*  435 */  _CmpUL_iRegL_iRegL_rule,
  /*  436 */  _CmpUL_iRegL_iRegL_rule,
  /*  437 */  _CmpP_iRegP_iRegP_rule,
  /*  438 */  _CmpP_iRegP_iRegP_rule,
  /*  439 */  _CmpN_iRegN_iRegN_rule,
  /*  440 */  _CmpN_iRegN_iRegN_rule,
  /*  441 */  _CmpF_fRegF_fRegF_rule,
  /*  442 */  _CmpF_fRegF_fRegF_rule,
  /*  443 */  _CmpD_fRegD_fRegD_rule,
  /*  444 */  _CmpD_fRegD_fRegD_rule,
  /*  445 */  _CmpI_iRegI_immI0_rule,
  /*  446 */  _CmpI_iRegI_immI0_rule,
  /*  447 */  _CmpU_iRegI_immI0_rule,
  /*  448 */  _CmpU_iRegI_immI0_rule,
  /*  449 */  _CmpL_iRegL_immL0_rule,
  /*  450 */  _CmpL_iRegL_immL0_rule,
  /*  451 */  _CmpUL_iRegL_immL0_rule,
  /*  452 */  _CmpUL_iRegL_immL0_rule,
  /*  453 */  _CmpP_iRegP_immP0_rule,
  /*  454 */  _CmpP_iRegP_immP0_rule,
  /*  455 */  _CmpN_iRegN_immN0_rule,
  /*  456 */  _CmpN_iRegN_immN0_rule,
  /*  457 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  458 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  459 */  rFlagsReg_rule,
  /*  460 */  _CmpI_iRegI_iRegI_rule,
  /*  461 */  _CmpI_iRegI_iRegI_rule,
  /*  462 */  _CmpU_iRegI_iRegI_rule,
  /*  463 */  _CmpU_iRegI_iRegI_rule,
  /*  464 */  _CmpL_iRegL_iRegL_rule,
  /*  465 */  _CmpL_iRegL_iRegL_rule,
  /*  466 */  _CmpUL_iRegL_iRegL_rule,
  /*  467 */  _CmpUL_iRegL_iRegL_rule,
  /*  468 */  _CmpP_iRegP_iRegP_rule,
  /*  469 */  _CmpP_iRegP_iRegP_rule,
  /*  470 */  _CmpN_iRegN_iRegN_rule,
  /*  471 */  _CmpN_iRegN_iRegN_rule,
  /*  472 */  _CmpF_fRegF_fRegF_rule,
  /*  473 */  _CmpF_fRegF_fRegF_rule,
  /*  474 */  _CmpD_fRegD_fRegD_rule,
  /*  475 */  _CmpD_fRegD_fRegD_rule,
  /*  476 */  _CmpI_iRegI_immI0_rule,
  /*  477 */  _CmpI_iRegI_immI0_rule,
  /*  478 */  _CmpU_iRegI_immI0_rule,
  /*  479 */  _CmpU_iRegI_immI0_rule,
  /*  480 */  _CmpU_iRegI_immI0_rule,
  /*  481 */  _CmpU_iRegI_immI0_rule,
  /*  482 */  _CmpL_iRegL_immL0_rule,
  /*  483 */  _CmpL_iRegL_immL0_rule,
  /*  484 */  _CmpUL_iRegL_immL0_rule,
  /*  485 */  _CmpUL_iRegL_immL0_rule,
  /*  486 */  _CmpUL_iRegL_immL0_rule,
  /*  487 */  _CmpUL_iRegL_immL0_rule,
  /*  488 */  _CmpP_iRegP_immP0_rule,
  /*  489 */  _CmpP_iRegP_immP0_rule,
  /*  490 */  _CmpN_iRegN_immN0_rule,
  /*  491 */  _CmpN_iRegN_immN0_rule,
  /*  492 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  493 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  494 */  _Binary_iRegINoSp_iRegI_rule,
  /*  495 */  _Binary_iRegINoSp_iRegI_rule,
  /*  496 */  _Binary_iRegINoSp_iRegI_rule,
  /*  497 */  _Binary_iRegINoSp_iRegI_rule,
  /*  498 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  499 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  500 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  501 */  _Binary_iRegLNoSp_iRegL_rule,
  /*  502 */  0,
  /*  503 */  0,
  /*  504 */  0,
  /*  505 */  0,
  /*  506 */  0,
  /*  507 */  iRegP_R10_rule,
  /*  508 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  509 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  510 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  511 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  512 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  513 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  514 */  _Binary_iRegP_R13_iRegI_R14_rule,
  /*  515 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  516 */  _Binary_iRegP_R13_immI_le_4_rule,
  /*  517 */  _Binary_iRegP_R13_immI_1_rule,
  /*  518 */  iRegI_R13_rule,
  /*  519 */  iRegP_R28_rule,
  /*  520 */  iRegP_R28_rule,
  /*  521 */  iRegI_R14_rule,
  /*  522 */  iRegI_R14_rule,
  /*  523 */  iRegP_R12_rule,
  /*  524 */  iRegP_R12_rule,
  /*  525 */  0,
  /*  526 */  inline_cache_RegP_rule,
  /*  527 */  iRegP_R10_rule,
  /*  528 */  0,
  /*  529 */  0,
  /*  530 */  0,
  /*  531 */  0,
  /*  532 */  0,
  /*  533 */  immI_24_rule,
  /*  534 */  immI_16_rule,
  /*  535 */  immI_16bits_rule,
  /*  536 */  immL_32bits_rule,
  /*  537 */  0,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  _LShiftL_iRegL_immIScale_rule,
  /*  542 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  543 */  _LShiftL_iRegL_immIScale_rule,
  /*  544 */  iRegL_rule,
  /*  545 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  546 */  iRegL_rule,
  /*  547 */  iRegI_rule,
  /*  548 */  iRegI_rule,
  /*  549 */  _XorI_iRegI_immI_M1_rule,
  /*  550 */  iRegI_rule,
  /*  551 */  _XorL_iRegL_immL_M1_rule,
  /*  552 */  iRegL_rule,
  /*  553 */  _XorI_iRegI_immI_M1_rule,
  /*  554 */  iRegI_rule,
  /*  555 */  _XorL_iRegL_immL_M1_rule,
  /*  556 */  iRegL_rule,
  /*  557 */  0,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "UIMMI6_GE32",
  /*   13 */  "IMMI_LE_4",
  /*   14 */  "IMMI_16",
  /*   15 */  "IMMI_24",
  /*   16 */  "IMMI_31",
  /*   17 */  "IMMI_63",
  /*   18 */  "IMMIADD",
  /*   19 */  "IMMISUB",
  /*   20 */  "IMMI5",
  /*   21 */  "IMML5",
  /*   22 */  "IMML",
  /*   23 */  "IMML0",
  /*   24 */  "IMMP",
  /*   25 */  "IMMP0",
  /*   26 */  "IMMP_1",
  /*   27 */  "IMMPOLLPAGE",
  /*   28 */  "IMMBYTEMAPBASE",
  /*   29 */  "IMMI_16BITS",
  /*   30 */  "IMMIPOWEROF2",
  /*   31 */  "IMML_32BITS",
  /*   32 */  "IMML_M1",
  /*   33 */  "IMML_PC_OFF",
  /*   34 */  "IMMLADD",
  /*   35 */  "IMMLSUB",
  /*   36 */  "IMMN",
  /*   37 */  "IMMN0",
  /*   38 */  "IMMNKLASS",
  /*   39 */  "IMMD",
  /*   40 */  "IMMD0",
  /*   41 */  "IMMF",
  /*   42 */  "IMMF0",
  /*   43 */  "IMMIOFFSET",
  /*   44 */  "IMMLOFFSET",
  /*   45 */  "IMMISCALE",
  /*   46 */  "IREGI",
  /*   47 */  "IREGINOSP",
  /*   48 */  "IREGI_R10",
  /*   49 */  "IREGI_R12",
  /*   50 */  "IREGI_R13",
  /*   51 */  "IREGI_R14",
  /*   52 */  "IREGL",
  /*   53 */  "IREGLNOSP",
  /*   54 */  "IREGL_R28",
  /*   55 */  "IREGL_R29",
  /*   56 */  "IREGL_R30",
  /*   57 */  "IREGP",
  /*   58 */  "IREGPNOSP",
  /*   59 */  "IREGP_R10",
  /*   60 */  "IREGP_R11",
  /*   61 */  "IREGP_R12",
  /*   62 */  "IREGP_R13",
  /*   63 */  "IREGP_R14",
  /*   64 */  "IREGP_R15",
  /*   65 */  "IREGP_R16",
  /*   66 */  "IREGP_R28",
  /*   67 */  "IREGN",
  /*   68 */  "IREGNNOSP",
  /*   69 */  "IREGIHEAPBASE",
  /*   70 */  "IREGL_R10",
  /*   71 */  "FREGF",
  /*   72 */  "FREGD",
  /*   73 */  "JAVATHREAD_REGP",
  /*   74 */  "INDIRECT",
  /*   75 */  "INDOFFI",
  /*   76 */  "INDOFFL",
  /*   77 */  "INDIRECTN",
  /*   78 */  "INDOFFIN",
  /*   79 */  "INDOFFLN",
  /*   80 */  "THREAD_ANCHOR_PC",
  /*   81 */  "STACKSLOTI",
  /*   82 */  "STACKSLOTF",
  /*   83 */  "STACKSLOTD",
  /*   84 */  "STACKSLOTL",
  /*   85 */  "IREGL2I",
  /*   86 */  "CMPOP",
  /*   87 */  "CMPOPU",
  /*   88 */  "CMPOPEQNE",
  /*   89 */  "CMPOPULTGE",
  /*   90 */  "CMPOPUEQNELEGT",
  /*   91 */  "RFLAGSREG",
  /*   92 */  "INLINE_CACHE_REGP",
  // last operand
  /*   93 */  "MEMORY",
  /*   94 */  "IREGIORL2I",
  /*   95 */  "IREGIORL",
  /*   96 */  "IREGNORP",
  /*   97 */  "IREGILNP",
  /*   98 */  "IREGILNPNOSP",
  /*   99 */  "IMMIORL",
  // last operand class
  /*  100 */  "_DecodeN_iRegN_",
  /*  101 */  "_LoadB_memory_",
  /*  102 */  "_LoadUB_memory_",
  /*  103 */  "_LoadS_memory_",
  /*  104 */  "_LoadUS_memory_",
  /*  105 */  "_LoadI_memory_",
  /*  106 */  "_ConvI2L__LoadI_memory__",
  /*  107 */  "_Binary_iRegP_iRegP",
  /*  108 */  "_Binary_iRegL_iRegL",
  /*  109 */  "_Binary_iRegI_iRegI",
  /*  110 */  "_Binary_iRegI_R12_iRegI_R13",
  /*  111 */  "_Binary_iRegN_iRegN",
  /*  112 */  "_ConvL2I_iRegL_",
  /*  113 */  "_ConvI2L_iRegI_",
  /*  114 */  "_RShiftI_iRegIorL2I_immI_31",
  /*  115 */  "_RShiftL_iRegL_immI_63",
  /*  116 */  "_CastP2X_iRegP_",
  /*  117 */  "_Binary_fRegF_fRegF",
  /*  118 */  "_Binary_fRegD_fRegD",
  /*  119 */  "_NegF_fRegF_",
  /*  120 */  "_NegD_fRegD_",
  /*  121 */  "_Binary__NegF_fRegF__fRegF",
  /*  122 */  "_Binary_fRegF__NegF_fRegF_",
  /*  123 */  "_Binary__NegD_fRegD__fRegD",
  /*  124 */  "_Binary_fRegD__NegD_fRegD_",
  /*  125 */  "_ConvI2L_iRegIorL2I_",
  /*  126 */  "_CastP2X__DecodeN_iRegN__",
  /*  127 */  "_CmpI_iRegI_iRegI",
  /*  128 */  "_CmpU_iRegI_iRegI",
  /*  129 */  "_CmpL_iRegL_iRegL",
  /*  130 */  "_CmpUL_iRegL_iRegL",
  /*  131 */  "_CmpP_iRegP_iRegP",
  /*  132 */  "_CmpN_iRegN_iRegN",
  /*  133 */  "_CmpF_fRegF_fRegF",
  /*  134 */  "_CmpD_fRegD_fRegD",
  /*  135 */  "_CmpI_iRegI_immI0",
  /*  136 */  "_CmpU_iRegI_immI0",
  /*  137 */  "_CmpL_iRegL_immL0",
  /*  138 */  "_CmpUL_iRegL_immL0",
  /*  139 */  "_CmpP_iRegP_immP0",
  /*  140 */  "_CmpN_iRegN_immN0",
  /*  141 */  "_CmpP__DecodeN_iRegN__immP0",
  /*  142 */  "_Binary_cmpOp__CmpI_iRegI_iRegI",
  /*  143 */  "_Binary_iRegINoSp_iRegI",
  /*  144 */  "_Binary_cmpOpU__CmpU_iRegI_iRegI",
  /*  145 */  "_Binary_cmpOp__CmpL_iRegL_iRegL",
  /*  146 */  "_Binary_cmpOpU__CmpUL_iRegL_iRegL",
  /*  147 */  "_Binary_iRegLNoSp_iRegL",
  /*  148 */  "_PartialSubtypeCheck_iRegP_R14_iRegP_R10",
  /*  149 */  "_Binary_iRegP_R11_iRegI_R12",
  /*  150 */  "_Binary_iRegP_R13_iRegI_R14",
  /*  151 */  "_Binary_iRegP_R13_immI_le_4",
  /*  152 */  "_Binary_iRegP_R13_immI_1",
  /*  153 */  "_Binary_iRegP_R11_iRegP_R13",
  /*  154 */  "_LShiftI_iRegIorL2I_immI_24",
  /*  155 */  "_LShiftI_iRegIorL2I_immI_16",
  /*  156 */  "_LShiftL_iRegL_immIScale",
  /*  157 */  "_LShiftL__ConvI2L_iRegI__immIScale",
  /*  158 */  "_XorI_iRegI_immI_M1",
  /*  159 */  "_XorL_iRegL_immL_M1",
  /*  160 */  "_AndI_iRegIorL2I_immIpowerOf2",
  // last internally defined operand
  /*  161 */  "loadConI",
  /*  162 */  "loadConL",
  /*  163 */  "loadConP",
  /*  164 */  "loadConP0",
  /*  165 */  "loadConP1",
  /*  166 */  "loadConPollPage",
  /*  167 */  "loadByteMapBase",
  /*  168 */  "loadConN",
  /*  169 */  "loadConN0",
  /*  170 */  "loadConNKlass",
  /*  171 */  "loadConF",
  /*  172 */  "loadConF0",
  /*  173 */  "loadConD",
  /*  174 */  "loadConD0",
  /*  175 */  "storePConditional",
  /*  176 */  "storeLConditional",
  /*  177 */  "storeIConditional",
  /*  178 */  "negF_reg_reg",
  /*  179 */  "negD_reg_reg",
  /*  180 */  "absI_reg",
  /*  181 */  "absL_reg",
  /*  182 */  "absF_reg",
  /*  183 */  "absD_reg",
  /*  184 */  "castX2P",
  /*  185 */  "castP2X",
  /*  186 */  "castPP",
  /*  187 */  "castII",
  /*  188 */  "checkCastPP",
  /*  189 */  "convI2Bool",
  /*  190 */  "convP2Bool",
  /*  191 */  "MoveF2I_stack_reg",
  /*  192 */  "MoveI2F_stack_reg",
  /*  193 */  "MoveD2L_stack_reg",
  /*  194 */  "MoveL2D_stack_reg",
  /*  195 */  "MoveF2I_reg_reg",
  /*  196 */  "MoveI2F_reg_reg",
  /*  197 */  "MoveD2L_reg_reg",
  /*  198 */  "MoveL2D_reg_reg",
  /*  199 */  "partialSubtypeCheckVsZero",
  /*  200 */  "tlsLoadP",
  /*  201 */  "cmpFastLock",
  /*  202 */  "cmpFastUnlock",
  /*  203 */  "countLeadingZerosI_b",
  /*  204 */  "countLeadingZerosL_b",
  /*  205 */  "countTrailingZerosI_b",
  /*  206 */  "countTrailingZerosL_b",
  /*  207 */  "popCountI_b",
  /*  208 */  "popCountL_b",
  /*  209 */  "absI_reg_b",
  /*  210 */  "absL_reg_b",
  /*  211 */  "loadB",
  /*  212 */  "loadB2L",
  /*  213 */  "loadUB",
  /*  214 */  "loadUB2L",
  /*  215 */  "loadS",
  /*  216 */  "loadS2L",
  /*  217 */  "loadUS",
  /*  218 */  "loadUS2L",
  /*  219 */  "loadI",
  /*  220 */  "loadI2L",
  /*  221 */  "loadUI2L",
  /*  222 */  "loadL",
  /*  223 */  "loadRange",
  /*  224 */  "loadP",
  /*  225 */  "loadN",
  /*  226 */  "loadKlass",
  /*  227 */  "loadNKlass",
  /*  228 */  "loadF",
  /*  229 */  "loadD",
  /*  230 */  "storeimmCM0",
  /*  231 */  "storeimmCM0_ordered",
  /*  232 */  "storeB",
  /*  233 */  "storeimmB0",
  /*  234 */  "storeC",
  /*  235 */  "storeimmC0",
  /*  236 */  "storeI",
  /*  237 */  "storeimmI0",
  /*  238 */  "storeL",
  /*  239 */  "storeimmL0",
  /*  240 */  "storeP",
  /*  241 */  "storeimmP0",
  /*  242 */  "storeN",
  /*  243 */  "storeImmN0",
  /*  244 */  "storeF",
  /*  245 */  "storeD",
  /*  246 */  "storeNKlass",
  /*  247 */  "loadPLocked",
  /*  248 */  "compareAndSwapB",
  /*  249 */  "compareAndSwapS",
  /*  250 */  "compareAndSwapI",
  /*  251 */  "compareAndSwapL",
  /*  252 */  "compareAndSwapP",
  /*  253 */  "compareAndSwapN",
  /*  254 */  "compareAndSwapBAcq",
  /*  255 */  "compareAndSwapSAcq",
  /*  256 */  "compareAndSwapIAcq",
  /*  257 */  "compareAndSwapLAcq",
  /*  258 */  "compareAndSwapPAcq",
  /*  259 */  "compareAndSwapNAcq",
  /*  260 */  "compareAndExchangeB",
  /*  261 */  "compareAndExchangeS",
  /*  262 */  "compareAndExchangeI",
  /*  263 */  "compareAndExchangeL",
  /*  264 */  "compareAndExchangeN",
  /*  265 */  "compareAndExchangeP",
  /*  266 */  "compareAndExchangeBAcq",
  /*  267 */  "compareAndExchangeSAcq",
  /*  268 */  "compareAndExchangeIAcq",
  /*  269 */  "compareAndExchangeLAcq",
  /*  270 */  "compareAndExchangeNAcq",
  /*  271 */  "compareAndExchangePAcq",
  /*  272 */  "weakCompareAndSwapB",
  /*  273 */  "weakCompareAndSwapS",
  /*  274 */  "weakCompareAndSwapI",
  /*  275 */  "weakCompareAndSwapL",
  /*  276 */  "weakCompareAndSwapN",
  /*  277 */  "weakCompareAndSwapP",
  /*  278 */  "weakCompareAndSwapBAcq",
  /*  279 */  "weakCompareAndSwapSAcq",
  /*  280 */  "weakCompareAndSwapIAcq",
  /*  281 */  "weakCompareAndSwapLAcq",
  /*  282 */  "weakCompareAndSwapNAcq",
  /*  283 */  "weakCompareAndSwapPAcq",
  /*  284 */  "get_and_setI",
  /*  285 */  "get_and_setL",
  /*  286 */  "get_and_setN",
  /*  287 */  "get_and_setP",
  /*  288 */  "get_and_setIAcq",
  /*  289 */  "get_and_setLAcq",
  /*  290 */  "get_and_setNAcq",
  /*  291 */  "get_and_setPAcq",
  /*  292 */  "get_and_addL",
  /*  293 */  "get_and_addL_no_res",
  /*  294 */  "get_and_addLi",
  /*  295 */  "get_and_addLi_no_res",
  /*  296 */  "get_and_addI",
  /*  297 */  "get_and_addI_no_res",
  /*  298 */  "get_and_addIi",
  /*  299 */  "get_and_addIi_no_res",
  /*  300 */  "get_and_addLAcq",
  /*  301 */  "get_and_addL_no_resAcq",
  /*  302 */  "get_and_addLiAcq",
  /*  303 */  "get_and_addLi_no_resAcq",
  /*  304 */  "get_and_addIAcq",
  /*  305 */  "get_and_addI_no_resAcq",
  /*  306 */  "get_and_addIiAcq",
  /*  307 */  "get_and_addIi_no_resAcq",
  /*  308 */  "addI_reg_reg",
  /*  309 */  "addI_reg_imm",
  /*  310 */  "addI_reg_imm_l2i",
  /*  311 */  "addP_reg_reg",
  /*  312 */  "lShiftL_regI_immGE32",
  /*  313 */  "addP_reg_imm",
  /*  314 */  "addL_reg_reg",
  /*  315 */  "addL_reg_imm",
  /*  316 */  "subI_reg_reg",
  /*  317 */  "subI_reg_imm",
  /*  318 */  "subL_reg_reg",
  /*  319 */  "subL_reg_imm",
  /*  320 */  "negI_reg",
  /*  321 */  "negL_reg",
  /*  322 */  "mulI",
  /*  323 */  "mulL",
  /*  324 */  "mulHiL_rReg",
  /*  325 */  "divI",
  /*  326 */  "signExtract",
  /*  327 */  "divL",
  /*  328 */  "signExtractL",
  /*  329 */  "modI",
  /*  330 */  "modL",
  /*  331 */  "lShiftI_reg_reg",
  /*  332 */  "lShiftI_reg_imm",
  /*  333 */  "urShiftI_reg_reg",
  /*  334 */  "urShiftI_reg_imm",
  /*  335 */  "rShiftI_reg_reg",
  /*  336 */  "rShiftI_reg_imm",
  /*  337 */  "lShiftL_reg_reg",
  /*  338 */  "lShiftL_reg_imm",
  /*  339 */  "urShiftL_reg_reg",
  /*  340 */  "urShiftL_reg_imm",
  /*  341 */  "urShiftP_reg_imm",
  /*  342 */  "rShiftL_reg_reg",
  /*  343 */  "rShiftL_reg_imm",
  /*  344 */  "regI_not_reg",
  /*  345 */  "regL_not_reg",
  /*  346 */  "addF_reg_reg",
  /*  347 */  "addD_reg_reg",
  /*  348 */  "subF_reg_reg",
  /*  349 */  "subD_reg_reg",
  /*  350 */  "mulF_reg_reg",
  /*  351 */  "mulD_reg_reg",
  /*  352 */  "maddF_reg_reg",
  /*  353 */  "maddD_reg_reg",
  /*  354 */  "msubF_reg_reg",
  /*  355 */  "msubD_reg_reg",
  /*  356 */  "nmsubF_reg_reg",
  /*  357 */  "nmsubF_reg_reg_0",
  /*  358 */  "nmsubD_reg_reg",
  /*  359 */  "nmsubD_reg_reg_0",
  /*  360 */  "nmaddF_reg_reg",
  /*  361 */  "nmaddF_reg_reg_0",
  /*  362 */  "nmaddD_reg_reg",
  /*  363 */  "nmaddD_reg_reg_0",
  /*  364 */  "maxF_reg_reg",
  /*  365 */  "minF_reg_reg",
  /*  366 */  "maxD_reg_reg",
  /*  367 */  "minD_reg_reg",
  /*  368 */  "divF_reg_reg",
  /*  369 */  "divD_reg_reg",
  /*  370 */  "sqrtF_reg",
  /*  371 */  "sqrtD_reg",
  /*  372 */  "andI_reg_reg",
  /*  373 */  "andI_reg_imm",
  /*  374 */  "orI_reg_reg",
  /*  375 */  "orI_reg_imm",
  /*  376 */  "xorI_reg_reg",
  /*  377 */  "xorI_reg_imm",
  /*  378 */  "andL_reg_reg",
  /*  379 */  "andL_reg_imm",
  /*  380 */  "orL_reg_reg",
  /*  381 */  "orL_reg_imm",
  /*  382 */  "xorL_reg_reg",
  /*  383 */  "xorL_reg_imm",
  /*  384 */  "bytes_reverse_int",
  /*  385 */  "bytes_reverse_long",
  /*  386 */  "bytes_reverse_unsigned_short",
  /*  387 */  "bytes_reverse_short",
  /*  388 */  "load_fence",
  /*  389 */  "membar_acquire",
  /*  390 */  "membar_acquire_lock",
  /*  391 */  "store_fence",
  /*  392 */  "membar_release",
  /*  393 */  "membar_storestore",
  /*  394 */  "membar_release_lock",
  /*  395 */  "membar_volatile",
  /*  396 */  "convI2L_reg_reg",
  /*  397 */  "convL2I_reg",
  /*  398 */  "convI2UL_reg_reg",
  /*  399 */  "convF2D_reg",
  /*  400 */  "convD2F_reg",
  /*  401 */  "convF2I_reg_reg",
  /*  402 */  "convI2F_reg_reg",
  /*  403 */  "convF2L_reg_reg",
  /*  404 */  "convL2F_reg_reg",
  /*  405 */  "convD2I_reg_reg",
  /*  406 */  "convI2D_reg_reg",
  /*  407 */  "convD2L_reg_reg",
  /*  408 */  "convL2D_reg_reg",
  /*  409 */  "convP2I",
  /*  410 */  "convN2I",
  /*  411 */  "encodeHeapOop",
  /*  412 */  "decodeHeapOop",
  /*  413 */  "decodeHeapOop_not_null",
  /*  414 */  "encodeKlass_not_null",
  /*  415 */  "decodeKlass_not_null",
  /*  416 */  "MoveF2I_reg_stack",
  /*  417 */  "MoveI2F_reg_stack",
  /*  418 */  "MoveD2L_reg_stack",
  /*  419 */  "MoveL2D_reg_stack",
  /*  420 */  "cmpF3_reg_reg",
  /*  421 */  "cmpD3_reg_reg",
  /*  422 */  "cmpL3_reg_reg",
  /*  423 */  "cmpLTMask_reg_reg",
  /*  424 */  "cmpLTMask_reg_zero",
  /*  425 */  "minI_rReg",
  /*  426 */  "maxI_rReg",
  /*  427 */  "branch",
  /*  428 */  "cmpFlag_branch",
  /*  429 */  "cmpI_branch",
  /*  430 */  "cmpI_loop",
  /*  431 */  "cmpU_branch",
  /*  432 */  "cmpU_loop",
  /*  433 */  "cmpL_branch",
  /*  434 */  "cmpL_loop",
  /*  435 */  "cmpUL_branch",
  /*  436 */  "cmpUL_loop",
  /*  437 */  "cmpP_branch",
  /*  438 */  "cmpP_loop",
  /*  439 */  "cmpN_branch",
  /*  440 */  "cmpN_loop",
  /*  441 */  "cmpF_branch",
  /*  442 */  "cmpF_loop",
  /*  443 */  "cmpD_branch",
  /*  444 */  "cmpD_loop",
  /*  445 */  "cmpI_reg_imm0_branch",
  /*  446 */  "cmpI_reg_imm0_loop",
  /*  447 */  "cmpUEqNeLeGt_reg_imm0_branch",
  /*  448 */  "cmpUEqNeLeGt_reg_imm0_loop",
  /*  449 */  "cmpL_reg_imm0_branch",
  /*  450 */  "cmpL_reg_imm0_loop",
  /*  451 */  "cmpULEqNeLeGt_reg_imm0_branch",
  /*  452 */  "cmpULEqNeLeGt_reg_imm0_loop",
  /*  453 */  "cmpP_imm0_branch",
  /*  454 */  "cmpP_imm0_loop",
  /*  455 */  "cmpN_imm0_branch",
  /*  456 */  "cmpN_imm0_loop",
  /*  457 */  "cmpP_narrowOop_imm0_branch",
  /*  458 */  "cmpP_narrowOop_imm0_loop",
  /*  459 */  "far_cmpFlag_branch",
  /*  460 */  "far_cmpI_branch",
  /*  461 */  "far_cmpI_loop",
  /*  462 */  "far_cmpU_branch",
  /*  463 */  "far_cmpU_loop",
  /*  464 */  "far_cmpL_branch",
  /*  465 */  "far_cmpLloop",
  /*  466 */  "far_cmpUL_branch",
  /*  467 */  "far_cmpUL_loop",
  /*  468 */  "far_cmpP_branch",
  /*  469 */  "far_cmpP_loop",
  /*  470 */  "far_cmpN_branch",
  /*  471 */  "far_cmpN_loop",
  /*  472 */  "far_cmpF_branch",
  /*  473 */  "far_cmpF_loop",
  /*  474 */  "far_cmpD_branch",
  /*  475 */  "far_cmpD_loop",
  /*  476 */  "far_cmpI_reg_imm0_branch",
  /*  477 */  "far_cmpI_reg_imm0_loop",
  /*  478 */  "far_cmpUEqNeLeGt_imm0_branch",
  /*  479 */  "far_cmpUEqNeLeGt_reg_imm0_loop",
  /*  480 */  "far_cmpULtGe_reg_imm0_branch",
  /*  481 */  "far_cmpULtGe_reg_imm0_loop",
  /*  482 */  "far_cmpL_reg_imm0_branch",
  /*  483 */  "far_cmpL_reg_imm0_loop",
  /*  484 */  "far_cmpULEqNeLeGt_reg_imm0_branch",
  /*  485 */  "far_cmpULEqNeLeGt_reg_imm0_loop",
  /*  486 */  "far_cmpULLtGe_reg_imm0_branch",
  /*  487 */  "far_cmpULLtGe_reg_imm0_loop",
  /*  488 */  "far_cmpP_imm0_branch",
  /*  489 */  "far_cmpP_imm0_loop",
  /*  490 */  "far_cmpN_imm0_branch",
  /*  491 */  "far_cmpN_imm0_loop",
  /*  492 */  "far_cmpP_narrowOop_imm0_branch",
  /*  493 */  "far_cmpP_narrowOop_imm0_loop",
  /*  494 */  "cmovI_cmpI",
  /*  495 */  "cmovI_cmpU",
  /*  496 */  "cmovI_cmpL",
  /*  497 */  "cmovI_cmpUL",
  /*  498 */  "cmovL_cmpL",
  /*  499 */  "cmovL_cmpUL",
  /*  500 */  "cmovL_cmpI",
  /*  501 */  "cmovL_cmpU",
  /*  502 */  "CallStaticJavaDirect",
  /*  503 */  "CallDynamicJavaDirect",
  /*  504 */  "CallRuntimeDirect",
  /*  505 */  "CallLeafDirect",
  /*  506 */  "CallLeafNoFPDirect",
  /*  507 */  "partialSubtypeCheck",
  /*  508 */  "string_compareU",
  /*  509 */  "string_compareL",
  /*  510 */  "string_compareUL",
  /*  511 */  "string_compareLU",
  /*  512 */  "string_indexofUU",
  /*  513 */  "string_indexofLL",
  /*  514 */  "string_indexofUL",
  /*  515 */  "string_indexof_conUU",
  /*  516 */  "string_indexof_conLL",
  /*  517 */  "string_indexof_conUL",
  /*  518 */  "stringU_indexof_char",
  /*  519 */  "clearArray_reg_reg",
  /*  520 */  "clearArray_imm_reg",
  /*  521 */  "string_equalsL",
  /*  522 */  "string_equalsU",
  /*  523 */  "array_equalsB",
  /*  524 */  "array_equalsC",
  /*  525 */  "safePoint",
  /*  526 */  "TailCalljmpInd",
  /*  527 */  "TailjmpInd",
  /*  528 */  "CreateException",
  /*  529 */  "RethrowException",
  /*  530 */  "Ret",
  /*  531 */  "ShouldNotReachHere",
  /*  532 */  "convP2I_b",
  /*  533 */  "convB2I_reg_reg_b",
  /*  534 */  "convI2S_reg_reg_b",
  /*  535 */  "convS2UI_reg_reg_b",
  /*  536 */  "convI2UL_reg_reg_b",
  /*  537 */  "bytes_reverse_int_b",
  /*  538 */  "bytes_reverse_long_b",
  /*  539 */  "bytes_reverse_unsigned_short_b",
  /*  540 */  "bytes_reverse_short_b",
  /*  541 */  "shaddP_reg_reg_b",
  /*  542 */  "shaddP_reg_reg_ext_b",
  /*  543 */  "shaddL_reg_reg_b",
  /*  544 */  "shaddL_reg_reg_b_0",
  /*  545 */  "shaddL_reg_reg_ext_b",
  /*  546 */  "shaddL_reg_reg_ext_b_0",
  /*  547 */  "minI_reg_b",
  /*  548 */  "maxI_reg_b",
  /*  549 */  "andnI_reg_reg_b",
  /*  550 */  "andnI_reg_reg_b_0",
  /*  551 */  "andnL_reg_reg_b",
  /*  552 */  "andnL_reg_reg_b_0",
  /*  553 */  "ornI_reg_reg_b",
  /*  554 */  "ornI_reg_reg_b_0",
  /*  555 */  "ornL_reg_reg_b",
  /*  556 */  "ornL_reg_reg_b_0",
  /*  557 */  "convI2Bool_andI_reg_immIpowerOf2",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  true,
  /*   87 */  true,
  /*   88 */  true,
  /*   89 */  true,
  /*   90 */  true,
  /*   91 */  false,
  /*   92 */  false,
  // last operand
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  // last operand class
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  // last internally defined operand
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // _last_machine_leaf: 14
  0, // AbsD: 15
  0, // AbsF: 16
  0, // AbsI: 17
  0, // AbsL: 18
  0, // AddD: 19
  0, // AddF: 20
  0, // AddI: 21
  0, // AddL: 22
  0, // AddP: 23
  0, // Allocate: 24
  0, // AllocateArray: 25
  0, // AndI: 26
  0, // AndL: 27
  0, // ArrayCopy: 28
  0, // AryEq: 29
  0, // AtanD: 30
  1, // Binary: 31
  1, // Bool: 32
  0, // BoxLock: 33
  0, // ReverseBytesI: 34
  0, // ReverseBytesL: 35
  0, // ReverseBytesUS: 36
  0, // ReverseBytesS: 37
  0, // CProj: 38
  0, // CallDynamicJava: 39
  0, // CallJava: 40
  0, // CallLeaf: 41
  0, // CallLeafNoFP: 42
  0, // CallRuntime: 43
  0, // CallStaticJava: 44
  0, // CastII: 45
  0, // CastX2P: 46
  0, // CastP2X: 47
  0, // CastPP: 48
  0, // Catch: 49
  0, // CatchProj: 50
  0, // CheckCastPP: 51
  0, // ClearArray: 52
  0, // ConstraintCast: 53
  0, // CMoveD: 54
  0, // CMoveVD: 55
  0, // CMoveF: 56
  0, // CMoveVF: 57
  0, // CMoveI: 58
  0, // CMoveL: 59
  0, // CMoveP: 60
  0, // CMoveN: 61
  1, // CmpN: 62
  1, // CmpD: 63
  0, // CmpD3: 64
  1, // CmpF: 65
  0, // CmpF3: 66
  1, // CmpI: 67
  1, // CmpL: 68
  0, // CmpL3: 69
  0, // CmpLTMask: 70
  1, // CmpP: 71
  1, // CmpU: 72
  1, // CmpUL: 73
  0, // CompareAndSwapB: 74
  0, // CompareAndSwapS: 75
  0, // CompareAndSwapI: 76
  0, // CompareAndSwapL: 77
  0, // CompareAndSwapP: 78
  0, // CompareAndSwapN: 79
  0, // WeakCompareAndSwapB: 80
  0, // WeakCompareAndSwapS: 81
  0, // WeakCompareAndSwapI: 82
  0, // WeakCompareAndSwapL: 83
  0, // WeakCompareAndSwapP: 84
  0, // WeakCompareAndSwapN: 85
  0, // CompareAndExchangeB: 86
  0, // CompareAndExchangeS: 87
  0, // CompareAndExchangeI: 88
  0, // CompareAndExchangeL: 89
  0, // CompareAndExchangeP: 90
  0, // CompareAndExchangeN: 91
  0, // GetAndAddB: 92
  0, // GetAndAddS: 93
  0, // GetAndAddI: 94
  0, // GetAndAddL: 95
  0, // GetAndSetB: 96
  0, // GetAndSetS: 97
  0, // GetAndSetI: 98
  0, // GetAndSetL: 99
  0, // GetAndSetP: 100
  0, // GetAndSetN: 101
  0, // Con: 102
  0, // ConN: 103
  0, // ConNKlass: 104
  0, // ConD: 105
  0, // ConF: 106
  0, // ConI: 107
  0, // ConL: 108
  0, // ConP: 109
  0, // Conv2B: 110
  0, // ConvD2F: 111
  0, // ConvD2I: 112
  0, // ConvD2L: 113
  0, // ConvF2D: 114
  0, // ConvF2I: 115
  0, // ConvF2L: 116
  0, // ConvI2D: 117
  0, // ConvI2F: 118
  0, // ConvI2L: 119
  0, // ConvL2D: 120
  0, // ConvL2F: 121
  0, // ConvL2I: 122
  0, // CountedLoop: 123
  0, // CountedLoopEnd: 124
  0, // OuterStripMinedLoop: 125
  0, // OuterStripMinedLoopEnd: 126
  0, // CountLeadingZerosI: 127
  0, // CountLeadingZerosL: 128
  0, // CountTrailingZerosI: 129
  0, // CountTrailingZerosL: 130
  0, // CreateEx: 131
  0, // DecodeN: 132
  0, // DecodeNKlass: 133
  0, // DivD: 134
  0, // DivF: 135
  0, // DivI: 136
  0, // DivL: 137
  0, // DivMod: 138
  0, // DivModI: 139
  0, // DivModL: 140
  0, // EncodeISOArray: 141
  0, // EncodeP: 142
  0, // EncodePKlass: 143
  1, // FastLock: 144
  1, // FastUnlock: 145
  0, // FmaD: 146
  0, // FmaF: 147
  0, // Goto: 148
  0, // Halt: 149
  0, // HasNegatives: 150
  0, // If: 151
  0, // RangeCheck: 152
  0, // IfFalse: 153
  0, // IfTrue: 154
  0, // Initialize: 155
  0, // JProj: 156
  0, // Jump: 157
  0, // JumpProj: 158
  0, // LShiftI: 159
  0, // LShiftL: 160
  0, // LoadB: 161
  0, // LoadUB: 162
  0, // LoadUS: 163
  0, // LoadD: 164
  0, // LoadD_unaligned: 165
  0, // LoadF: 166
  0, // LoadI: 167
  0, // LoadKlass: 168
  0, // LoadNKlass: 169
  0, // LoadL: 170
  0, // LoadL_unaligned: 171
  0, // LoadPLocked: 172
  0, // LoadP: 173
  0, // LoadN: 174
  0, // LoadRange: 175
  0, // LoadS: 176
  0, // LoadBarrier: 177
  0, // LoadBarrierSlowReg: 178
  0, // LoadBarrierWeakSlowReg: 179
  0, // Lock: 180
  0, // Loop: 181
  0, // LoopLimit: 182
  0, // Mach: 183
  0, // MachNullCheck: 184
  0, // MachProj: 185
  0, // MaxD: 186
  0, // MaxF: 187
  0, // MaxI: 188
  0, // MaxL: 189
  0, // MemBarAcquire: 190
  0, // LoadFence: 191
  0, // SetVectMaskI: 192
  0, // MemBarAcquireLock: 193
  0, // MemBarCPUOrder: 194
  0, // MemBarRelease: 195
  0, // StoreFence: 196
  0, // MemBarReleaseLock: 197
  0, // MemBarVolatile: 198
  0, // MemBarStoreStore: 199
  0, // MergeMem: 200
  0, // MinD: 201
  0, // MinF: 202
  0, // MinI: 203
  0, // MinL: 204
  0, // ModD: 205
  0, // ModF: 206
  0, // ModI: 207
  0, // ModL: 208
  0, // MoveI2F: 209
  0, // MoveF2I: 210
  0, // MoveL2D: 211
  0, // MoveD2L: 212
  0, // MulD: 213
  0, // MulF: 214
  0, // MulHiL: 215
  0, // MulI: 216
  0, // MulL: 217
  0, // Multi: 218
  0, // NegD: 219
  0, // NegF: 220
  0, // NeverBranch: 221
  0, // OnSpinWait: 222
  0, // Opaque1: 223
  0, // OpaqueLoopInit: 224
  0, // OpaqueLoopStride: 225
  0, // Opaque2: 226
  0, // Opaque3: 227
  0, // Opaque4: 228
  0, // ProfileBoolean: 229
  0, // OrI: 230
  0, // OrL: 231
  1, // OverflowAddI: 232
  1, // OverflowSubI: 233
  1, // OverflowMulI: 234
  1, // OverflowAddL: 235
  1, // OverflowSubL: 236
  1, // OverflowMulL: 237
  0, // PCTable: 238
  0, // Parm: 239
  0, // PartialSubtypeCheck: 240
  0, // Phi: 241
  0, // PopCountI: 242
  0, // PopCountL: 243
  0, // PopCountVI: 244
  0, // PrefetchAllocation: 245
  0, // Proj: 246
  0, // RShiftI: 247
  0, // RShiftL: 248
  0, // Region: 249
  0, // Rethrow: 250
  0, // Return: 251
  0, // Root: 252
  0, // RoundDouble: 253
  0, // RoundDoubleMode: 254
  0, // RoundDoubleModeV: 255
  0, // RoundFloat: 256
  0, // SafePoint: 257
  0, // SafePointScalarObject: 258
  0, // SCMemProj: 259
  0, // CopySignD: 260
  0, // CopySignF: 261
  0, // SignumD: 262
  0, // SignumF: 263
  0, // SqrtD: 264
  0, // SqrtF: 265
  0, // Start: 266
  0, // StartOSR: 267
  0, // StoreB: 268
  0, // StoreC: 269
  0, // StoreCM: 270
  0, // StorePConditional: 271
  0, // StoreIConditional: 272
  0, // StoreLConditional: 273
  0, // StoreD: 274
  0, // StoreF: 275
  0, // StoreI: 276
  0, // StoreL: 277
  0, // StoreP: 278
  0, // StoreN: 279
  0, // StoreNKlass: 280
  0, // StrComp: 281
  0, // StrCompressedCopy: 282
  0, // StrEquals: 283
  0, // StrIndexOf: 284
  0, // StrIndexOfChar: 285
  0, // StrInflatedCopy: 286
  0, // SubD: 287
  0, // SubF: 288
  0, // SubI: 289
  0, // SubL: 290
  0, // TailCall: 291
  0, // TailJump: 292
  0, // ThreadLocal: 293
  0, // Unlock: 294
  0, // URShiftI: 295
  0, // URShiftL: 296
  0, // XorI: 297
  0, // XorL: 298
  0, // Vector: 299
  0, // AddVB: 300
  0, // AddVS: 301
  0, // AddVI: 302
  0, // AddReductionVI: 303
  0, // AddVL: 304
  0, // AddReductionVL: 305
  0, // AddVF: 306
  0, // AddReductionVF: 307
  0, // AddVD: 308
  0, // AddReductionVD: 309
  0, // SubVB: 310
  0, // SubVS: 311
  0, // SubVI: 312
  0, // SubVL: 313
  0, // SubVF: 314
  0, // SubVD: 315
  0, // MulVB: 316
  0, // MulVS: 317
  0, // MulVI: 318
  0, // MulReductionVI: 319
  0, // MulVL: 320
  0, // MulReductionVL: 321
  0, // MulVF: 322
  0, // MulReductionVF: 323
  0, // MulVD: 324
  0, // MulReductionVD: 325
  0, // FmaVD: 326
  0, // FmaVF: 327
  0, // DivVF: 328
  0, // DivVD: 329
  0, // AbsVB: 330
  0, // AbsVS: 331
  0, // AbsVI: 332
  0, // AbsVL: 333
  0, // AbsVF: 334
  0, // AbsVD: 335
  0, // NegVF: 336
  0, // NegVD: 337
  0, // SqrtVD: 338
  0, // SqrtVF: 339
  0, // LShiftCntV: 340
  0, // RShiftCntV: 341
  0, // LShiftVB: 342
  0, // LShiftVS: 343
  0, // LShiftVI: 344
  0, // LShiftVL: 345
  0, // RShiftVB: 346
  0, // RShiftVS: 347
  0, // RShiftVI: 348
  0, // RShiftVL: 349
  0, // URShiftVB: 350
  0, // URShiftVS: 351
  0, // URShiftVI: 352
  0, // URShiftVL: 353
  0, // AndV: 354
  0, // OrV: 355
  0, // XorV: 356
  0, // MinV: 357
  0, // MaxV: 358
  0, // MinReductionV: 359
  0, // MaxReductionV: 360
  0, // LoadVector: 361
  0, // StoreVector: 362
  0, // Pack: 363
  0, // PackB: 364
  0, // PackS: 365
  0, // PackI: 366
  0, // PackL: 367
  0, // PackF: 368
  0, // PackD: 369
  0, // Pack2L: 370
  0, // Pack2D: 371
  0, // ReplicateB: 372
  0, // ReplicateS: 373
  0, // ReplicateI: 374
  0, // ReplicateL: 375
  0, // ReplicateF: 376
  0, // ReplicateD: 377
  0, // Extract: 378
  0, // ExtractB: 379
  0, // ExtractUB: 380
  0, // ExtractC: 381
  0, // ExtractS: 382
  0, // ExtractI: 383
  0, // ExtractL: 384
  0, // ExtractF: 385
  0, // ExtractD: 386
  0, // Digit: 387
  0, // LowerCase: 388
  0, // UpperCase: 389
  0 // no trailing comma // Whitespace: 390
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R7",
  "R7_H",
  "R28",
  "R28_H",
  "R29",
  "R29_H",
  "R30",
  "R30_H",
  "R31",
  "R31_H",
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R12",
  "R12_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "R16",
  "R16_H",
  "R17",
  "R17_H",
  "R9",
  "R9_H",
  "R18",
  "R18_H",
  "R19",
  "R19_H",
  "R20",
  "R20_H",
  "R21",
  "R21_H",
  "R22",
  "R22_H",
  "R24",
  "R24_H",
  "R25",
  "R25_H",
  "R26",
  "R26_H",
  "R23",
  "R23_H",
  "R27",
  "R27_H",
  "R4",
  "R4_H",
  "R8",
  "R8_H",
  "R0",
  "R0_H",
  "R1",
  "R1_H",
  "R2",
  "R2_H",
  "R3",
  "R3_H",
  "F0",
  "F0_H",
  "F1",
  "F1_H",
  "F2",
  "F2_H",
  "F3",
  "F3_H",
  "F4",
  "F4_H",
  "F5",
  "F5_H",
  "F6",
  "F6_H",
  "F7",
  "F7_H",
  "F28",
  "F28_H",
  "F29",
  "F29_H",
  "F30",
  "F30_H",
  "F31",
  "F31_H",
  "F10",
  "F10_H",
  "F11",
  "F11_H",
  "F12",
  "F12_H",
  "F13",
  "F13_H",
  "F14",
  "F14_H",
  "F15",
  "F15_H",
  "F16",
  "F16_H",
  "F17",
  "F17_H",
  "F8",
  "F8_H",
  "F9",
  "F9_H",
  "F18",
  "F18_H",
  "F19",
  "F19_H",
  "F20",
  "F20_H",
  "F21",
  "F21_H",
  "F22",
  "F22_H",
  "F23",
  "F23_H",
  "F24",
  "F24_H",
  "F25",
  "F25_H",
  "F26",
  "F26_H",
  "F27",
  "F27_H",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	x7->as_VMReg()         ,
	x7->as_VMReg()->next() ,
	x28->as_VMReg()        ,
	x28->as_VMReg()->next(),
	x29->as_VMReg()        ,
	x29->as_VMReg()->next(),
	x30->as_VMReg()        ,
	x30->as_VMReg()->next(),
	x31->as_VMReg()        ,
	x31->as_VMReg()->next(),
	x10->as_VMReg()        ,
	x10->as_VMReg()->next(),
	x11->as_VMReg()        ,
	x11->as_VMReg()->next(),
	x12->as_VMReg()        ,
	x12->as_VMReg()->next(),
	x13->as_VMReg()        ,
	x13->as_VMReg()->next(),
	x14->as_VMReg()        ,
	x14->as_VMReg()->next(),
	x15->as_VMReg()        ,
	x15->as_VMReg()->next(),
	x16->as_VMReg()        ,
	x16->as_VMReg()->next(),
	x17->as_VMReg()        ,
	x17->as_VMReg()->next(),
	x9->as_VMReg()         ,
	x9->as_VMReg()->next() ,
	x18->as_VMReg()        ,
	x18->as_VMReg()->next(),
	x19->as_VMReg()        ,
	x19->as_VMReg()->next(),
	x20->as_VMReg()        ,
	x20->as_VMReg()->next(),
	x21->as_VMReg()        ,
	x21->as_VMReg()->next(),
	x22->as_VMReg()        ,
	x22->as_VMReg()->next(),
	x24->as_VMReg()        ,
	x24->as_VMReg()->next(),
	x25->as_VMReg()        ,
	x25->as_VMReg()->next(),
	x26->as_VMReg()        ,
	x26->as_VMReg()->next(),
	x23->as_VMReg()        ,
	x23->as_VMReg()->next(),
	x27->as_VMReg()        ,
	x27->as_VMReg()->next(),
	x4->as_VMReg()         ,
	x4->as_VMReg()->next() ,
	x8->as_VMReg()         ,
	x8->as_VMReg()->next() ,
	x0->as_VMReg()         ,
	x0->as_VMReg()->next() ,
	x1->as_VMReg()         ,
	x1->as_VMReg()->next() ,
	x2->as_VMReg()         ,
	x2->as_VMReg()->next() ,
	x3->as_VMReg()         ,
	x3->as_VMReg()->next() ,
	f0->as_VMReg()          ,
	f0->as_VMReg()->next()  ,
	f1->as_VMReg()          ,
	f1->as_VMReg()->next()  ,
	f2->as_VMReg()          ,
	f2->as_VMReg()->next()  ,
	f3->as_VMReg()          ,
	f3->as_VMReg()->next()  ,
	f4->as_VMReg()          ,
	f4->as_VMReg()->next()  ,
	f5->as_VMReg()          ,
	f5->as_VMReg()->next()  ,
	f6->as_VMReg()          ,
	f6->as_VMReg()->next()  ,
	f7->as_VMReg()          ,
	f7->as_VMReg()->next()  ,
	f28->as_VMReg()         ,
	f28->as_VMReg()->next() ,
	f29->as_VMReg()         ,
	f29->as_VMReg()->next() ,
	f30->as_VMReg()         ,
	f30->as_VMReg()->next() ,
	f31->as_VMReg()         ,
	f31->as_VMReg()->next() ,
	f10->as_VMReg()         ,
	f10->as_VMReg()->next() ,
	f11->as_VMReg()         ,
	f11->as_VMReg()->next() ,
	f12->as_VMReg()         ,
	f12->as_VMReg()->next() ,
	f13->as_VMReg()         ,
	f13->as_VMReg()->next() ,
	f14->as_VMReg()         ,
	f14->as_VMReg()->next() ,
	f15->as_VMReg()         ,
	f15->as_VMReg()->next() ,
	f16->as_VMReg()         ,
	f16->as_VMReg()->next() ,
	f17->as_VMReg()         ,
	f17->as_VMReg()->next() ,
	f8->as_VMReg()          ,
	f8->as_VMReg()->next()  ,
	f9->as_VMReg()          ,
	f9->as_VMReg()->next()  ,
	f18->as_VMReg()         ,
	f18->as_VMReg()->next() ,
	f19->as_VMReg()         ,
	f19->as_VMReg()->next() ,
	f20->as_VMReg()         ,
	f20->as_VMReg()->next() ,
	f21->as_VMReg()         ,
	f21->as_VMReg()->next() ,
	f22->as_VMReg()         ,
	f22->as_VMReg()->next() ,
	f23->as_VMReg()         ,
	f23->as_VMReg()->next() ,
	f24->as_VMReg()         ,
	f24->as_VMReg()->next() ,
	f25->as_VMReg()         ,
	f25->as_VMReg()->next() ,
	f26->as_VMReg()         ,
	f26->as_VMReg()->next() ,
	f27->as_VMReg()         ,
	f27->as_VMReg()->next() ,
	x6->as_VMReg()         // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x7',  // R7
  (unsigned char)'\x7',  // R7_H
  (unsigned char)'\x1C',  // R28
  (unsigned char)'\x1C',  // R28_H
  (unsigned char)'\x1D',  // R29
  (unsigned char)'\x1D',  // R29_H
  (unsigned char)'\x1E',  // R30
  (unsigned char)'\x1E',  // R30_H
  (unsigned char)'\x1F',  // R31
  (unsigned char)'\x1F',  // R31_H
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x10',  // R16
  (unsigned char)'\x10',  // R16_H
  (unsigned char)'\x11',  // R17
  (unsigned char)'\x11',  // R17_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x9',  // R9_H
  (unsigned char)'\x12',  // R18
  (unsigned char)'\x12',  // R18_H
  (unsigned char)'\x13',  // R19
  (unsigned char)'\x13',  // R19_H
  (unsigned char)'\x14',  // R20
  (unsigned char)'\x14',  // R20_H
  (unsigned char)'\x15',  // R21
  (unsigned char)'\x15',  // R21_H
  (unsigned char)'\x16',  // R22
  (unsigned char)'\x16',  // R22_H
  (unsigned char)'\x18',  // R24
  (unsigned char)'\x18',  // R24_H
  (unsigned char)'\x19',  // R25
  (unsigned char)'\x19',  // R25_H
  (unsigned char)'\x1A',  // R26
  (unsigned char)'\x1A',  // R26_H
  (unsigned char)'\x17',  // R23
  (unsigned char)'\x17',  // R23_H
  (unsigned char)'\x1B',  // R27
  (unsigned char)'\x1B',  // R27_H
  (unsigned char)'\x4',  // R4
  (unsigned char)'\x4',  // R4_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x8',  // R8_H
  (unsigned char)'\x0',  // R0
  (unsigned char)'\x0',  // R0_H
  (unsigned char)'\x1',  // R1
  (unsigned char)'\x1',  // R1_H
  (unsigned char)'\x2',  // R2
  (unsigned char)'\x2',  // R2_H
  (unsigned char)'\x3',  // R3
  (unsigned char)'\x3',  // R3_H
  (unsigned char)'\x0',  // F0
  (unsigned char)'\x0',  // F0_H
  (unsigned char)'\x1',  // F1
  (unsigned char)'\x1',  // F1_H
  (unsigned char)'\x2',  // F2
  (unsigned char)'\x2',  // F2_H
  (unsigned char)'\x3',  // F3
  (unsigned char)'\x3',  // F3_H
  (unsigned char)'\x4',  // F4
  (unsigned char)'\x4',  // F4_H
  (unsigned char)'\x5',  // F5
  (unsigned char)'\x5',  // F5_H
  (unsigned char)'\x6',  // F6
  (unsigned char)'\x6',  // F6_H
  (unsigned char)'\x7',  // F7
  (unsigned char)'\x7',  // F7_H
  (unsigned char)'\x1C',  // F28
  (unsigned char)'\x1C',  // F28_H
  (unsigned char)'\x1D',  // F29
  (unsigned char)'\x1D',  // F29_H
  (unsigned char)'\x1E',  // F30
  (unsigned char)'\x1E',  // F30_H
  (unsigned char)'\x1F',  // F31
  (unsigned char)'\x1F',  // F31_H
  (unsigned char)'\xA',  // F10
  (unsigned char)'\xA',  // F10_H
  (unsigned char)'\xB',  // F11
  (unsigned char)'\xB',  // F11_H
  (unsigned char)'\xC',  // F12
  (unsigned char)'\xC',  // F12_H
  (unsigned char)'\xD',  // F13
  (unsigned char)'\xD',  // F13_H
  (unsigned char)'\xE',  // F14
  (unsigned char)'\xE',  // F14_H
  (unsigned char)'\xF',  // F15
  (unsigned char)'\xF',  // F15_H
  (unsigned char)'\x10',  // F16
  (unsigned char)'\x10',  // F16_H
  (unsigned char)'\x11',  // F17
  (unsigned char)'\x11',  // F17_H
  (unsigned char)'\x8',  // F8
  (unsigned char)'\x8',  // F8_H
  (unsigned char)'\x9',  // F9
  (unsigned char)'\x9',  // F9_H
  (unsigned char)'\x12',  // F18
  (unsigned char)'\x12',  // F18_H
  (unsigned char)'\x13',  // F19
  (unsigned char)'\x13',  // F19_H
  (unsigned char)'\x14',  // F20
  (unsigned char)'\x14',  // F20_H
  (unsigned char)'\x15',  // F21
  (unsigned char)'\x15',  // F21_H
  (unsigned char)'\x16',  // F22
  (unsigned char)'\x16',  // F22_H
  (unsigned char)'\x17',  // F23
  (unsigned char)'\x17',  // F23_H
  (unsigned char)'\x18',  // F24
  (unsigned char)'\x18',  // F24_H
  (unsigned char)'\x19',  // F25
  (unsigned char)'\x19',  // F25_H
  (unsigned char)'\x1A',  // F26
  (unsigned char)'\x1A',  // F26_H
  (unsigned char)'\x1B',  // F27
  (unsigned char)'\x1B',  // F27_H
  (unsigned char)'\x6' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegINoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegI_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R10_REG_mask();
}

const RegMask *iRegI_R12Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R12_REG_mask();
}

const RegMask *iRegI_R13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R13_REG_mask();
}

const RegMask *iRegI_R14Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R14_REG_mask();
}

const RegMask *iRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *iRegLNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG_mask();
}

const RegMask *iRegL_R28Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R28_REG_mask();
}

const RegMask *iRegL_R29Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R29_REG_mask();
}

const RegMask *iRegL_R30Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R30_REG_mask();
}

const RegMask *iRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *iRegPNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_PTR_REG_mask();
}

const RegMask *iRegP_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *iRegP_R11Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R11_REG_mask();
}

const RegMask *iRegP_R12Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R12_REG_mask();
}

const RegMask *iRegP_R13Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R13_REG_mask();
}

const RegMask *iRegP_R14Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R14_REG_mask();
}

const RegMask *iRegP_R15Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R15_REG_mask();
}

const RegMask *iRegP_R16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R16_REG_mask();
}

const RegMask *iRegP_R28Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R28_REG_mask();
}

const RegMask *iRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegNNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegIHeapbaseOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &HEAPBASE_REG_mask();
}

const RegMask *iRegL_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *fRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask();
}

const RegMask *fRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask();
}

const RegMask *javaThread_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &JAVA_THREAD_REG_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indirectNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffINOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffLNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *thread_anchor_pcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *iRegL2IOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &REG_FLAGS_mask();
}

const RegMask *inline_cache_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

//------------------Define members for classes derived from MachNode----------
// Build short branch version of this instruction
MachNode *far_cmpI_branchNode::short_branch_version() {
  cmpI_branchNode *node = new cmpI_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_loopNode::short_branch_version() {
  cmpI_loopNode *node = new cmpI_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpU_branchNode::short_branch_version() {
  cmpU_branchNode *node = new cmpU_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpU_loopNode::short_branch_version() {
  cmpU_loopNode *node = new cmpU_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_branchNode::short_branch_version() {
  cmpL_branchNode *node = new cmpL_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpLloopNode::short_branch_version() {
  cmpL_loopNode *node = new cmpL_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUL_branchNode::short_branch_version() {
  cmpUL_branchNode *node = new cmpUL_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUL_loopNode::short_branch_version() {
  cmpUL_loopNode *node = new cmpUL_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_branchNode::short_branch_version() {
  cmpP_branchNode *node = new cmpP_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_loopNode::short_branch_version() {
  cmpP_loopNode *node = new cmpP_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_branchNode::short_branch_version() {
  cmpN_branchNode *node = new cmpN_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_loopNode::short_branch_version() {
  cmpN_loopNode *node = new cmpN_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpF_branchNode::short_branch_version() {
  cmpF_branchNode *node = new cmpF_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpF_loopNode::short_branch_version() {
  cmpF_loopNode *node = new cmpF_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpD_branchNode::short_branch_version() {
  cmpD_branchNode *node = new cmpD_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpD_loopNode::short_branch_version() {
  cmpD_loopNode *node = new cmpD_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_reg_imm0_branchNode::short_branch_version() {
  cmpI_reg_imm0_branchNode *node = new cmpI_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_reg_imm0_loopNode::short_branch_version() {
  cmpI_reg_imm0_loopNode *node = new cmpI_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUEqNeLeGt_imm0_branchNode::short_branch_version() {
  cmpUEqNeLeGt_reg_imm0_branchNode *node = new cmpUEqNeLeGt_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpUEqNeLeGt_reg_imm0_loopNode::short_branch_version() {
  cmpUEqNeLeGt_reg_imm0_loopNode *node = new cmpUEqNeLeGt_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_reg_imm0_branchNode::short_branch_version() {
  cmpL_reg_imm0_branchNode *node = new cmpL_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_reg_imm0_loopNode::short_branch_version() {
  cmpL_reg_imm0_loopNode *node = new cmpL_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpULEqNeLeGt_reg_imm0_branchNode::short_branch_version() {
  cmpULEqNeLeGt_reg_imm0_branchNode *node = new cmpULEqNeLeGt_reg_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpULEqNeLeGt_reg_imm0_loopNode::short_branch_version() {
  cmpULEqNeLeGt_reg_imm0_loopNode *node = new cmpULEqNeLeGt_reg_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_imm0_branchNode::short_branch_version() {
  cmpP_imm0_branchNode *node = new cmpP_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_imm0_loopNode::short_branch_version() {
  cmpP_imm0_loopNode *node = new cmpP_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_imm0_branchNode::short_branch_version() {
  cmpN_imm0_branchNode *node = new cmpN_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpN_imm0_loopNode::short_branch_version() {
  cmpN_imm0_loopNode *node = new cmpN_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_narrowOop_imm0_branchNode::short_branch_version() {
  cmpP_narrowOop_imm0_branchNode *node = new cmpP_narrowOop_imm0_branchNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpP_narrowOop_imm0_loopNode::short_branch_version() {
  cmpP_narrowOop_imm0_loopNode *node = new cmpP_narrowOop_imm0_loopNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
  // Do not increment node index counter, since node reuses my index
  Compile* C = Compile::current();
  C->set_unique(C->unique() - 1);
}

void branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpFlag_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpFlag_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpU_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpU_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpU_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpU_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpF_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpF_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpF_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpF_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpD_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpD_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpD_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpD_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpULEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpULEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpULEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpULEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpFlag_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpFlag_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpU_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpU_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpU_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpU_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpLloopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpLloopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUL_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUL_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUL_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUL_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpF_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpF_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpF_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpF_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpD_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpD_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpD_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpD_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUEqNeLeGt_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUEqNeLeGt_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpUEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpUEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULtGe_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULtGe_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULtGe_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULtGe_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULEqNeLeGt_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULLtGe_reg_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULLtGe_reg_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpULLtGe_reg_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpULLtGe_reg_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpN_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpN_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpP_narrowOop_imm0_loopNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpP_narrowOop_imm0_loopNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP0Node::reloc() const {
  return 1;
}

int loadConP1Node::reloc() const {
  return 1;
}

int loadConPollPageNode::reloc() const {
  return 1;
}

int loadByteMapBaseNode::reloc() const {
  return 1;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int storeimmCM0Node::reloc() const {
  return 2;
}

int storeimmCM0_orderedNode::reloc() const {
  return 2;
}

int storeBNode::reloc() const {
  return 3;
}

int storeimmB0Node::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 3;
}

int storeimmC0Node::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 3;
}

int storeimmI0Node::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeimmL0Node::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeimmP0Node::reloc() const {
  return 3;
}

int storeNNode::reloc() const {
  return 2;
}

int storeImmN0Node::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeNKlassNode::reloc() const {
  return 2;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int get_and_addINode::reloc() const {
  return 1;
}

int get_and_addI_no_resNode::reloc() const {
  return 1;
}

int get_and_addIAcqNode::reloc() const {
  return 1;
}

int get_and_addI_no_resAcqNode::reloc() const {
  return 1;
}

int addI_reg_regNode::reloc() const {
  return 2;
}

int addI_reg_immNode::reloc() const {
  return 1;
}

int subI_reg_regNode::reloc() const {
  return 2;
}

int subI_reg_immNode::reloc() const {
  return 1;
}

int negI_regNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 2;
}

int divINode::reloc() const {
  return 2;
}

int signExtractNode::reloc() const {
  return 1;
}

int modINode::reloc() const {
  return 2;
}

int lShiftI_reg_regNode::reloc() const {
  return 2;
}

int lShiftI_reg_immNode::reloc() const {
  return 1;
}

int urShiftI_reg_regNode::reloc() const {
  return 2;
}

int urShiftI_reg_immNode::reloc() const {
  return 1;
}

int rShiftI_reg_regNode::reloc() const {
  return 2;
}

int rShiftI_reg_immNode::reloc() const {
  return 1;
}

int lShiftL_reg_regNode::reloc() const {
  return 1;
}

int urShiftL_reg_regNode::reloc() const {
  return 1;
}

int rShiftL_reg_regNode::reloc() const {
  return 1;
}

int absI_regNode::reloc() const {
  return 1;
}

int bytes_reverse_intNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_shortNode::reloc() const {
  return 1;
}

int bytes_reverse_shortNode::reloc() const {
  return 1;
}

int convI2L_reg_regNode::reloc() const {
  return 1;
}

int convI2UL_reg_regNode::reloc() const {
  return 1;
}

int convI2F_reg_regNode::reloc() const {
  return 1;
}

int convI2D_reg_regNode::reloc() const {
  return 1;
}

int cmpLTMask_reg_zeroNode::reloc() const {
  return 1;
}

int cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_imm0_loopNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_loopNode::reloc() const {
  return 1;
}

int far_cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int far_cmpP_imm0_loopNode::reloc() const {
  return 1;
}

int far_cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int far_cmpP_narrowOop_imm0_loopNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int partialSubtypeCheckVsZeroNode::reloc() const {
  return 1;
}

int safePointNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int TailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int convB2I_reg_reg_bNode::reloc() const {
  return 1;
}

int convI2S_reg_reg_bNode::reloc() const {
  return 1;
}

int convS2UI_reg_reg_bNode::reloc() const {
  return 1;
}

int convI2UL_reg_reg_bNode::reloc() const {
  return 1;
}

int bytes_reverse_int_bNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_short_bNode::reloc() const {
  return 1;
}

int bytes_reverse_short_bNode::reloc() const {
  return 1;
}

int countLeadingZerosI_bNode::reloc() const {
  return 1;
}

int countTrailingZerosI_bNode::reloc() const {
  return 1;
}

int popCountI_bNode::reloc() const {
  return 1;
}

int convI2Bool_andI_reg_immIpowerOf2Node::reloc() const {
  return 1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4108 "src/hotspot/cpu/riscv/riscv.ad"

    __ lb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6817 "ad_riscv.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4123 "src/hotspot/cpu/riscv/riscv.ad"

    __ lb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6833 "ad_riscv.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4138 "src/hotspot/cpu/riscv/riscv.ad"

    __ lbu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6849 "ad_riscv.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4153 "src/hotspot/cpu/riscv/riscv.ad"

    __ lbu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6865 "ad_riscv.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4168 "src/hotspot/cpu/riscv/riscv.ad"

    __ lh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6881 "ad_riscv.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4183 "src/hotspot/cpu/riscv/riscv.ad"

    __ lh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6897 "ad_riscv.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4198 "src/hotspot/cpu/riscv/riscv.ad"

    __ lhu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6913 "ad_riscv.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4213 "src/hotspot/cpu/riscv/riscv.ad"

    __ lhu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6929 "ad_riscv.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4228 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6946 "ad_riscv.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4244 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6963 "ad_riscv.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 4260 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6980 "ad_riscv.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4275 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 6997 "ad_riscv.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4291 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7013 "ad_riscv.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4306 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7030 "ad_riscv.cpp"
  }
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4322 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7046 "ad_riscv.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4337 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7063 "ad_riscv.cpp"
  }
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4353 "src/hotspot/cpu/riscv/riscv.ad"

    __ lwu(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7079 "ad_riscv.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4368 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7095 "ad_riscv.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4383 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7112 "ad_riscv.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1785 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    int64_t con = (int64_t)opnd_array(1)->constant();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, con);
  
#line 7131 "ad_riscv.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1785 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    int64_t con = (int64_t)opnd_array(1)->constantL();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, con);
  
#line 7150 "ad_riscv.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1793 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL || con == (address)1) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      if (rtype == relocInfo::oop_type) {
        __ movoop(dst_reg, (jobject)con, /*immediate*/true);
      } else if (rtype == relocInfo::metadata_type) {
        __ mov_metadata(dst_reg, (Metadata*)con);
      } else {
        assert(rtype == relocInfo::none, "unexpected reloc type");
        __ li(dst_reg, opnd_array(1)->constant());
      }
    }
  
#line 7180 "ad_riscv.cpp"
  }
}

void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1848 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mv(dst_reg, zr);
  
#line 7197 "ad_riscv.cpp"
  }
}

void loadConP1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1812 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ li(dst_reg, 1);
  
#line 7215 "ad_riscv.cpp"
  }
}

void loadConPollPageNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1819 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    int32_t offset = 0;
    address page = (address)opnd_array(1)->constant();
    unsigned long align = (unsigned long)page & 0xfff;
    assert(align == 0, "polling page must be page aligned");
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ la_patchable(dst_reg, Address(page, relocInfo::poll_type), offset);
    __ addi(dst_reg, dst_reg, offset);
  
#line 7237 "ad_riscv.cpp"
  }
}

void loadByteMapBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1830 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ load_byte_map_base(opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 7253 "ad_riscv.cpp"
  }
}

void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1835 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::oop_type, "unexpected reloc type");
      __ set_narrow_oop(dst_reg, (jobject)con);
    }
  
#line 7277 "ad_riscv.cpp"
  }
}

void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1848 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mv(dst_reg, zr);
  
#line 7294 "ad_riscv.cpp"
  }
}

void loadConNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 1854 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::metadata_type, "unexpected reloc type");
      __ set_narrow_klass(dst_reg, (Klass *)con);
    }
  
#line 7318 "ad_riscv.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 4529 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 7334 "ad_riscv.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 4530 "src/hotspot/cpu/riscv/riscv.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 7343 "ad_riscv.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 4543 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_w_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), zr);
  
#line 7358 "ad_riscv.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 4559 "src/hotspot/cpu/riscv/riscv.ad"

    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 7374 "ad_riscv.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 4560 "src/hotspot/cpu/riscv/riscv.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 7383 "ad_riscv.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 4573 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_d_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), zr);
  
#line 7398 "ad_riscv.cpp"
  }
}

void storeimmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4591 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7415 "ad_riscv.cpp"
  }
}

void storeimmCM0_orderedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4608 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7433 "ad_riscv.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4624 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7450 "ad_riscv.cpp"
  }
}

void storeimmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4638 "src/hotspot/cpu/riscv/riscv.ad"

    __ sb(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7467 "ad_riscv.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4653 "src/hotspot/cpu/riscv/riscv.ad"

    __ sh(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7484 "ad_riscv.cpp"
  }
}

void storeimmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4667 "src/hotspot/cpu/riscv/riscv.ad"

    __ sh(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7501 "ad_riscv.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4682 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7519 "ad_riscv.cpp"
  }
}

void storeimmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4697 "src/hotspot/cpu/riscv/riscv.ad"

    __ sw(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7536 "ad_riscv.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4712 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7554 "ad_riscv.cpp"
  }
}

void storeimmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4728 "src/hotspot/cpu/riscv/riscv.ad"

    __ sd(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7571 "ad_riscv.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4743 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7589 "ad_riscv.cpp"
  }
}

void storeimmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4759 "src/hotspot/cpu/riscv/riscv.ad"

    __ sd(zr, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7606 "ad_riscv.cpp"
  }
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4774 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7624 "ad_riscv.cpp"
  }
}

void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 4791 "src/hotspot/cpu/riscv/riscv.ad"

    __ sw(as_Register(R27_enc), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7641 "ad_riscv.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4806 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsw(as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7658 "ad_riscv.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4821 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fsd(as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7676 "ad_riscv.cpp"
  }
}

void storeNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4837 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
  
#line 7694 "ad_riscv.cpp"
  }
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4880 "src/hotspot/cpu/riscv/riscv.ad"

    __ la(t0, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
    __ lr_d(opnd_array(0)->as_Register(ra_,this)/* dst */, t0, Assembler::aq);
  
#line 7711 "ad_riscv.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 4901 "src/hotspot/cpu/riscv/riscv.ad"

    __ la(t0, Address(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->disp(ra_,this,idx1)));
    __ sc_d(opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, t0, Assembler::rl);
  
#line 7730 "ad_riscv.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 4924 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* cr */);
    __ xorr(opnd_array(0)->as_Register(ra_,this)/* cr */,opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
  
#line 7750 "ad_riscv.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 4946 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* cr */);
    __ xorr(opnd_array(0)->as_Register(ra_,this)/* cr */,opnd_array(0)->as_Register(ra_,this)/* cr */, opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */);
  
#line 7770 "ad_riscv.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 4971 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 7794 "ad_riscv.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 4994 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            Assembler::relaxed /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 7818 "ad_riscv.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1867 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7838 "ad_riscv.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1881 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7858 "ad_riscv.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1881 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7878 "ad_riscv.cpp"
  }
}

void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1874 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7898 "ad_riscv.cpp"
  }
}

void compareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5084 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 7922 "ad_riscv.cpp"
  }
}

void compareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5109 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            Assembler::aq /* acquire */, Assembler::rl /* release */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            true /* result as bool */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 7946 "ad_riscv.cpp"
  }
}

void compareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1888 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7966 "ad_riscv.cpp"
  }
}

void compareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1902 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 7986 "ad_riscv.cpp"
  }
}

void compareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1902 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 8006 "ad_riscv.cpp"
  }
}

void compareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 1895 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */,
               /*result as bool*/ true);
  
#line 8026 "ad_riscv.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5209 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 8050 "ad_riscv.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5231 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 8074 "ad_riscv.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5252 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8094 "ad_riscv.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5272 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8114 "ad_riscv.cpp"
  }
}

void compareAndExchangeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5292 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8134 "ad_riscv.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5312 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8154 "ad_riscv.cpp"
  }
}

void compareAndExchangeBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5335 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                            /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 8178 "ad_riscv.cpp"
  }
}

void compareAndExchangeSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5359 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                            /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                            /*result_as_bool*/ false, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
  
#line 8202 "ad_riscv.cpp"
  }
}

void compareAndExchangeIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5382 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8222 "ad_riscv.cpp"
  }
}

void compareAndExchangeLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5404 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8242 "ad_riscv.cpp"
  }
}

void compareAndExchangeNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5426 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8262 "ad_riscv.cpp"
  }
}

void compareAndExchangePAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  {
    MacroAssembler _masm(&cbuf);

#line 5448 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
               /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */);
  
#line 8282 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5470 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                                 /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 8307 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5494 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                                 /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 8332 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5515 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8352 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5535 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8372 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5555 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8392 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5575 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::relaxed, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8412 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapBAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5600 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int8,
                                 /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 8437 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapSAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// res
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  {
    MacroAssembler _masm(&cbuf);

#line 5626 "src/hotspot/cpu/riscv/riscv.ad"

    __ weak_cmpxchg_narrow_value(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int16,
                                 /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(4)->as_Register(ra_,this,idx4)/* res */,
                                 opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */);
    __ xori(opnd_array(4)->as_Register(ra_,this,idx4)/* res */, opnd_array(4)->as_Register(ra_,this,idx4)/* res */, 1);
  
#line 8462 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5649 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int32,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8482 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5671 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8502 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5693 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::uint32,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8522 "ad_riscv.cpp"
  }
}

void weakCompareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 5715 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmpxchg_weak(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */, Assembler::int64,
                    /*acquire*/ Assembler::aq, /*release*/ Assembler::rl, opnd_array(0)->as_Register(ra_,this)/* res */);
    __ xori(opnd_array(0)->as_Register(ra_,this)/* res */, opnd_array(0)->as_Register(ra_,this)/* res */, 1);
  
#line 8542 "ad_riscv.cpp"
  }
}

void get_and_setINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5732 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8559 "ad_riscv.cpp"
  }
}

void get_and_setLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5747 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8576 "ad_riscv.cpp"
  }
}

void get_and_setNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5762 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgwu(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8593 "ad_riscv.cpp"
  }
}

void get_and_setPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5777 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8610 "ad_riscv.cpp"
  }
}

void get_and_setIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5794 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgalw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8627 "ad_riscv.cpp"
  }
}

void get_and_setLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5811 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8644 "ad_riscv.cpp"
  }
}

void get_and_setNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5828 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgalwu(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8661 "ad_riscv.cpp"
  }
}

void get_and_setPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 5845 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_xchgal(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8678 "ad_riscv.cpp"
  }
}

void get_and_addLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5860 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8695 "ad_riscv.cpp"
  }
}

void get_and_addL_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5877 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8712 "ad_riscv.cpp"
  }
}

void get_and_addLiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5892 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8729 "ad_riscv.cpp"
  }
}

void get_and_addLi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5909 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_add(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8746 "ad_riscv.cpp"
  }
}

void get_and_addINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5924 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8763 "ad_riscv.cpp"
  }
}

void get_and_addI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5941 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8780 "ad_riscv.cpp"
  }
}

void get_and_addIiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5956 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8797 "ad_riscv.cpp"
  }
}

void get_and_addIi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5973 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8814 "ad_riscv.cpp"
  }
}

void get_and_addLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 5990 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8831 "ad_riscv.cpp"
  }
}

void get_and_addL_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6006 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8848 "ad_riscv.cpp"
  }
}

void get_and_addLiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6023 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8865 "ad_riscv.cpp"
  }
}

void get_and_addLi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6040 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addal(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8882 "ad_riscv.cpp"
  }
}

void get_and_addIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6057 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8899 "ad_riscv.cpp"
  }
}

void get_and_addI_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6074 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8916 "ad_riscv.cpp"
  }
}

void get_and_addIiAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6091 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8933 "ad_riscv.cpp"
  }
}

void get_and_addIi_no_resAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 6108 "src/hotspot/cpu/riscv/riscv.ad"

    __ atomic_addalw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 8950 "ad_riscv.cpp"
  }
}

void addI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6132 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 8970 "ad_riscv.cpp"
  }
}

void addI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6148 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    int32_t con = (int32_t)opnd_array(2)->constant();
    __ addiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             opnd_array(2)->constant());
  
#line 8991 "ad_riscv.cpp"
  }
}

void addI_reg_imm_l2iNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6165 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ addiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             opnd_array(2)->constant());
  
#line 9011 "ad_riscv.cpp"
  }
}

void addP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6182 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9031 "ad_riscv.cpp"
  }
}

void lShiftL_regI_immGE32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scale
  {
    MacroAssembler _masm(&cbuf);

#line 6198 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ slli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), opnd_array(2)->constant()& 63);
  
#line 9049 "ad_riscv.cpp"
  }
}

void addP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6214 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 9070 "ad_riscv.cpp"
  }
}

void addL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6231 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9090 "ad_riscv.cpp"
  }
}

void addL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6247 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 9111 "ad_riscv.cpp"
  }
}

void subI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6265 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9131 "ad_riscv.cpp"
  }
}

void subI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6282 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addiw
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant());
  
#line 9152 "ad_riscv.cpp"
  }
}

void subL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6299 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9172 "ad_riscv.cpp"
  }
}

void subL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6315 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // src2 is imm, so actually call the addi
    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constantL());
  
#line 9193 "ad_riscv.cpp"
  }
}

void negI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6333 "src/hotspot/cpu/riscv/riscv.ad"

    // actually call the subw
    __ negw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 9212 "ad_riscv.cpp"
  }
}

void negL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6349 "src/hotspot/cpu/riscv/riscv.ad"

    // actually call the sub
    __ neg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 9231 "ad_riscv.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6366 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv64 mulw will sign-extension to high 32 bits in dst reg
    __ mulw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9251 "ad_riscv.cpp"
  }
}

void mulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6383 "src/hotspot/cpu/riscv/riscv.ad"

    __ mul(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9270 "ad_riscv.cpp"
  }
}

void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6398 "src/hotspot/cpu/riscv/riscv.ad"

    __ mulh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9289 "ad_riscv.cpp"
  }
}

void divINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2199 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, false);
  
#line 9309 "ad_riscv.cpp"
  }
}

void signExtractNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 6423 "src/hotspot/cpu/riscv/riscv.ad"

    __ srliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 31);
  
#line 9327 "ad_riscv.cpp"
  }
}

void divLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2207 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, false);
  
#line 9347 "ad_riscv.cpp"
  }
}

void signExtractLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 6445 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 63);
  
#line 9366 "ad_riscv.cpp"
  }
}

void modINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2215 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, true);
  
#line 9386 "ad_riscv.cpp"
  }
}

void modLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2223 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, true);
  
#line 9406 "ad_riscv.cpp"
  }
}

void lShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6483 "src/hotspot/cpu/riscv/riscv.ad"

    __ sllw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9425 "ad_riscv.cpp"
  }
}

void lShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6498 "src/hotspot/cpu/riscv/riscv.ad"

    // the shift amount is encoded in the lower
    // 5 bits of the I-immediate field for RV32I
    __ slliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 9446 "ad_riscv.cpp"
  }
}

void urShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6516 "src/hotspot/cpu/riscv/riscv.ad"

    __ srlw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9465 "ad_riscv.cpp"
  }
}

void urShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6531 "src/hotspot/cpu/riscv/riscv.ad"

    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srliw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 9486 "ad_riscv.cpp"
  }
}

void rShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6549 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv will sign-ext dst high 32 bits
    __ sraw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9506 "ad_riscv.cpp"
  }
}

void rShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6565 "src/hotspot/cpu/riscv/riscv.ad"

    // riscv will sign-ext dst high 32 bits
    __ sraiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             (unsigned) opnd_array(2)->constant()& 0x1f);
  
#line 9526 "ad_riscv.cpp"
  }
}

void lShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6585 "src/hotspot/cpu/riscv/riscv.ad"

    __ sll(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9545 "ad_riscv.cpp"
  }
}

void lShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6601 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ slli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 9567 "ad_riscv.cpp"
  }
}

void urShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6621 "src/hotspot/cpu/riscv/riscv.ad"

    __ srl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9586 "ad_riscv.cpp"
  }
}

void urShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6637 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 9608 "ad_riscv.cpp"
  }
}

void urShiftP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6656 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srli(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 9630 "ad_riscv.cpp"
  }
}

void rShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6676 "src/hotspot/cpu/riscv/riscv.ad"

    __ sra(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9649 "ad_riscv.cpp"
  }
}

void rShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6692 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    // the shift amount is encoded in the lower
    // 6 bits of the I-immediate field for RV64I
    __ srai(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned) opnd_array(2)->constant()& 0x3f);
  
#line 9671 "ad_riscv.cpp"
  }
}

void regI_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 6709 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), -1);
  
#line 9688 "ad_riscv.cpp"
  }
}

void regL_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 6721 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), -1);
  
#line 9705 "ad_riscv.cpp"
  }
}

void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6738 "src/hotspot/cpu/riscv/riscv.ad"

    __ fadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9724 "ad_riscv.cpp"
  }
}

void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6753 "src/hotspot/cpu/riscv/riscv.ad"

    __ fadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9743 "ad_riscv.cpp"
  }
}

void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6768 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9762 "ad_riscv.cpp"
  }
}

void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6783 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9781 "ad_riscv.cpp"
  }
}

void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6798 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmul_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9800 "ad_riscv.cpp"
  }
}

void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6813 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmul_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 9819 "ad_riscv.cpp"
  }
}

void maddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6830 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9840 "ad_riscv.cpp"
  }
}

void maddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6848 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9861 "ad_riscv.cpp"
  }
}

void msubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6866 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9882 "ad_riscv.cpp"
  }
}

void msubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6884 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
               as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9903 "ad_riscv.cpp"
  }
}

void nmsubF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6903 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9924 "ad_riscv.cpp"
  }
}

void nmsubF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6903 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9945 "ad_riscv.cpp"
  }
}

void nmsubD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6922 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9966 "ad_riscv.cpp"
  }
}

void nmsubD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6922 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmsub_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 9987 "ad_riscv.cpp"
  }
}

void nmaddF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6941 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 10008 "ad_riscv.cpp"
  }
}

void nmaddF_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6941 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 10029 "ad_riscv.cpp"
  }
}

void nmaddD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6960 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 10050 "ad_riscv.cpp"
  }
}

void nmaddD_reg_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 6960 "src/hotspot/cpu/riscv/riscv.ad"

    __ fnmadd_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
                as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
                as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
                as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 10071 "ad_riscv.cpp"
  }
}

void maxF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 6977 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 false /* is_double */, false /* is_min */);
  
#line 10091 "ad_riscv.cpp"
  }
}

void minF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 6993 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 false /* is_double */, true /* is_min */);
  
#line 10111 "ad_riscv.cpp"
  }
}

void maxD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 7009 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 true /* is_double */, false /* is_min */);
  
#line 10131 "ad_riscv.cpp"
  }
}

void minD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 7025 "src/hotspot/cpu/riscv/riscv.ad"

    __ minmax_FD(as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* dst */),
                 as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
                 true /* is_double */, true /* is_min */);
  
#line 10151 "ad_riscv.cpp"
  }
}

void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7040 "src/hotspot/cpu/riscv/riscv.ad"

    __ fdiv_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10170 "ad_riscv.cpp"
  }
}

void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7055 "src/hotspot/cpu/riscv/riscv.ad"

    __ fdiv_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10189 "ad_riscv.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7070 "src/hotspot/cpu/riscv/riscv.ad"

    __ fneg_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10206 "ad_riscv.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7084 "src/hotspot/cpu/riscv/riscv.ad"

    __ fneg_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10223 "ad_riscv.cpp"
  }
}

void absI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7102 "src/hotspot/cpu/riscv/riscv.ad"

    __ sraiw(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 0x1f);
    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 10241 "ad_riscv.cpp"
  }
}

void absL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7121 "src/hotspot/cpu/riscv/riscv.ad"

    __ srai(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 0x3f);
    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 10259 "ad_riscv.cpp"
  }
}

void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7135 "src/hotspot/cpu/riscv/riscv.ad"

    __ fabs_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10276 "ad_riscv.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7148 "src/hotspot/cpu/riscv/riscv.ad"

    __ fabs_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10293 "ad_riscv.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7161 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsqrt_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10310 "ad_riscv.cpp"
  }
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7174 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsqrt_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
               as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10327 "ad_riscv.cpp"
  }
}

void andI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7194 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10347 "ad_riscv.cpp"
  }
}

void andI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7211 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andi(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constant()));
  
#line 10367 "ad_riscv.cpp"
  }
}

void orI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7228 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10387 "ad_riscv.cpp"
  }
}

void orI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7245 "src/hotspot/cpu/riscv/riscv.ad"

    __ ori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (int32_t)(opnd_array(2)->constant()));
  
#line 10406 "ad_riscv.cpp"
  }
}

void xorI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7261 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10426 "ad_riscv.cpp"
  }
}

void xorI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7278 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constant()));
  
#line 10445 "ad_riscv.cpp"
  }
}

void andL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7294 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10465 "ad_riscv.cpp"
  }
}

void andL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7311 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ andi(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constantL()));
  
#line 10485 "ad_riscv.cpp"
  }
}

void orL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7328 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10505 "ad_riscv.cpp"
  }
}

void orL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7345 "src/hotspot/cpu/riscv/riscv.ad"

    __ ori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (int32_t)(opnd_array(2)->constantL()));
  
#line 10524 "ad_riscv.cpp"
  }
}

void xorL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7361 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ xorr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 10544 "ad_riscv.cpp"
  }
}

void xorL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 7378 "src/hotspot/cpu/riscv/riscv.ad"

    __ xori(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (int32_t)(opnd_array(2)->constantL()));
  
#line 10563 "ad_riscv.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 7397 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_w_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10580 "ad_riscv.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 7411 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10597 "ad_riscv.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7424 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_h_h_u(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10613 "ad_riscv.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7437 "src/hotspot/cpu/riscv/riscv.ad"

    __ revb_h_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10629 "ad_riscv.cpp"
  }
}

void load_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7453 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadLoad | MacroAssembler::LoadStore);
  
#line 10645 "ad_riscv.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7466 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_acquire");
    __ membar(MacroAssembler::LoadLoad | MacroAssembler::LoadStore);
  
#line 10662 "ad_riscv.cpp"
  }
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7480 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_acquire_lock (elided)");
  
#line 10678 "ad_riscv.cpp"
  }
}

void store_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7493 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
  
#line 10694 "ad_riscv.cpp"
  }
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7506 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_release");
    __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
  
#line 10711 "ad_riscv.cpp"
  }
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7519 "src/hotspot/cpu/riscv/riscv.ad"

    __ membar(MacroAssembler::StoreStore);
  
#line 10727 "ad_riscv.cpp"
  }
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7531 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_release_lock (elided)");
  
#line 10743 "ad_riscv.cpp"
  }
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 7545 "src/hotspot/cpu/riscv/riscv.ad"

    __ block_comment("membar_volatile");
    __ membar(MacroAssembler::StoreLoad);
  
#line 10760 "ad_riscv.cpp"
  }
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7562 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 10779 "ad_riscv.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7578 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 10798 "ad_riscv.cpp"
  }
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void convI2BoolNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7632 "src/hotspot/cpu/riscv/riscv.ad"

    __ snez(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10853 "ad_riscv.cpp"
  }
}

void convP2BoolNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7647 "src/hotspot/cpu/riscv/riscv.ad"

    __ snez(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10869 "ad_riscv.cpp"
  }
}

void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7662 "src/hotspot/cpu/riscv/riscv.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
  
#line 10885 "ad_riscv.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7674 "src/hotspot/cpu/riscv/riscv.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
  
#line 10901 "ad_riscv.cpp"
  }
}

void convI2UL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 7689 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ zero_extend(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 32);
  
#line 10919 "ad_riscv.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7705 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_s(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10935 "ad_riscv.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7718 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_d(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10951 "ad_riscv.cpp"
  }
}

void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7733 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_w_s_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 10967 "ad_riscv.cpp"
  }
}

void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7746 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_w(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 10983 "ad_riscv.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7761 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_l_s_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 10999 "ad_riscv.cpp"
  }
}

void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7774 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_s_l(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11015 "ad_riscv.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7789 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_w_d_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 11031 "ad_riscv.cpp"
  }
}

void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7802 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_w(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11047 "ad_riscv.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7817 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_l_d_safe(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_FloatRegister(ra_,this,idx1)/* src */);
  
#line 11063 "ad_riscv.cpp"
  }
}

void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7830 "src/hotspot/cpu/riscv/riscv.ad"

    __ fcvt_d_l(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11079 "ad_riscv.cpp"
  }
}

void convP2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7844 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ zero_extend(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 32);
  
#line 11096 "ad_riscv.cpp"
  }
}

void convN2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7862 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ mv(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 11113 "ad_riscv.cpp"
  }
}

void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7875 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ encode_heap_oop(d, s);
  
#line 11131 "ad_riscv.cpp"
  }
}

void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7890 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop(d, s);
  
#line 11149 "ad_riscv.cpp"
  }
}

void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7905 "src/hotspot/cpu/riscv/riscv.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop_not_null(d, s);
  
#line 11167 "ad_riscv.cpp"
  }
}

void encodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7920 "src/hotspot/cpu/riscv/riscv.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ encode_klass_not_null(dst_reg, src_reg, t0);
  
#line 11185 "ad_riscv.cpp"
  }
}

void decodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 7937 "src/hotspot/cpu/riscv/riscv.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register tmp_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    __ decode_klass_not_null(dst_reg, src_reg, tmp_reg);
  
#line 11205 "ad_riscv.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7959 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ lw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 11222 "ad_riscv.cpp"
  }
}

void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7978 "src/hotspot/cpu/riscv/riscv.ad"

    __ flw(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 11238 "ad_riscv.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 7996 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ ld(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 11255 "ad_riscv.cpp"
  }
}

void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8015 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 11272 "ad_riscv.cpp"
  }
}

void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8034 "src/hotspot/cpu/riscv/riscv.ad"

    __ fsw(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 11288 "ad_riscv.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8052 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 11305 "ad_riscv.cpp"
  }
}

void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8071 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ fsd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 11322 "ad_riscv.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8090 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    __ sd(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 11339 "ad_riscv.cpp"
  }
}

void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8109 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_x_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11355 "ad_riscv.cpp"
  }
}

void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8127 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_w_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11371 "ad_riscv.cpp"
  }
}

void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8145 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_x_d(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11387 "ad_riscv.cpp"
  }
}

void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8163 "src/hotspot/cpu/riscv/riscv.ad"

    __ fmv_d_x(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 11403 "ad_riscv.cpp"
  }
}

void cmpF3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 8184 "src/hotspot/cpu/riscv/riscv.ad"

    // we want -1 for unordered or less than, 0 for equal and 1 for greater than.
    __ float_compare(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* op1 */),
                     as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op2 */), -1 /*unordered_result < 0*/);
  
#line 11422 "ad_riscv.cpp"
  }
}

void cmpD3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 8204 "src/hotspot/cpu/riscv/riscv.ad"

    // we want -1 for unordered or less than, 0 for equal and 1 for greater than.
    __ double_compare(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* op1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op2 */), -1 /*unordered_result < 0*/);
  
#line 11440 "ad_riscv.cpp"
  }
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 8222 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_l2i(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */));
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), t0);
  
#line 11458 "ad_riscv.cpp"
  }
}

void cmpLTMask_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    MacroAssembler _masm(&cbuf);

#line 8240 "src/hotspot/cpu/riscv/riscv.ad"

    __ slt(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* p */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* q */));
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), zr, as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 11476 "ad_riscv.cpp"
  }
}

void cmpLTMask_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/riscv/riscv.ad"

    __ sraiw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op */), 31);
  
#line 11493 "ad_riscv.cpp"
  }
}

void minI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 8283 "src/hotspot/cpu/riscv/riscv.ad"

    Label Lsrc1, Ldone;
    __ ble(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ j(Ldone);
    __ bind(Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);
  
#line 11516 "ad_riscv.cpp"
  }
}

void maxI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 8312 "src/hotspot/cpu/riscv/riscv.ad"

    Label Lsrc1, Ldone;
    __ bge(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ j(Ldone);
    __ bind(Lsrc1);
    __ mv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */));
    __ bind(Ldone);

  
#line 11540 "ad_riscv.cpp"
  }
}

void branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1911 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(1)->label();
    __ j(*L);
  
#line 11557 "ad_riscv.cpp"
  }
}

void cmpFlag_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8356 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* cr */), *(opnd_array(3)->label()));
  
#line 11575 "ad_riscv.cpp"
  }
}

void cmpI_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8375 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11594 "ad_riscv.cpp"
  }
}

void cmpI_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8394 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11613 "ad_riscv.cpp"
  }
}

void cmpU_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8414 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11633 "ad_riscv.cpp"
  }
}

void cmpU_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8434 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11653 "ad_riscv.cpp"
  }
}

void cmpL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8455 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11672 "ad_riscv.cpp"
  }
}

void cmpL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8474 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11691 "ad_riscv.cpp"
  }
}

void cmpUL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8493 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11711 "ad_riscv.cpp"
  }
}

void cmpUL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8512 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11731 "ad_riscv.cpp"
  }
}

void cmpP_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8533 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11751 "ad_riscv.cpp"
  }
}

void cmpP_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8553 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11771 "ad_riscv.cpp"
  }
}

void cmpN_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8574 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11791 "ad_riscv.cpp"
  }
}

void cmpN_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8594 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11811 "ad_riscv.cpp"
  }
}

void cmpF_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8614 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11830 "ad_riscv.cpp"
  }
}

void cmpF_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8631 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11849 "ad_riscv.cpp"
  }
}

void cmpD_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8649 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11869 "ad_riscv.cpp"
  }
}

void cmpD_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8667 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()));
  
#line 11889 "ad_riscv.cpp"
  }
}

void cmpI_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8687 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 11908 "ad_riscv.cpp"
  }
}

void cmpI_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8706 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 11927 "ad_riscv.cpp"
  }
}

void cmpUEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8726 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 11946 "ad_riscv.cpp"
  }
}

void cmpUEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8746 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 11965 "ad_riscv.cpp"
  }
}

void cmpL_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8766 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 11984 "ad_riscv.cpp"
  }
}

void cmpL_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8785 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()));
  
#line 12003 "ad_riscv.cpp"
  }
}

void cmpULEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8805 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12022 "ad_riscv.cpp"
  }
}

void cmpULEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8824 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12041 "ad_riscv.cpp"
  }
}

void cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8841 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12060 "ad_riscv.cpp"
  }
}

void cmpP_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8857 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12079 "ad_riscv.cpp"
  }
}

void cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8875 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12098 "ad_riscv.cpp"
  }
}

void cmpN_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8892 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12117 "ad_riscv.cpp"
  }
}

void cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8909 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12136 "ad_riscv.cpp"
  }
}

void cmpP_narrowOop_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8925 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()));
  
#line 12155 "ad_riscv.cpp"
  }
}

void far_cmpFlag_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8942 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* cr */), *(opnd_array(3)->label()), /* is_far */ true);
  
#line 12173 "ad_riscv.cpp"
  }
}

void far_cmpI_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8960 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12192 "ad_riscv.cpp"
  }
}

void far_cmpI_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8974 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12211 "ad_riscv.cpp"
  }
}

void far_cmpU_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 8988 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12231 "ad_riscv.cpp"
  }
}

void far_cmpU_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9003 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12251 "ad_riscv.cpp"
  }
}

void far_cmpL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9018 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12270 "ad_riscv.cpp"
  }
}

void far_cmpLloopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9032 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12289 "ad_riscv.cpp"
  }
}

void far_cmpUL_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9046 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12309 "ad_riscv.cpp"
  }
}

void far_cmpUL_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9061 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12329 "ad_riscv.cpp"
  }
}

void far_cmpP_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9079 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12349 "ad_riscv.cpp"
  }
}

void far_cmpP_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9097 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12369 "ad_riscv.cpp"
  }
}

void far_cmpN_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9115 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                       as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12389 "ad_riscv.cpp"
  }
}

void far_cmpN_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9133 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                  as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12409 "ad_riscv.cpp"
  }
}

void far_cmpF_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9151 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                        *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12429 "ad_riscv.cpp"
  }
}

void far_cmpF_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9167 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode(), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                        *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12449 "ad_riscv.cpp"
  }
}

void far_cmpD_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9184 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12469 "ad_riscv.cpp"
  }
}

void far_cmpD_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9200 "src/hotspot/cpu/riscv/riscv.ad"

    __ float_cmp_branch(opnd_array(1)->ccode()| MacroAssembler::double_branch_mask, as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* op1 */),
                        as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* op2 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12489 "ad_riscv.cpp"
  }
}

void far_cmpI_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9218 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12508 "ad_riscv.cpp"
  }
}

void far_cmpI_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9235 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12527 "ad_riscv.cpp"
  }
}

void far_cmpUEqNeLeGt_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9252 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12546 "ad_riscv.cpp"
  }
}

void far_cmpUEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9270 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12565 "ad_riscv.cpp"
  }
}

void far_cmpULtGe_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 1917 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 12593 "ad_riscv.cpp"
  }
}

void far_cmpULtGe_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 1917 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 12621 "ad_riscv.cpp"
  }
}

void far_cmpL_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9318 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12640 "ad_riscv.cpp"
  }
}

void far_cmpL_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9335 "src/hotspot/cpu/riscv/riscv.ad"

    __ cmp_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), zr, *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12659 "ad_riscv.cpp"
  }
}

void far_cmpULEqNeLeGt_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9352 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12678 "ad_riscv.cpp"
  }
}

void far_cmpULEqNeLeGt_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9369 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpUEqNeLeGt_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12697 "ad_riscv.cpp"
  }
}

void far_cmpULLtGe_reg_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 1917 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 12725 "ad_riscv.cpp"
  }
}

void far_cmpULLtGe_reg_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {

#line 1917 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Label* L = opnd_array(4)->label();
    switch (opnd_array(1)->ccode()) {
      case(BoolTest::ge):
        __ j(*L);
        break;
      case(BoolTest::lt):
        break;
      default:
        Unimplemented();
    }
  
#line 12753 "ad_riscv.cpp"
  }
}

void far_cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9414 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12772 "ad_riscv.cpp"
  }
}

void far_cmpP_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9428 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12791 "ad_riscv.cpp"
  }
}

void far_cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9443 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12810 "ad_riscv.cpp"
  }
}

void far_cmpN_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9458 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12829 "ad_riscv.cpp"
  }
}

void far_cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9472 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12848 "ad_riscv.cpp"
  }
}

void far_cmpP_narrowOop_imm0_loopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lbl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9486 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmpEqNe_imm0_branch(opnd_array(1)->ccode(), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), *(opnd_array(4)->label()), /* is_far */ true);
  
#line 12867 "ad_riscv.cpp"
  }
}

void cmovI_cmpINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9503 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12889 "ad_riscv.cpp"
  }
}

void cmovI_cmpUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9520 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12911 "ad_riscv.cpp"
  }
}

void cmovI_cmpLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9537 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12933 "ad_riscv.cpp"
  }
}

void cmovI_cmpULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9554 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12955 "ad_riscv.cpp"
  }
}

void cmovL_cmpLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9571 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12977 "ad_riscv.cpp"
  }
}

void cmovL_cmpULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9588 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 12999 "ad_riscv.cpp"
  }
}

void cmovL_cmpINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9605 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode(),
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 13021 "ad_riscv.cpp"
  }
}

void cmovL_cmpUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9622 "src/hotspot/cpu/riscv/riscv.ad"

    __ enc_cmove(opnd_array(1)->ccode()| MacroAssembler::unsigned_branch_mask,
                 as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* op2 */),
                 as_Register(opnd_array(4)->reg(ra_,this,idx4)/* dst */), as_Register(opnd_array(5)->reg(ra_,this,idx5)/* src */));
  
#line 13043 "ad_riscv.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1960 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);

    address addr = (address)opnd_array(1)->method();
    address call = NULL;
    assert_cond(addr != NULL);
    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      call = __ trampoline_call(Address(addr, relocInfo::runtime_call_type), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      call = __ trampoline_call(Address(addr, rspec), &cbuf);
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }

      // Emit stub for static call
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    }
  
#line 13086 "ad_riscv.cpp"
  }
  {

#line 2002 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 13098 "ad_riscv.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 1992 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    int method_index = resolved_method_index(cbuf);
    address call = __ ic_call((address)opnd_array(1)->method(), method_index);
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 13119 "ad_riscv.cpp"
  }
  {

#line 2002 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 13131 "ad_riscv.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2010 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 13170 "ad_riscv.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2010 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 13209 "ad_riscv.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2010 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a jr (they
    // will be in a reachable segment) otherwise we have to use a jalr
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb != NULL) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full");
        return;
      }
    } else {
      Label retaddr;
      __ la(t1, retaddr);
      __ la(t0, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ addi(sp, sp, -2 * wordSize);
      __ sd(t1, Address(sp, wordSize));
      __ jalr(t0);
      __ bind(retaddr);
      __ addi(sp, sp, 2 * wordSize);
    }
  
#line 13248 "ad_riscv.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 1933 "src/hotspot/cpu/riscv/riscv.ad"

    Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
    Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
    Register temp_reg = as_Register(R12_enc);
    Register result_reg = as_Register(opnd_array(0)->reg(ra_,this)/* result */);
    Register cr_reg = t1;

    Label miss;
    Label done;
    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss);
    if ((0x1 /*primary()*/)) {
      __ mv(result_reg, zr);
    } else {
      __ mv(cr_reg, zr);
      __ j(done);
    }

    __ bind(miss);
    if (!(0x1 /*primary()*/)) {
      __ li(cr_reg, 1);
    }

    __ bind(done);
  
#line 13287 "ad_riscv.cpp"
  }
}

void partialSubtypeCheckVsZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1933 "src/hotspot/cpu/riscv/riscv.ad"

    Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
    Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
    Register temp_reg = as_Register(R12_enc);
    Register result_reg = as_Register(R15_enc);
    Register cr_reg = t1;

    Label miss;
    Label done;
    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss);
    if ((0x0 /*primary()*/)) {
      __ mv(result_reg, zr);
    } else {
      __ mv(cr_reg, zr);
      __ j(done);
    }

    __ bind(miss);
    if (!(0x0 /*primary()*/)) {
      __ li(cr_reg, 1);
    }

    __ bind(done);
  
#line 13327 "ad_riscv.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    MacroAssembler _masm(&cbuf);

#line 9774 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::UU);
  
#line 13350 "ad_riscv.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    MacroAssembler _masm(&cbuf);

#line 9792 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::LL);
  
#line 13372 "ad_riscv.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    MacroAssembler _masm(&cbuf);

#line 9809 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::UL);
  
#line 13394 "ad_riscv.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    MacroAssembler _masm(&cbuf);

#line 9827 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R28_enc), as_Register(R29_enc), as_Register(R30_enc),
                      StrIntrinsicNode::LU);
  
#line 13416 "ad_riscv.cpp"
  }
}

void string_indexofUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    MacroAssembler _masm(&cbuf);

#line 9846 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UU);
  
#line 13447 "ad_riscv.cpp"
  }
}

void string_indexofLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    MacroAssembler _masm(&cbuf);

#line 9867 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::LL);
  
#line 13478 "ad_riscv.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  unsigned idx10 = idx9 + opnd_array(9)->num_edges(); 	// tmp5
  unsigned idx11 = idx10 + opnd_array(10)->num_edges(); 	// tmp6
  {
    MacroAssembler _masm(&cbuf);

#line 9888 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                      opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                      opnd_array(10)->as_Register(ra_,this,idx10)/* tmp5 */, opnd_array(11)->as_Register(ra_,this,idx11)/* tmp6 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UL);
  
#line 13509 "ad_riscv.cpp"
  }
}

void string_indexof_conUUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 9910 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UU);
  
#line 13538 "ad_riscv.cpp"
  }
}

void string_indexof_conLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 9931 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::LL);
  
#line 13567 "ad_riscv.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// result
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp1
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp2
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp3
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 9952 "src/hotspot/cpu/riscv/riscv.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof_linearscan(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                                 opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                                 opnd_array(6)->as_Register(ra_,this,idx6)/* tmp1 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp2 */,
                                 opnd_array(8)->as_Register(ra_,this,idx8)/* tmp3 */, opnd_array(9)->as_Register(ra_,this,idx9)/* tmp4 */,
                                 icnt2, opnd_array(5)->as_Register(ra_,this,idx5)/* result */, StrIntrinsicNode::UL);
  
#line 13596 "ad_riscv.cpp"
  }
}

void stringU_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// result
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 9972 "src/hotspot/cpu/riscv/riscv.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */,
                           opnd_array(4)->as_Register(ra_,this,idx4)/* result */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                           opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */, false /* isU */);
  
#line 13621 "ad_riscv.cpp"
  }
}

void clearArray_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    MacroAssembler _masm(&cbuf);

#line 9990 "src/hotspot/cpu/riscv/riscv.ad"

    address tpc = __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
    if (tpc == NULL) {
      ciEnv::current()->record_failure("CodeCache is full");
      return;
    }
  
#line 13642 "ad_riscv.cpp"
  }
}

void clearArray_imm_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    MacroAssembler _masm(&cbuf);

#line 10010 "src/hotspot/cpu/riscv/riscv.ad"

    __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, (uint64_t)opnd_array(1)->constantL());
  
#line 13659 "ad_riscv.cpp"
  }
}

void string_equalsLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 10025 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 1);
  
#line 13679 "ad_riscv.cpp"
  }
}

void string_equalsUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 10041 "src/hotspot/cpu/riscv/riscv.ad"

    // Count is in 8-bit bytes; non-Compact chars are 16 bits.
    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, 2);
  
#line 13699 "ad_riscv.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 10058 "src/hotspot/cpu/riscv/riscv.ad"

    __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp4 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 1);
  
#line 13722 "ad_riscv.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 10075 "src/hotspot/cpu/riscv/riscv.ad"

    __ arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp4 */,
                     opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R28_enc), 2);
  
#line 13745 "ad_riscv.cpp"
  }
}

void safePointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10094 "src/hotspot/cpu/riscv/riscv.ad"

    __ read_polling_page(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* poll */), 0, relocInfo::poll_type);
  
#line 13761 "ad_riscv.cpp"
  }
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 2039 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register flag = t1;
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;

    assert_different_registers(oop, box, tmp, disp_hdr, t0);

    // Load markWord from object into displaced_header.
    __ ld(disp_hdr, Address(oop, oopDesc::mark_offset_in_bytes()));

    // Always do locking in runtime.
    if (EmitSync & 0x01) {
      __ mv(flag, 1);
      return;
    }

    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_enter(box, oop, disp_hdr, tmp, true, cont, /*slow_case*/NULL, NULL, flag);
    }

    // Check for existing monitor
    if ((EmitSync & 0x02) == 0) {
      __ andi(t0, disp_hdr, markOopDesc::monitor_value);
      __ bnez(t0, object_has_monitor);
    }

    // Set tmp to be (markWord of object | UNLOCK_VALUE).
    __ ori(tmp, disp_hdr, markOopDesc::unlocked_value);

    // Initialize the box. (Must happen before we update the object mark!)
    __ sd(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // Compare object markWord with an unlocked value (tmp) and if
    // equal exchange the stack address of our box with object markWord.
    // On failure disp_hdr contains the possibly locked markWord.
    __ cmpxchg(/*memory address*/oop, /*expected value*/tmp, /*new value*/box, Assembler::int64, Assembler::aq,
               Assembler::rl, /*result*/disp_hdr);
    __ mv(flag, zr);
    __ beq(disp_hdr, tmp, cont); // prepare zero flag and goto cont if we won the cas

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // If the compare-and-exchange succeeded, then we found an unlocked
    // object, will have now locked it will continue at label cont
    // We did not see an unlocked object so try the fast recursive case.

    // Check if the owner is self by comparing the value in the
    // markWord of object (disp_hdr) with the stack pointer.
    __ sub(disp_hdr, disp_hdr, sp);
    __ li(tmp, (intptr_t) (~(os::vm_page_size()-1) | (uintptr_t)markOopDesc::lock_mask_in_place));
    // If (mark & lock_mask) == 0 and mark - sp < page_size, we are stack-locking and goto cont,
    // hence we can store 0 as the displaced header in the box, which indicates that it is a
    // recursive lock.
    __ andr(tmp/*==0?*/, disp_hdr, tmp);
    __ sd(tmp/*==0, perhaps*/, Address(box, BasicLock::displaced_header_offset_in_bytes()));
    __ mv(flag, tmp); // we can use the value of tmp as the result here

    if ((EmitSync & 0x02) == 0) {
      __ j(cont);

      // Handle existing monitor.
      __ bind(object_has_monitor);
      // The object's monitor m is unlocked iff m->owner == NULL,
      // otherwise m->owner may contain a thread or a stack address.
      //
      // Try to CAS m->owner from NULL to current thread.
      __ add(tmp, disp_hdr, (ObjectMonitor::owner_offset_in_bytes() - markOopDesc::monitor_value));
      __ cmpxchg(/*memory address*/tmp, /*expected value*/zr, /*new value*/xthread, Assembler::int64, Assembler::aq,
                 Assembler::rl, /*result*/flag); // cas succeeds if flag == zr(expected)

      // Store a non-null value into the box to avoid looking like a re-entrant
      // lock. The fast-path monitor unlock code checks for
      // markOopDesc::monitor_value so use markOopDesc::unused_mark which has the
      // relevant bit set, and also matches ObjectSynchronizer::slow_enter.
      __ mv(tmp, (address)markOopDesc::unused_mark());
      __ sd(tmp, Address(box, BasicLock::displaced_header_offset_in_bytes()));
    }

    __ bind(cont);
  
#line 13874 "ad_riscv.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 2126 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Register flag = t1;
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp1 */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;

    assert_different_registers(oop, box, tmp, disp_hdr, flag);

    // Always do locking in runtime.
    if (EmitSync & 0x01) {
      __ mv(flag, 1);
      return;
    }

    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_exit(oop, tmp, cont, flag);
    }

    // Find the lock address and load the displaced header from the stack.
    __ ld(disp_hdr, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // If the displaced header is 0, we have a recursive unlock.
    __ mv(flag, disp_hdr);
    __ beqz(disp_hdr, cont);

    // Handle existing monitor.
    if ((EmitSync & 0x02) == 0) {
      __ ld(tmp, Address(oop, oopDesc::mark_offset_in_bytes()));
      __ andi(t0, tmp, markOopDesc::monitor_value);
      __ bnez(t0, object_has_monitor);
    }

    // Check if it is still a light weight lock, this is true if we
    // see the stack address of the basicLock in the markWord of the
    // object.

    __ cmpxchg(/*memory address*/oop, /*expected value*/box, /*new value*/disp_hdr, Assembler::int64, Assembler::relaxed,
               Assembler::rl, /*result*/tmp);
    __ xorr(flag, box, tmp); // box == tmp if cas succeeds
    __ j(cont);

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // Handle existing monitor.
    if ((EmitSync & 0x02) == 0) {
      __ bind(object_has_monitor);
      STATIC_ASSERT(markOopDesc::monitor_value <= INT_MAX);
      __ add(tmp, tmp, -(int)markOopDesc::monitor_value); // monitor
      __ ld(flag, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
      __ ld(disp_hdr, Address(tmp, ObjectMonitor::recursions_offset_in_bytes()));
      __ xorr(flag, flag, xthread); // Will be 0 if we are the owner.
      __ orr(flag, flag, disp_hdr); // Will be 0 if there are 0 recursions
      __ bnez(flag, cont);

      __ ld(flag, Address(tmp, ObjectMonitor::EntryList_offset_in_bytes()));
      __ ld(disp_hdr, Address(tmp, ObjectMonitor::cxq_offset_in_bytes()));
      __ orr(flag, flag, disp_hdr); // Will be 0 if both are 0.
      __ bnez(flag, cont);
      // need a release store here
      __ la(tmp, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
      __ membar(MacroAssembler::LoadStore | MacroAssembler::StoreStore);
      __ sd(zr, Address(tmp)); // set unowned
    }

    __ bind(cont);
  
#line 13959 "ad_riscv.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2231 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    __ jr(target_reg);
  
#line 13978 "ad_riscv.cpp"
  }
}

void TailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2238 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    // exception oop should be in x10
    // ret addr has been popped into ra
    // callee expects it in x13
    __ mv(x13, ra);
    __ jr(target_reg);
  
#line 14001 "ad_riscv.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2249 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    __ far_jump(RuntimeAddress(OptoRuntime::rethrow_stub()));
  
#line 14030 "ad_riscv.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2254 "src/hotspot/cpu/riscv/riscv.ad"

    MacroAssembler _masm(&cbuf);
    Assembler::CompressibleRegion cr(&_masm);
    __ ret();
  
#line 14047 "ad_riscv.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10232 "src/hotspot/cpu/riscv/riscv.ad"

    Assembler::CompressibleRegion cr(&_masm);
    if (is_reachable()) {
      __ halt();
    }
  
#line 14066 "ad_riscv.cpp"
  }
}

void convP2I_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 36 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14082 "ad_riscv.cpp"
  }
}

void convB2I_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 51 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ sext_b(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14100 "ad_riscv.cpp"
  }
}

void convI2S_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 66 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ sext_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14118 "ad_riscv.cpp"
  }
}

void convS2UI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 81 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14135 "ad_riscv.cpp"
  }
}

void convI2UL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 96 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ zext_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14152 "ad_riscv.cpp"
  }
}

void bytes_reverse_int_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 111 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_w_w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14168 "ad_riscv.cpp"
  }
}

void bytes_reverse_long_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 125 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ rev8(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14184 "ad_riscv.cpp"
  }
}

void bytes_reverse_unsigned_short_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 139 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_h_h_u(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14200 "ad_riscv.cpp"
  }
}

void bytes_reverse_short_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 153 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ revb_h_h(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14216 "ad_riscv.cpp"
  }
}

void shaddP_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 168 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 14238 "ad_riscv.cpp"
  }
}

void shaddP_reg_reg_ext_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 186 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 14260 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 205 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 14282 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 205 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             t0,
             opnd_array(2)->constant());
  
#line 14304 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_ext_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 223 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             t0,
             opnd_array(3)->constant());
  
#line 14326 "ad_riscv.cpp"
  }
}

void shaddL_reg_reg_ext_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 223 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ shadd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             t0,
             opnd_array(2)->constant());
  
#line 14348 "ad_riscv.cpp"
  }
}

void countLeadingZerosI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 242 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14364 "ad_riscv.cpp"
  }
}

void countLeadingZerosL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 256 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14380 "ad_riscv.cpp"
  }
}

void countTrailingZerosI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 270 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ ctzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14396 "ad_riscv.cpp"
  }
}

void countTrailingZerosL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 284 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ ctz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14412 "ad_riscv.cpp"
  }
}

void popCountI_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 299 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ cpopw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14428 "ad_riscv.cpp"
  }
}

void popCountL_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 314 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ cpop(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 14444 "ad_riscv.cpp"
  }
}

void minI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 329 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ min(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14461 "ad_riscv.cpp"
  }
}

void maxI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 343 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14478 "ad_riscv.cpp"
  }
}

void absI_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 361 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ negw(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
  
#line 14495 "ad_riscv.cpp"
  }
}

void absL_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 379 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ neg(t0, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ max(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), t0);
  
#line 14512 "ad_riscv.cpp"
  }
}

void andnI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 395 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14532 "ad_riscv.cpp"
  }
}

void andnI_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 395 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 14552 "ad_riscv.cpp"
  }
}

void andnL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 411 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14572 "ad_riscv.cpp"
  }
}

void andnL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 411 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ andn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 14592 "ad_riscv.cpp"
  }
}

void ornI_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 428 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14612 "ad_riscv.cpp"
  }
}

void ornI_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 428 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 14632 "ad_riscv.cpp"
  }
}

void ornL_reg_reg_bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 444 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14652 "ad_riscv.cpp"
  }
}

void ornL_reg_reg_b_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 444 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 14672 "ad_riscv.cpp"
  }
}

void convI2Bool_andI_reg_immIpowerOf2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 461 "src/hotspot/cpu/riscv/riscv_b.ad"

    __ bexti(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, exact_log2((juint)(opnd_array(2)->constant())));
  
#line 14689 "ad_riscv.cpp"
  }
}

const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0_orderedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangePAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapBAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapSAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* weakCompareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resAcqNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    true ,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    false,  // CMoveD
    false,  // CMoveVD
    false,  // CMoveF
    false,  // CMoveVF
    true ,  // CMoveI
    true ,  // CMoveL
    false,  // CMoveP
    false,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    true ,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    true ,  // CompareAndExchangeN
    false,  // GetAndAddB
    false,  // GetAndAddS
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    false,  // GetAndSetB
    false,  // GetAndSetS
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    false,  // EncodeISOArray
    true ,  // EncodeP
    true ,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    false,  // HasNegatives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    false,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // LoadBarrier
    false,  // LoadBarrierSlowReg
    false,  // LoadBarrierWeakSlowReg
    false,  // Lock
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachNullCheck
    false,  // MachProj
    true ,  // MaxD
    true ,  // MaxF
    true ,  // MaxI
    false,  // MaxL
    true ,  // MemBarAcquire
    true ,  // LoadFence
    false,  // SetVectMaskI
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinD
    true ,  // MinF
    true ,  // MinI
    false,  // MinL
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // Opaque2
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    false,  // OverflowAddI
    false,  // OverflowSubI
    false,  // OverflowMulI
    false,  // OverflowAddL
    false,  // OverflowSubL
    false,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    false,  // PopCountVI
    false,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    false,  // RoundDouble
    false,  // RoundDoubleMode
    false,  // RoundDoubleModeV
    false,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    false,  // CopySignD
    false,  // CopySignF
    false,  // SignumD
    false,  // SignumF
    true ,  // SqrtD
    true ,  // SqrtF
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StoreNKlass
    true ,  // StrComp
    false,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    false,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVS
    false,  // AddVI
    false,  // AddReductionVI
    false,  // AddVL
    false,  // AddReductionVL
    false,  // AddVF
    false,  // AddReductionVF
    false,  // AddVD
    false,  // AddReductionVD
    false,  // SubVB
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVB
    false,  // MulVS
    false,  // MulVI
    false,  // MulReductionVI
    false,  // MulVL
    false,  // MulReductionVL
    false,  // MulVF
    false,  // MulReductionVF
    false,  // MulVD
    false,  // MulReductionVD
    false,  // FmaVD
    false,  // FmaVF
    false,  // DivVF
    false,  // DivVD
    false,  // AbsVB
    false,  // AbsVS
    false,  // AbsVI
    false,  // AbsVL
    false,  // AbsVF
    false,  // AbsVD
    false,  // NegVF
    false,  // NegVD
    false,  // SqrtVD
    false,  // SqrtVF
    false,  // LShiftCntV
    false,  // RShiftCntV
    false,  // LShiftVB
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // LShiftVL
    false,  // RShiftVB
    false,  // RShiftVS
    false,  // RShiftVI
    false,  // RShiftVL
    false,  // URShiftVB
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // URShiftVL
    false,  // AndV
    false,  // OrV
    false,  // XorV
    false,  // MinV
    false,  // MaxV
    false,  // MinReductionV
    false,  // MaxReductionV
    false,  // LoadVector
    false,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    false,  // ReplicateB
    false,  // ReplicateS
    false,  // ReplicateI
    false,  // ReplicateL
    false,  // ReplicateF
    false,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false   // Whitespace
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 1 * VMRegImpl::slots_per_word; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::in_preserve_stack_slots() { return 2 * VMRegImpl::slots_per_word; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes / BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 2384 "src/hotspot/cpu/riscv/riscv.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 15219 "ad_riscv.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 2390 "src/hotspot/cpu/riscv/riscv.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, NULL, length);
  
#line 15230 "ad_riscv.cpp"

}

OptoRegPair Matcher::return_value(uint ideal_reg, bool is_outgoing) {

#line 2397 "src/hotspot/cpu/riscv/riscv.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R10_num,                           // Op_RegN
      R10_num,                           // Op_RegI
      R10_num,                           // Op_RegP
      F10_num,                           // Op_RegF
      F10_num,                           // Op_RegD
      R10_num                            // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R10_H_num,                         // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      F10_H_num,                         // Op_RegD
      R10_H_num                          // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 15265 "ad_riscv.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg, bool is_outgoing) {

#line 2397 "src/hotspot/cpu/riscv/riscv.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R10_num,                           // Op_RegN
      R10_num,                           // Op_RegI
      R10_num,                           // Op_RegP
      F10_num,                           // Op_RegF
      F10_num,                           // Op_RegD
      R10_num                            // Op_RegL
    };

    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                      // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R10_H_num,                         // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      F10_H_num,                         // Op_RegD
      R10_H_num                          // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 15300 "ad_riscv.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R31_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(R31_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(R8_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R2_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DRISCV64=1
#ifndef RISCV64
#  error "RISCV64 must be defined"
#endif // RISCV64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
